<?php
/**
 * Admin Menu
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Admin;

use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Modules\Importer\Module as ImportManagerModule;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Admin Menu class.
 */
class Admin_Menu {
	use Singleton;

	const MENU_SLUG = 'nebotheme';

	/**
	 * The class constructor
	 */
	private function __construct() {
		$this->add_hooks();
	}

	/**
	 * Add hooks.
	 */
	private function add_hooks() {
		add_action( 'admin_menu', [ $this, 'add_menu_to_dashboard' ] );
		add_action( 'admin_init', [ $this, 'activation_redirect' ] );
	}

	/**
	 * Redirect after activation.
	 */
	public function activation_redirect() {
		global $pagenow;

		$merlin = ImportManagerModule::instance()->get_merlin();

		if ( is_admin() && ! $merlin->is_completed() && isset( $_GET['activated'] ) && 'themes.php' === $pagenow ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			wp_safe_redirect( admin_url( 'admin.php?page=' . self::MENU_SLUG ) );
		}
	}

	/**
	 * Check current page.
	 */
	public static function is_page_dashboard() {
		return isset( $_GET['page'] ) && self::MENU_SLUG === $_GET['page']; // phpcs:ignore WordPress.Security.NonceVerification.Recommended
	}

	/**
	 * Add menu to dashboard.
	 */
	public function add_menu_to_dashboard() {
		add_submenu_page(
			self::MENU_SLUG,
			esc_html__( 'Theme Dashboard', 'yaffo' ),
			esc_html__( 'Theme Dashboard', 'yaffo' ),
			'manage_options',
			self::MENU_SLUG,
			null,
			0
		);

		add_menu_page(
			'Yaffo Theme',
			'NeboTheme',
			'manage_options',
			self::MENU_SLUG,
			[ $this, 'render_welcome' ],
			'dashicons-cloud',
			9999
		);
	}

	/**
	 * Echoes welcome page.
	 */
	public function render_welcome() {
		$merlin = ImportManagerModule::instance()->get_merlin();

		?>
		<div class="nebotheme-theme-panel">
			<h1>
				<?php echo esc_html( get_admin_page_title() ); ?>
			</h1>
			<hr>
			<div class="nebotheme-theme-panel__row">
				<div class="nebotheme-theme-panel__col">
					<?php if ( ! $merlin->is_completed() ) : ?>
						<h3>
							<i class="dashicons dashicons-admin-plugins"></i>
							<?php esc_html_e( 'Quick Setup', 'yaffo' ); ?>
						</h3>
						<p>
							<?php esc_html_e( 'Installing a child theme, activate all the necessary plugins, import content including widgets and theme options.', 'yaffo' ); ?>
						</p>
						<p>
							<a href="<?php echo esc_url( admin_url( "admin.php?page={$merlin->get_merlin_page_name()}" ) ); ?>" class="button button-primary button-large">
								<?php esc_html_e( 'Run Quick Setup', 'yaffo' ); ?>
							</a>
						</p>
					<?php else : ?>
						<h3>
							<i class="dashicons dashicons-admin-appearance"></i>
							<?php esc_html_e( 'Theme Options', 'yaffo' ); ?>
						</h3>
						<p>
							<?php esc_html_e( 'That\'s where you can customize the appearance of your theme and disable/enable specific theme features.', 'yaffo' ); ?>
						</p>
						<p>
							<a target="_blank" href="<?php echo esc_url( admin_url( '/customize.php' ) ); ?>" class="button button-large">
								<?php esc_html_e( 'Go to Customize', 'yaffo' ); ?>
							</a>
						</p>
					<?php endif; ?>
				</div>
				<div class="nebotheme-theme-panel__col">
					<h3>
						<i class="dashicons dashicons-book"></i>
						<?php esc_html_e( 'Documentation', 'yaffo' ); ?>
					</h3>
					<p>
						<?php esc_html_e( 'We have prepared a detailed guide to explore all aspects of the theme. Any of the answers may be included here.', 'yaffo' ); ?>
					</p>
					<p>
						<a href="<?php echo esc_url( nebotheme_get_url_docs_theme() ); ?>"
							target="_blank" class="button button-large">
							<?php esc_html_e( 'Read Documentation', 'yaffo' ); ?>
						</a>
					</p>
				</div>
				<div class="nebotheme-theme-panel__col">
					<h3>
						<i class="dashicons dashicons-sos"></i>
						<?php esc_html_e( 'Support Ticket', 'yaffo' ); ?>
					</h3>
					<p>
						<?php esc_html_e( 'We aim through our Service Center to give the finest customer experience. Upon activation of your theme, you may ask us for assistance at any time.', 'yaffo' ); ?>
					</p>
					<p>
						<a href="<?php echo esc_url( nebotheme_get_url_support() ); ?>"
							target="_blank" class="button button-large">
							<?php esc_html_e( 'Get Support', 'yaffo' ); ?>
						</a>
					</p>
				</div>
			</div>
		</div>
		<?php
	}
}
