<?php
/**
 * Admin
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Admin;

use NeboTheme\Yaffo\Admin\Admin_Menu;
use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Theme;
use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Utils;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Admin class.
 */
class Admin {

	const NONCE_USER = 'nebotheme_user_fields';

	use Singleton;

	/**
	 * Get data for nonce.
	 */
	private static function get_nonce_user_data() {
		return basename( __FILE__ );
	}

	/**
	 * The class constructor
	 */
	public function __construct() {
		Meta_Boxes::instance();
		Admin_Menu::instance();

		$this->add_hooks();
	}

	/**
	 * Register hooks.
	 */
	private function add_hooks() {
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );

		add_action( 'show_user_profile', [ $this, 'add_customer_meta_fields' ] );
		add_action( 'edit_user_profile', [ $this, 'add_customer_meta_fields' ] );

		add_action( 'personal_options_update', [ $this, 'save_customer_meta_fields' ] );
		add_action( 'edit_user_profile_update', [ $this, 'save_customer_meta_fields' ] );
	}

	/**
	 * Show Address Fields on edit user pages.
	 *
	 * @param WP_User $user
	 */
	public function add_customer_meta_fields( $user ) {
		if ( ! is_admin() || ! current_user_can( 'edit_user', $user->ID ) ) {
			return;
		}

		$sections = $this->get_customer_meta_sections();

		foreach ( $sections as $section_data ) {
			?>
			<div class="nebotheme-customer-fields">
				<input
					type="hidden"
					name="<?php echo esc_attr( self::NONCE_USER ); ?>"
					value="<?php echo esc_attr( wp_create_nonce( self::get_nonce_user_data() ) ); ?>"
				/>
				<h2><?php echo esc_html( $section_data['title'] ); ?></h2>
				<table class="form-table" role="presentation">
					<?php foreach ( $section_data['fields'] as $field_name => $field ) : ?>
						<tr>
							<th>
								<label for="<?php echo esc_attr( $field_name ); ?>"><?php echo esc_html( $field['label'] ); ?></label>
							</th>
							<td>
								<?php
								$value = get_user_meta( $user->ID, $field_name, true );
								$attrs = [
									'id' => $field_name,
									'name' => $field_name,
									'type' => Utils::get_if_isset( $field, 'type', 'text' ),
								];

								if ( 'checkbox' === $attrs['type'] ) {
									if ( $value ) {
										$attrs['checked'] = 'checked';
									}
								} else {
									$attrs['value'] = $value;
								}

								echo '<input ' . nebotheme_get_render_attrs( $attrs ) . '/>';

								if ( ! empty( $field['description'] ) ) :
									?>
									<p class="description"><?php echo nebotheme_get_kses_content( $field['description'] ); ?></p>
								<?php endif; ?>
							</td>
						</tr>
					<?php endforeach; ?>
				</table>
			</div>
			<?php
		}
	}

	/**
	 * Get customer meta fields.
	 *
	 * @return array
	 */
	public function get_customer_meta_sections() {
		$fields = [
			'additional_info' => [
				'title' => esc_html__( 'Additional Info', 'yaffo' ),
				'fields' => [
					'briefly' => [
						'label' => esc_html__( 'Briefly', 'yaffo' ),
						'description' => esc_html__( 'is used only in the widget', 'yaffo' ) . ': "' . esc_html__( 'NeboTheme - About Author', 'yaffo' ) . '"',
						'type' => 'text',
					],
				],
			],
			'social_media' => [
				'title' => esc_html__( 'Social Media', 'yaffo' ),
				'fields' => [],
			],
		];

		foreach ( nebotheme_get_user_socials() as $social_name => $social_data ) {
			$label = $social_data['label'] . ' ' . Utils::get_if_isset( $social_data, 'label_admin', '' );

			$fields['social_media']['fields'][ $social_name ] = [
				'label' => $label,
				'type' => 'url',
			];
		}

		$fields['social_media']['fields']['hide_email'] = [
			'label' => esc_html__( 'Hide Email For Visitors', 'yaffo' ),
			'type' => 'checkbox',
		];

		return $fields;
	}

	/**
	 * Save Address Fields on edit user pages.
	 *
	 * @param int $user_id User ID of the user being saved.
	 */
	public function save_customer_meta_fields( $user_id ) {
		if ( ! isset( $_POST[ self::NONCE_USER ] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST[ self::NONCE_USER ] ) ), self::get_nonce_user_data() ) ) {
			return;
		}

		if ( ! is_admin() || ! current_user_can( 'edit_user', $user_id ) ) {
			return;
		}

		foreach ( $this->get_customer_meta_sections() as $section_data ) {
			foreach ( $section_data['fields'] as $meta_key => $field ) {
				$meta_value = Utils::get_if_isset( $_POST, $meta_key, null );
				$is_meta_value = ! is_null( $meta_value );

				if ( isset( $field['type'] ) && 'checkbox' === $field['type'] ) {
					update_user_meta( $user_id, $meta_key, $is_meta_value );
				} elseif ( $is_meta_value ) {
					update_user_meta( $user_id, $meta_key, $this->get_sanitize_user_data( $meta_value ) );
				}
			}
		}
	}

	/**
	 * Clear data.
	 *
	 * Clean variables using sanitize_text_field. Arrays are cleaned recursively. Non-scalar values are ignored.
	 *
	 * @param string|array $data User ID of the user being saved.
	 */
	public function get_sanitize_user_data( $data ) {
		if ( is_array( $data ) ) {
			return array_map( __FUNCTION__, $data );
		} else {
			return is_scalar( $data ) ? sanitize_text_field( $data ) : $data;
		}
	}

	/**
	 * Add style & scripts.
	 */
	public function admin_enqueue_scripts() {
		$theme = wp_get_theme();
		$suffix = Theme::get_suffix_assets();
		$config = [
			'color_palettes' => nebotheme_get_color_palette(),
			'debug' => ( defined( 'WP_DEBUG' ) && WP_DEBUG ),
			'is_edit' => Utils::is_edit(),
			'l10n' => [],
			'nonce' => wp_create_nonce( 'nebotheme' ),
		];

		$config = apply_filters( 'nebotheme_config_admin', $config );

		wp_enqueue_script(
			'yaffo-admin',
			Theme::URI_JS . "/admin/script{$suffix}.js",
			[
				'jquery',
			],
			$theme->version,
			true
		);

		wp_localize_script( 'yaffo-admin', 'nebothemeAdminConfig', $config );
		wp_localize_script( 'yaffo-admin', 'nebothemeCommonConfig', Theme::get_config_common() );

		wp_enqueue_style( 'yaffo-admin', Theme::get_style_url( 'admin' ), [], $theme->version );

		if ( ! is_customize_preview() ) {
			wp_enqueue_style(
				'select2',
				Theme::URI_LIB . "/select2/css/select2{$suffix}.css",
				[],
				'4.0.13'
			);

			wp_enqueue_script(
				'select2',
				Theme::URI_LIB . "/select2/js/select2.full{$suffix}.js",
				[],
				'4.0.13',
				true
			);
		}
	}

	/**
	 * Get common data (etc frontend, admin, customize)
	 */
	public static function get_config_common() {
		return apply_filters( 'nebotheme_config_common_admin', [
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		] );
	}
}
