<?php
/**
 * Meta Boxes.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Admin;

use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Sidebars\Sidebar_Manager;
use NeboTheme\Yaffo\Admin\MetaBoxes\Controls\Base as BaseControl;
use NeboTheme\Yaffo\Theme;
use NeboTheme\Yaffo\Utils;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Meta Boxes.
 */
class Meta_Boxes {
	use Singleton;

	const NONCE_NAME = 'nebotheme_meta_boxes_nonce';
	const PM_VALUE_INHERIT = 'default';
	const PM_PREFIX = 'nebotheme_';
	const PM_SEPARATOR_VALUE = ',';
	const PM_EMPTY = 'empty';
	const PM_KEY_CSS = '_nebotheme_post_css';

	const CONTROL_COLOR = 'color';
	const CONTROL_HEADING = 'heading';
	const CONTROL_HIDDEN = 'hidden';
	const CONTROL_IMAGE = 'image';
	const CONTROL_NUMBER = 'number';
	const CONTROL_SELECT = 'select';
	const CONTROL_SLIDER = 'slider';
	const CONTROL_SWITCHER = 'switcher';
	const CONTROL_TEXT = 'text';

	/**
	 * Tabs.
	 *
	 * @var array
	 */
	private static $tabs = [];

	/**
	 * Fields.
	 *
	 * @var array
	 */
	private static $fields = [];

	/**
	 * The class constructor
	 */
	private function __construct() {
		$this->add_hooks();
	}

	/**
	 * Add hooks.
	 */
	private function add_hooks() {
		add_action( 'save_post', [ $this, 'delete_pm_css' ] );
		add_action( 'save_post', [ $this, 'save_post' ] );
		add_action( 'yaffo_core_meta_boxes_render', [ $this, 'render' ] );
		add_filter( 'nebotheme_export_skip_post_meta', [ $this, 'export_skip_post_meta' ], 10, 2 );
		add_filter( 'the_content', [ $this, 'add_css_in_content' ] );

		if ( Utils::is_edit() ) {
			add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );
			add_filter( 'nebotheme_config_admin', [ $this, 'config' ] );
			add_filter( 'admin_body_class', [ $this, 'admin_body_class' ] );
		}
	}

	/**
	 * Skip meta post for export.
	 */
	public function export_skip_post_meta( $meta_ids ) {
		$meta_ids[] = self::PM_KEY_CSS;

		return $meta_ids;
	}

	/**
	 * Add style.
	 */
	public function add_css_in_content( $content ) {
		if ( is_singular() ) {

			$css = $this->get_post_css();

			if ( $css ) {
				$content = '<style type="text/css">' .
					$css .
				'</style>' . $content;
			}
		}

		return $content;
	}

	/**
	 * Add style & script.
	 */
	public function admin_enqueue_scripts() {
		$suffix_assets = Theme::instance()::get_suffix_assets();

		wp_enqueue_script( 'wp-color-picker' );
		wp_enqueue_style( 'wp-color-picker' );

		/* Libs */
		wp_enqueue_style(
			'nouislider',
			Theme::URI_LIB . "/nouislider/nouislider{$suffix_assets}.css",
			[],
			'14.0.2'
		);

		wp_enqueue_script(
			'nouislider',
			Theme::URI_LIB . "/nouislider/nouislider{$suffix_assets}.js",
			[],
			'14.0.2',
			true
		);

		wp_enqueue_script(
			'wp-color-picker-alpha',
			Theme::URI_LIB . "/wp-color-picker-alpha/wp-color-picker-alpha{$suffix_assets}.js",
			[
				'wp-color-picker',
			],
			'2.1.3',
			true
		);

		$post_css = $this->get_generated_post_css();

		if ( $post_css ) {
			wp_register_style( 'yaffo-editor-pm', false, [], wp_get_theme()->get( 'Version' ) );
			wp_enqueue_style( 'yaffo-editor-pm' );
			wp_add_inline_style( 'yaffo-editor-pm', $post_css );
		}
	}

	/**
	 * Get fields with selector.
	 */
	public function get_fields_with_selectors() {
		return array_filter(
			$this->get_visible_fields(),
			function( $field ) {
				return isset( $field['selectors'] );
			},
			ARRAY_FILTER_USE_BOTH
		);
	}

	/**
	 * Get visible fields.
	 */
	public function get_visible_fields() {
		return array_filter( $this->get_post_type_fields(), [ __CLASS__, 'is_field_visible' ], ARRAY_FILTER_USE_BOTH );
	}

	/**
	 * Get Gutenberg Editor Container Custom Classes
	 */
	public function admin_body_class( $body_classes ) {
		$post = get_post();

		if ( $post ) {
			$sidebar_layout = $this->get_pm_for_frontend( 'sidebar_layout' );

			if (
				$sidebar_layout && ( ! nebotheme_is_core_active() && Sidebar_Manager::is_active() ) ||
				$sidebar_layout && nebotheme_is_core_active()
			) {
				$body_classes .= ' nebotheme--with-sidebar';
			}
		}

		return $body_classes;
	}

	/**
	 * Add data to meta-boxes page.
	 */
	public function config( $localize ) {
		$localize['meta_boxes'] = [
			'value_inherit' => self::PM_VALUE_INHERIT,
			'fields' => self::get_fields(),
			'pm_separator_value' => self::PM_SEPARATOR_VALUE,
			'pm_value_negative' => YAFFO_VALUE_NEGATIVE,
		];

		$localize['l10n']['select'] = esc_html__( 'Select', 'yaffo' );

		return $localize;
	}

	/**
	 * Save fields data to post meta.
	 */
	public function save_post( $post_id ) {
		/* < Verification */
		if (
			! isset( $_POST[ self::NONCE_NAME ] ) ||
			! wp_verify_nonce(
				sanitize_text_field( wp_unslash( $_POST[ self::NONCE_NAME ] ) ),
				self::get_nonce_data()
			)
		) {
			return;
		}

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( isset( $_POST['post_type'] ) ) {
			$post_type = sanitize_text_field( wp_unslash( $_POST['post_type'] ) );
		} else {
			return;
		}

		if ( 'page' === $post_type ) {
			if ( ! current_user_can( 'edit_page', $post_id ) ) {
				return;
			}
		} elseif ( ! current_user_can( 'edit_post', $post_id ) ) {
			return;
		}
		/* > Verification */

		foreach ( $this->get_post_type_fields() as $field_name => $field_args ) {
			$when_delete = null;
			$field_id = self::get_prefix( $field_name );
			$post_meta_new = Utils::get_if_isset( $_POST, $field_id, $when_delete );

			/* For select multiple empty */
			if (
				$when_delete === $post_meta_new &&
				self::CONTROL_SELECT === $field_args['type'] &&
				$this->get_control( $field_name, $field_args )->get_args( 'multiple' )
			) {
				$post_meta_new = '';
			}

			$post_meta_old = get_post_meta( $post_id, $field_id, true );
			$comparison_pm_values = [];

			if ( isset( $field_args['default'] ) && self::CONTROL_TEXT !== $field_args['type'] ) {
				$comparison_pm_values[] = $field_args['default'];
			} else {
				$comparison_pm_values[] = self::PM_VALUE_INHERIT;
			}

			if (
				$when_delete !== $post_meta_new &&
				! in_array( $post_meta_new, $comparison_pm_values, true )
			) {
				if ( is_numeric( $post_meta_new ) ) {
					$control = $this->get_control( $field_name, $field_args );

					switch ( $control->get_type() ) {
						case self::CONTROL_NUMBER:
						case self::CONTROL_SLIDER:
							$min = $control->get_args( 'min' );
							$max = $control->get_args( 'max' );

							if ( $min && $min > $post_meta_new ) {
								$post_meta_new = $min;
							} elseif ( $max && $max < $post_meta_new ) {
								$post_meta_new = $max;
							}

							break;
					}
				}

				if ( $post_meta_new !== $post_meta_old ) {
					update_post_meta( $post_id, $field_id, $post_meta_new );
				}
			} elseif ( $post_meta_new ) {
				delete_post_meta( $post_id, $field_id, $post_meta_old );
			}
		}
	}

	/**
	 * Remove css from post meta.
	 */
	public function delete_pm_css( $post_id ) {
		delete_post_meta( $post_id, self::PM_KEY_CSS );
	}

	/**
	 * Get class control.
	 */
	private static function get_control_class( $type ) {
		$class_name = __NAMESPACE__ . '\\MetaBoxes\\Controls\\' . ucfirst( $type );

		if ( ! class_exists( $class_name ) ) {
			return;
		}

		return $class_name;
	}

	/**
	 * Get class control.
	 *
	 * @return null|BaseControl
	 */
	private static function get_control( $name, $args = [] ) {
		if ( empty( $args ) ) {
			$args = self::get_field( $name );
		}

		if ( empty( $args ) ) {
			return false;
		}

		$control_class = self::get_control_class( $args['type'] );

		if ( empty( $control_class ) ) {
			return false;
		}

		return new $control_class( $name, $args );
	}

	/**
	 * Get data for nonce.
	 */
	private static function get_nonce_data() {
		return basename( __FILE__ );
	}

	/**
	 * Get style.
	 */
	public function get_post_css( $post = null ) {
		$post = get_post( $post );
		$css = get_post_meta( $post->ID, self::PM_KEY_CSS, true );

		if ( ! $css && self::PM_EMPTY !== $css ) {
			$generate_css = $this->get_generated_post_css();
			$css = $generate_css ? $generate_css : self::PM_EMPTY;
		}

		if ( ! $css ) {
			update_post_meta( $post->ID, self::PM_KEY_CSS, self::PM_EMPTY );
		}

		return self::PM_EMPTY !== $css ? $css : false;
	}

	/**
	 * Generate css by meta boxes.
	 */
	private function get_generated_post_css() {
		$css = '';
		$fields_for_css = $this->get_fields_with_selectors();
		$post = get_post();
		$selectors_stack = [];

		// < Filter
		foreach ( $fields_for_css as $field_name => $field ) {
			$pm_value_css = self::get_pm_for_render( $field_name );

			if ( ! $pm_value_css && '0' !== $pm_value_css ) {
				continue;
			}

			foreach ( $field['selectors'] as $selector => $properties_pattern ) {
				$properties = str_replace( [ '{{VALUE}}' ], [ $pm_value_css ], $properties_pattern );

				$selectors_stack[ $selector ][] = $properties;
			}
		}

		// > Filter
		foreach ( $selectors_stack as $selector => $properties ) {
			$css .= ".nebotheme-var-block[data-id=\"{$post->ID}\"] {$selector}" . '{' . join( '', $properties ) . '}';
		}

		return $css;
	}

	/**
	 * Echoes meta boxes
	 */
	public function render() {
		?>
		<!-- Nonce -->
		<input
			type="hidden"
			name="<?php echo esc_attr( self::NONCE_NAME ); ?>"
			value="<?php echo esc_attr( wp_create_nonce( self::get_nonce_data() ) ); ?>"
		/>
		<div class="nebotheme-admin-reset" data-nebotheme-admin-reset>
			<div id="nebotheme-meta-boxes" class="nebotheme-meta-boxes">
				<div class="nebotheme-meta-boxes-tabs">
					<?php $this->render_tabs(); ?>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Echoes each tab.
	 */
	public function render_tabs() {
		$tabs = $this->get_tabs_allow();
		$fields_allow = $this->get_post_type_fields();
		/* Tab Title */
		?>
		<div class="tabs-triggers">
			<div class="tabs-triggers-inner">
				<?php
				foreach ( $tabs as $tab_id => $tab ) {
					?>
					<a href="#" data-tab-id="<?php echo esc_attr( $tab_id ); ?>">
						<?php echo esc_html( $tab['label'] ); ?>
					</a>
					<?php
				}
				?>
			</div>
		</div>
		<div class="tabs-content">
			<ul>
				<?php
				foreach ( $tabs as $tab_id => $tab ) {
					$tab_attr = [
						'data-tab-id' => $tab_id,
					];
					?>
					<li <?php nebotheme_render_attrs( $tab_attr ); ?>>
						<?php
						echo '<h3 class="tab-title">' .
							esc_html( $tab['label'] ) .
						'</h3>';

						foreach ( $fields_allow as $field_name => $field_args ) {
							if ( $field_args['tab'] !== $tab_id ) {
								continue;
							}

							$control = self::get_control( $field_name );

							if ( ! $control ) {
								return;
							}

							$control->render();
						}
						?>
					</li>
					<?php
				}
				?>
			</ul>
		</div>
		<?php
	}

	/**
	 * Get allowed fields.
	 */
	public function get_post_type_fields() {
		$fields = self::get_fields();

		return array_filter( $fields, [ $this, 'post_type_array_filter' ], ARRAY_FILTER_USE_BOTH );
	}

	/**
	 * Filter array by post type.
	 */
	public function post_type_array_filter( $item ) {
		$post = get_post();

		if ( isset( $item['post_type'] ) ) {
			return in_array( $post->post_type, $item['post_type'], true );
		}

		return true;
	}

	/**
	 * Get allowed tabs.
	 */
	public function get_tabs_allow() {
		$tabs = $this->get_tabs();

		return array_filter( $tabs, [ $this, 'post_type_array_filter' ], ARRAY_FILTER_USE_BOTH );
	}

	/**
	 * Get fields by name.
	 */
	public static function get_field( $field_name ) {
		return Utils::get_if_isset( self::get_fields(), $field_name );
	}

	/**
	 * Get all fields.
	 */
	public static function get_fields() {
		if ( static::$fields ) {
			return static::$fields;
		}

		static::$fields = apply_filters( 'nebotheme_meta_boxes_fields', static::$fields );

		foreach ( static::$fields as $field_name => &$field_args ) {
			$control = self::get_control( $field_name, $field_args );

			if ( $control ) {
				$field_args = $control->get_args();
			}
		}

		return static::$fields;
	}

	/**
	 * Condition from Kirki to Meta Boxes.
	 */
	public static function conversion_condition( array $active_callback ) {
		$condition = [];

		foreach ( $active_callback as $item ) {
			$is_negative = false;
			$value = $item['value'];

			if ( isset( $item['operator'] ) ) {
				$is_negative = (bool) preg_match( '/!/', $item['operator'] );
			}

			if ( ! $value ) {
				$value = [ '', 'false', false, YAFFO_VALUE_NEGATIVE ];
			}

			$condition[ $item['setting'] . ( $is_negative ? '!' : '' ) ] = $value;
		}

		return $condition;
	}

	/**
	 * Get all tabs.
	 */
	public function get_tabs() {
		if ( self::$tabs ) {
			return self::$tabs;
		}

		self::$tabs = apply_filters( 'nebotheme_meta_boxes_tabs', self::$tabs );

		return self::$tabs;
	}

	/**
	 * Get Post Meta for render field.
	 */
	public static function get_pm_for_render( $field_name, $post = null ) {
		$control = self::get_control( $field_name );

		if ( ! $control || ! self::is_field_visible( $control->get_args() ) ) {
			return false;
		}

		return $control->get_value_for_render( $post );
	}

	/**
	 * Get Post Meta for frontend.
	 */
	public static function get_pm_for_frontend( $field_name, $post = null ) {
		$control = self::get_control( $field_name );

		if ( ! $control || ! self::is_field_visible( $control->get_args() ) ) {
			return false;
		}

		return $control->get_value_for_frontend( $post );
	}

	/**
	 * Get Post Meta.
	 */
	public static function get_pm( $field_name, $post = null ) {
		$post = get_post( $post );

		if ( ! $post ) {
			return;
		}

		$pm_id = self::get_prefix( $field_name );
		$pm = get_post_meta( $post->ID, $pm_id, true );

		return $pm;
	}

	/**
	 * Get Post Meta with default data and theme-mod.
	 */
	public static function get_pm_safe( $field_name, $post = null ) {
		$post = get_post( $post );

		if (
			! $field_name ||
			(
				! is_singular() &&
				! $post
			)
		) {
			return null;
		}

		$pm = self::get_pm( $field_name, $post );

		if ( self::is_pm_inherit( $pm ) ) {
			$field = self::get_field( $field_name );

			if ( $field ) {
				$theme_mod_name = Utils::get_if_isset( $field, 'theme_mod_name' );

				if ( $theme_mod_name ) {
					$pm = nebotheme_get_tm( $theme_mod_name );
				} elseif (
					isset( $field['default'] ) &&
					self::PM_VALUE_INHERIT !== $field['default']
				) {
					$pm = $field['default'];
				}
			}
		}

		return nebotheme_get_transformed_value( $pm );
	}

	/**
	 * Get Post Meta width default.
	 */
	public static function get_pm_default( $field_name, $return_fail = null ) {
		$field = self::get_field( $field_name );

		if ( $field && isset( $field['default'] ) ) {
			return $field['default'];
		}

		return $return_fail;
	}

	/**
	 * Check field condition.
	 */
	public static function is_field_visible( $field ) {
		if ( empty( $field['condition'] ) ) {
			return true;
		}

		foreach ( $field['condition'] as $condition_key => $condition_value ) {
			if ( YAFFO_VALUE_NEGATIVE === $condition_value ) {
				$condition_value = [ $condition_value, '', false, null ];
			}

			preg_match( '/([a-z_\-0-9]+)(?:\[([a-z_]+)])?(!?)$/i', $condition_key, $condition_key_parts );

			$field_name = $condition_key_parts[1];
			$is_negative = ! ! $condition_key_parts[3];
			$pm = nebotheme_get_transformed_value( self::get_pm_safe( $field_name ), true );

			if ( is_array( $condition_value ) && ! empty( $condition_value ) ) {
				$is_contains = in_array( $pm, $condition_value, true );
			} elseif ( is_array( $pm ) && ! empty( $pm ) ) {
				$is_contains = in_array( $condition_value, $pm, true );
			} else {
				$is_contains = ( $pm === $condition_value );
			}

			if ( $is_negative && $is_contains || ! $is_negative && ! $is_contains ) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Check if need get default or theme-mod
	 */
	public static function is_pm_inherit( $pm ) {
		return in_array( $pm, [ self::PM_VALUE_INHERIT, '', null ], true );
	}

	/**
	 * Get name with prefix.
	 */
	public static function get_prefix( $field_name ) {
		return self::PM_PREFIX . $field_name;
	}
}
