<?php
/**
 * Base Class control
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Admin\MetaBoxes\Controls;

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Utils;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Metabox base class
 */
abstract class Base {
	/**
	 * Has reset button
	 *
	 * @var bool
	 */
	protected $has_reset_button = false;

	/**
	 * Name with prefix
	 *
	 * @var string
	 */
	protected $id = null;

	/**
	 * Control Name
	 *
	 * @var string
	 */
	protected $name = null;

	/**
	 * Control args
	 *
	 * @var array
	 */
	protected $args = null;

	/**
	 * Control value
	 *
	 * @var mixed
	 */
	protected $value = null;

	/**
	 * The class constructor
	 *
	 * @param string $name
	 * @param array $args
	 */
	public function __construct( $name, array $args = [] ) {
		$this->name = $name;
		$this->args = $args;
		$this->id = Meta_Boxes::get_prefix( $name );

		if ( empty( $this->args['ensured'] ) ) {
			$this->ensure_args();
		}

		$this->ensure_value();
	}

	/**
	 * Get type control.
	 */
	abstract public function get_type();

	/**
	 * Retrieve args
	 */
	public function get_args( $argument = null, $fail = false ) {
		if ( $argument ) {
			return Utils::get_if_isset( $this->args, $argument, $fail );
		}

		return $this->args;
	}

	/**
	 * Is has 'theme_mod_name' in args
	 */
	public function has_theme_mod_name() {
		return ! empty( $this->args['theme_mod_name'] );
	}

	/**
	 * Get value for frontend
	 */
	public function get_value_for_frontend( $post = null ) {
		return Meta_Boxes::get_pm_safe( $this->name, $post );
	}

	/**
	 * Get value for render
	 */
	public function get_value_for_render( $post = null ) {
		return $this->get_value_for_frontend( $post );
	}

	/**
	 * Get value theme mod
	 */
	public function get_theme_mod() {
		if ( ! $this->has_theme_mod_name() ) {
			return null;
		}

		$theme_mod = nebotheme_get_tm( $this->args['theme_mod_name'] );

		return $theme_mod;
	}

	/**
	 * Check has default argument.
	 */
	public function has_default() {
		return ! empty( $this->args['default'] );
	}

	/**
	 * Echoes control.
	 */
	final public function render() {
		?>
		<div class="nebotheme-meta-boxes-field-section" data-name="<?php echo esc_attr( $this->name ); ?>" data-type="<?php echo esc_attr( $this->get_type() ); ?>">
			<div class="nebotheme-meta-boxes-field-section-inner">
				<?php if ( ! empty( $this->args['label'] ) ) : ?>
					<label class="nebotheme-meta-boxes-field-label" for="<?php echo esc_attr( $this->id ); ?>"><?php echo nebotheme_unsanitize( $this->args['label'] ); ?></label>
				<?php endif; ?>
				<div class="nebotheme-meta-boxes-field-content">
					<div <?php nebotheme_render_attrs( $this->get_wrap_attrs() ); ?>>
						<div class="nebotheme-meta-boxes-field-inner">
							<?php $this->render_content(); ?>
							<?php $this->render_field(); ?>
						</div>
						<?php $this->render_reset_button(); ?>
					</div>
					<?php if ( ! empty( $this->args['description'] ) ) : ?>
						<div class="nebotheme-meta-boxes-field-description">
							<small><?php echo nebotheme_get_kses_content( $this->args['description'] ); ?></small>
						</div>
					<?php endif; ?>
				</div>
			</div>
		</div>
		<?php
	}

	/**
	 * Prepares args control
	 */
	protected function ensure_args() {
		$this->args = array_merge( $this->get_args_default(), $this->args );

		$this->args['ensured'] = true;
	}

	/**
	 * Retrieve args default
	 */
	protected function get_args_default() {
		return [
			'ensured' => false,
		];
	}

	/**
	 * Prepares value
	 */
	protected function ensure_value() {
		if ( null !== $this->value ) {
			return;
		}

		$value = get_post_meta( get_the_ID(), $this->id, true );

		if (
			empty( $value ) &&
			$this->has_default()
		) {
			$value = $this->args['default'];
		}

		$this->value = $value;

		return $this->value;
	}

	/**
	 * Echoes additional content.
	 */
	protected function render_content() {}

	/**
	 * Echoes attributes for field.
	 */
	protected function render_attrs() {
		nebotheme_render_attrs( $this->get_control_attrs() );
	}

	/**
	 * Get attributes as array.
	 */
	protected function get_control_attrs() {
		$attrs = [
			'id' => $this->id,
			'name' => $this->id,
			'value' => $this->value,
			'data-theme-mod' => nebotheme_get_transformed_value( $this->get_theme_mod(), true ),
		];

		return $attrs;
	}

	/**
	 * Echoes field.
	 */
	protected function render_field() {
		?>
		<input <?php $this->render_attrs(); ?>>
		<?php
	}

	/**
	 * Get attributes for wrapper as array.
	 */
	protected function get_wrap_attrs() {
		return [
			'class' => [
				'nebotheme-meta-boxes-field',
				'nebotheme-meta-boxes-field-' . $this->get_type(),
			],
			'data-name' => $this->name,
		];
	}

	/**
	 * Echoes reset value button
	 */
	private function render_reset_button() {
		if ( ! $this->has_reset_button ) {
			return;
		}

		$attrs = [
			'title' => esc_html__( 'Discard Changes', 'yaffo' ),
			'href' => '#',
			'class' => [
				'nebotheme-meta-boxes-default-trigger',
				'dashicons-image-rotate',
				'dashicons',
			],
		];

		echo '<a ' . nebotheme_get_render_attrs( $attrs ) . '></a>';
	}
}
