<?php
/**
 * Blog organizer
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo;

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Homepage;
use NeboTheme\Yaffo\Classes\WP_Query_Agent;
use NeboTheme\Yaffo\Utils;

use NeboTheme\Yaffo\Traits\Singleton;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Blog class.
 */
class Blog_Main {
	const CALLBACKS_LOCATIONS  = [
		'is_home',
		'is_page',
	];
	const AVAILABLE_QUERY_VARS = [
		'tax_query',
		'paged',
	];

	use Singleton;

	/**
	 * The WordPress Query class.
	 *
	 * @var \WP_Query
	 */
	private static $wp_query;

	/**
	 * The Query vars.
	 *
	 * @var array
	 */
	private static $default_query_vars = [];

	/**
	 * Status blog.
	 *
	 * @var bool
	 */
	private static $is_active;

	/**
	 * If on ajax.
	 *
	 * @var bool
	 */
	private static $is_ajax;

	/**
	 * Layout blog.
	 *
	 * @var string
	 */
	private static $current_layout = '';

	/**
	 * The class constructor
	 */
	public function __construct( $_wp_query = null ) {
		add_action( 'wp_ajax_nopriv_nebotheme_ajax_blog', [ $this, 'ajax_blog' ] );
		add_action( 'wp_ajax_nebotheme_ajax_blog', [ $this, 'ajax_blog' ] );

		if ( ! Utils::is_ajax() ) {
			add_action( 'pre_get_posts', [ $this, 'set_main_query_var' ] );
		}

		add_filter( 'nebotheme_config_frontend', function( $config ) {
			$current_location = false;

			foreach ( self::CALLBACKS_LOCATIONS as $callback_location ) {
				if ( is_callable( $callback_location ) ) {
					if ( call_user_func( $callback_location ) ) {
						$current_location = (string) $callback_location;

						break;
					}
				}
			}

			if ( $current_location ) {
				$config['ajax_blog'] = [
					'location'     => $current_location,
					'pagenum_link' => get_pagenum_link(),
				];

				if ( 'is_page' === $current_location ) {
					$config['ajax_blog']['page_id'] = get_post()->ID;
				}
			}

			return $config;
		} );
	}

	/**
	 * Set main query vars.
	 */
	public function set_main_query_var( $wp_query ) {
		if ( $wp_query->is_main_query() && $wp_query->is_home() ) {
			$wp_query->query_vars += self::get_default_query_vars();
		}
	}

	/**
	 * Render search result on ajax.
	 */
	public function ajax_blog() {
		if ( empty( $_POST['query_vars'] ) ) {
			wp_die( '', '', [ 'response' => 400 ] );
		}

		self::$is_ajax = true;

		$location = isset( $_POST['location'] ) ? sanitize_text_field( wp_unslash( $_POST['location'] ) ) : false;

		if ( ! $location || ! in_array( $location, self::CALLBACKS_LOCATIONS, true ) ) {
			wp_send_json_error( 'Bad location', 400 );
		}

		if ( 'is_page' === $location ) {
			$page_id = isset( $_POST['page_id'] ) ? sanitize_text_field( wp_unslash( $_POST['page_id'] ) ) : false;
			$page    = get_post( $page_id );

			if ( ! $page || ( 'publish' !== get_post_status( $page ) && ! current_user_can( 'edit_published_posts', $page ) ) ) {
				wp_send_json_error( 'Bad page', 400 );
			}

			setup_postdata( $GLOBALS['post'] =& $page ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited, Squiz.PHP.DisallowMultipleAssignments.Found
		}

		$query_vars = isset( $_POST['query_vars'] ) ? wp_unslash( $_POST['query_vars'] ) : []; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$query_vars = array_intersect_key( $query_vars, array_flip( self::AVAILABLE_QUERY_VARS ) );

		$query_vars['ignore_sticky_posts'] = true;
		$query_vars['post_status']         = 'publish';

		WP_Query_Agent::loop_start();

		if ( 'is_page' === $location ) {
			self::$default_query_vars += $query_vars;

			wp();

			self::init_query();
		} else {
			add_action( 'pre_get_posts', function( $wp_query ) use ( $location ) {
				if ( $wp_query->is_main_query() ) {
					$wp_query->{$location} = true;
					$wp_query->query_vars  = self::get_default_query_vars();

					$this->init_blog();
				}
			} );

			self::$default_query_vars += $query_vars;

			wp( $query_vars );

			self::init_query();
		}

		if ( ! self::$wp_query->have_posts() ) {
			wp_send_json_error( 'No have posts', 400 );

			return;
		}

		$blog_html = Utils::get_ob_html( function() {
			$this->render_posts();
			$this->render_pagination();
		} );

		WP_Query_Agent::loop_end();

		self::$is_ajax = false;

		wp_send_json_success( $blog_html );
	}

	/**
	 * Init WP Query.
	 */
	public static function init_query() {
		global $wp_query;

		if ( self::$wp_query instanceof \WP_Query ) {
			return;
		}

		if ( Homepage::is_blog_on_page_enable() ) {
			self::$wp_query = new \WP_Query( self::get_default_query_vars() );
		} else {
			self::$default_query_vars['paged'] = $wp_query->get( 'paged' );
			self::$wp_query                    = $wp_query;
		}
	}

	/**
	 * Get current layout.
	 */
	public static function get_layout() {
		if ( is_archive() ) {
			$layout = nebotheme_get_tm( 'archive_blog_layout' );
		} elseif ( is_search() ) {
			$layout = nebotheme_get_tm( 'search_blog_layout' );
		} elseif ( is_home() || is_page() ) {
			$layout = Homepage::get_settings( 'homepage_blog_layout' );
		}

		if ( empty( $layout ) ) {
			$layout = 'list';
		}

		return $layout;
	}

	/**
	 * Get query vars for blog.
	 *
	 * @return  array
	 */
	public static function get_default_query_vars() {
		static $init_default_query_vars = false;

		if ( ! $init_default_query_vars ) {
			if ( Homepage::is_enable_slider_settings() || ! Homepage::is_first_page() ) {
				if ( Homepage::is_enable_slider_settings() && ! Homepage::is_products_slider() ) {
					$slider_posts = Homepage::get_wp_query_slider()->posts;

					if ( $slider_posts ) {
						self::$default_query_vars['post__not_in'] = nebotheme_wp_list_pluck( $slider_posts, 'ID' );
					}
				}
			}

			if ( Homepage::is_blog_on_page_enable() ) {
				$category_ids = Meta_Boxes::get_pm( 'homepage_blog_category' );
				$tags_ids     = Meta_Boxes::get_pm( 'homepage_blog_tag' );

				self::$default_query_vars['posts_per_page'] = Meta_Boxes::get_pm_safe( 'homepage_blog_posts_per_page' );

				if ( empty( self::$default_query_vars['tax_query'] ) ) {
					if ( $category_ids && $tags_ids ) {
						self::$default_query_vars['tax_query'] = [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_tax_query
						'relation' => strtoupper( Meta_Boxes::get_pm_safe( 'homepage_blog_relation' ) ),
						];

						if ( $category_ids ) {
							self::$default_query_vars['tax_query'][] = [
								'field'            => 'term_id',
								'include_children' => true,
								'operator'         => 'IN',
								'taxonomy'         => 'category',
								'terms'            => $category_ids,
							];
						}

						if ( $tags_ids ) {
							self::$default_query_vars['tax_query'][] = [
								'field'    => 'term_id',
								'operator' => 'IN',
								'taxonomy' => 'post_tag',
								'terms'    => $tags_ids,
							];
						}
					} elseif ( $tags_ids ) {
						self::$default_query_vars['tag__in'] = $tags_ids;
					} elseif ( $category_ids ) {
						self::$default_query_vars['category__in'] = $category_ids;
					}
				}
			}

			$init_default_query_vars = true;
		}

		return self::$default_query_vars;
	}

	/**
	 * Init Blog.
	 */
	public function init_blog() {
		nebotheme_is_masonry();
	}

	/**
	 * Echoes posts.
	 */
	public function render_start_posts( $template_name ) {
		self::$current_layout = $template_name;

		$attrs = [
			'class' => [
				'nebotheme-blog__posts',
			],
			'data-layout' => $template_name,
		];

		if ( 'grid' === $template_name && nebotheme_is_masonry() ) {
			$attrs['class'][] = 'nebotheme-blog-grid--masonry';
		}

		echo '<div ' . nebotheme_get_render_attrs( $attrs ) . '>' .
			'<div class="nebotheme-blog__posts__inner">';
	}

	/**
	 * Echoes blog.
	 */
	public function render() {
		$this->init_blog();
		self::init_query();

		if ( ! self::$wp_query->have_posts() ) {
			return;
		}

		do_action( 'nebotheme_render_blog_before' );

		WP_Query_Agent::loop_start();

		$classes = [ 'nebotheme-blog' ];

		if ( Homepage::get_settings( 'homepage_blog_loadmore' ) ) {
			$classes[] = 'nebotheme-blog--loadmore';
		}

		?>
		<div class="<?php echo esc_attr( join( ' ', $classes ) ); ?>">
			<?php $this->render_blog_variable(); ?>
		</div>
		<?php

		WP_Query_Agent::loop_end();

		self::$is_active      = false;
		self::$current_layout = '';

		do_action( 'nebotheme_render_blog_after' );
	}

	/**
	 * Echoes blog variation.
	 */
	private function render_blog_variable() {
		echo '<div class="nebotheme-blog__variable">';

		$this->render_posts();
		$this->render_pagination();

		echo '</div>';
	}

	/**
	 * Echoes blog posts.
	 */
	private function render_posts() {
		$blog_layout        = static::get_layout();
		$blog_layouts_array = explode( '/', $blog_layout );

		if ( ! $blog_layouts_array ) {
			return;
		}

		if ( self::is_ajax() && 1 < count( $blog_layouts_array ) ) {
			$blog_layouts_array = [ $blog_layouts_array[1] ];
		}

		$template_name     = $blog_layouts_array[0];
		$template_name_old = $template_name;

		$this->render_start_posts( $template_name );

		self::$is_active      = true;
		self::$current_layout = $template_name;

		while ( self::$wp_query->have_posts() ) {
			self::$wp_query->the_post();

			$template_name_old = $template_name;
			$template_name     = Utils::get_if_isset(
				$blog_layouts_array,
				self::$wp_query->current_post,
				$template_name
			);

			if ( $template_name_old !== $template_name ) {
				echo '</div>';
				echo '</div>';
				$this->render_start_posts( $template_name );
			}

			get_template_part( "template-parts/blog/post-{$template_name}" );
		}

		echo '</div></div>';
	}

	/**
	 * Get current page from query vars.
	 */
	public static function get_paged() {
		if ( self::is_ajax() ) {
			$paged = Utils::get_if_isset( self::$default_query_vars, 'paged', 1 );
		} else {
			$paged = get_query_var( 'paged' );
		}

		return max( 1, $paged );
	}

	/**
	 * Pagination arguments.
	 */
	public static function get_public_pagination_args() {
		return [
			'type' => 'list',
			'prev_text' => '<span class="text">' . esc_html__( 'Previous', 'yaffo' ) . '</span>',
			'next_text' => '<span class="text">' . esc_html__( 'Next', 'yaffo' ) . '</span>',
		];
	}

	/**
	 * Pagination arguments.
	 */
	protected function get_pagination_args() {
		global $wp_rewrite;

		$args = [
			'current' => self::get_paged(),
			'total'   => self::$wp_query->max_num_pages,
		];

		$args = array_merge( $args, static::get_public_pagination_args() );

		if ( ( is_home() || is_page() ) && Homepage::get_settings( 'homepage_blog_loadmore' ) ) {
			$args['next_text'] = nebotheme_get_tm_fallback( 'homepage_blog_loadmore_text' );
		}

		if ( self::is_ajax() && ! empty( $_POST['pagenum_link'] ) ) {
			$pagenum_link = html_entity_decode( sanitize_text_field( wp_unslash( $_POST['pagenum_link'] ) ) );
			$url_parts    = explode( '?', $pagenum_link );
			$pagenum_link = trailingslashit( $url_parts[0] ) . '%_%';

			$format  = $wp_rewrite->using_index_permalinks() && ! strpos( $pagenum_link, 'index.php' ) ? 'index.php/' : '';
			$format .= $wp_rewrite->using_permalinks() ? user_trailingslashit( $wp_rewrite->pagination_base . '/%#%', 'paged' ) : '?paged=%#%';

			$pagenum_link = preg_replace( '/\/page\/(\d)*/', '', $pagenum_link );

			$args['base']   = $pagenum_link;
			$args['format'] = $format;
		}

		return $args;
	}

	/**
	 * Echoes pagination for blog.
	 */
	private function render_pagination() {
		if ( self::$wp_query->max_num_pages <= 1 ) {
			return;
		}

		?>
		<div class="nebotheme-pagination">
			<?php echo paginate_links( $this->get_pagination_args() ); ?>
		</div>
		<?php
	}

	/**
	 * Check blog status
	 *
	 * @return bool
	 */
	public static function is_active() {
		return self::$is_active;
	}

	/**
	 * Check blog status
	 *
	 * @return bool
	 */
	public static function is_ajax() {
		return self::$is_ajax;
	}

	/**
	 * Get blog current layout.
	 *
	 * @return bool
	 */
	public static function get_current_layout() {
		return self::$current_layout;
	}
}
