<?php
/**
 * Headline class.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo;

use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Modules\Seo\Module as SeoModule;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Headline class.
 */
final class Headline {
	use Singleton;

	/**
	 * The class constructor
	 */
	protected function __construct() {
		add_action( 'nebotheme_render_main_start', [ $this, 'render' ] );

		add_filter( 'body_class', [ $this, 'body_class' ] );
		add_filter( 'get_the_archive_title', [ $this, 'get_the_archive_title' ] );
	}

	/**
	 * Archive Title
	 *
	 * Removes default prefixes, like "Category:" from archive titles.
	 *
	 * @param string $title Archive title.
	 */
	public function get_the_archive_title( $title ) {
		if ( is_category() ) {
			$title = single_cat_title( '', false );
		} elseif ( is_tag() ) {
			$title = single_tag_title( '', false );
		} elseif ( is_author() ) {
			$title = get_the_author();
		}

		return $title;
	}

	/**
	 * Filter for body classes.
	 */
	public function body_class( array $body_classes ) {
		if ( static::is_active() ) {
			$body_classes[] = 'nebotheme--headline';
		}

		return $body_classes;
	}

	/**
	 * Check if headline is active.
	 */
	public static function is_active() {
		$is_active = true;

		if ( is_home() || is_single() || ( is_author() && nebotheme_is_render_author_box() ) ) {
			$is_active = false;
		} elseif ( is_page() ) {
			$is_active = Meta_Boxes::get_pm_safe( 'headline_show' );
		}

		return apply_filters( 'nebotheme_headline_is_active', $is_active );
	}

	/**
	 * Echoes Headline.
	 */
	public function render() {
		if ( ! static::is_active() ) {
			return;
		}

		?>
		<div class="nebotheme-headline">
			<div class="grid-container">
				<div class="nebotheme-headline__inner nebotheme-padding-children">
					<?php $this->render_title(); ?>
					<?php $this->render_subtitle(); ?>
					<?php $this->render_description(); ?>
					<?php $this->render_breadcrumbs(); ?>
				</div>
			</div>
		</div><!-- .nebotheme-headline -->
		<?php
	}

	/**
	 * Get Link.
	 *
	 * @param string $text
	 * @param string $url
	 * @param bool $is_current
	 */
	protected function generate_link_array( $text, $url = '', $is_current = false ) {
		return [
			$text,
			$url,
			$is_current,
		];
	}

	/**
	 * Echoes headline title.
	 */
	public function render_title() {
		$tag = nebotheme_get_tag_h1();
		$title = '';

		if ( is_archive() ) {
			$title = get_the_archive_title();
		} elseif ( is_page() ) {
			$title = get_the_title();
		} elseif ( is_search() ) {
			$title = get_search_query( true );

			if ( ! have_posts() ) {
				$title = esc_html__( 'Nothing Found', 'yaffo' );
			} elseif ( ! $title ) {
				$title = esc_html__( 'Search page', 'yaffo' );
			}
		} elseif ( is_404() ) {
			$title = '404';
		}

		if ( ! $title ) {
			return;
		}

		echo '<' . tag_escape( $tag ) . ' class="nebotheme-headline__title nebotheme-font-adaptive">' .
			nebotheme_get_kses_content( $title ) .
		'</' . tag_escape( $tag ) . '>';
	}

	/**
	 * Echoes headline subtitle.
	 */
	public function render_subtitle() {
		$subtitle = '';

		if ( is_category() ) {
			$subtitle = esc_html__( 'Browsing Category', 'yaffo' );
		} elseif ( is_tag() ) {
			$subtitle = esc_html__( 'Browsing Tag', 'yaffo' );
		} elseif ( is_date() ) {
			$subtitle = esc_html__( 'Browsing Date', 'yaffo' );
		} elseif ( is_author() ) {
			$subtitle = esc_html__( 'Browsing Author', 'yaffo' );
		} elseif ( is_search() ) {
			global $wp_query;

			if ( have_posts() && $wp_query->found_posts > 1 ) {
				$subtitle = esc_html(
					sprintf(
						/* translators: post count */
						esc_html__( '%s Search Results For', 'yaffo' ),
						$wp_query->found_posts
					)
				) . ':';
			} else {
				$subtitle = esc_html__( 'Search Results', 'yaffo' );
			}
		}

		if ( ! $subtitle ) {
			return;
		}

		echo '<div class="nebotheme-headline__subtitle">' .
			esc_html( $subtitle ) .
		'</div>';
	}

	/**
	 * Echoes headline description.
	 */
	public function render_description() {
		$description = get_the_archive_description();

		if ( ! $description ) {
			return;
		}

		echo '<div class="nebotheme-headline__description">' .
			do_shortcode( $description ) .
		'</div>';
	}

	/**
	 * Echoes breadcrumbs.
	 */
	public function render_breadcrumbs() {
		if ( ! is_page() ) {
			return;
		}

		if ( SeoModule::is_render_breadcrumb() ) {
			SeoModule::render_breadcrumb();

			return;
		}

		global $post;

		$home_link = esc_url( home_url( '/' ) );
		$post_title = $post ? nebotheme_get_post_title() : '';

		$links_data = [];

		if ( is_single() ) {
			$category = get_the_category();
			$number_category = count( $category );

			if ( $number_category < 1 ) {
				$links_data[] = $this->generate_link_array( $post_title );
			} elseif ( $number_category >= 1 ) {
				$links_data[] = $this->generate_link_array(
					get_category_parents( $category[0], true, '' ) .
					' <span>' . $post_title . '</span>'
				);
			}
		} elseif ( is_page() ) {
			$links_data[] = $this->generate_link_array( $post_title, '', true );
		}

		if ( $links_data ) {
			array_unshift(
				$links_data,
				$this->generate_link_array( _x( 'Home', 'breadcrumb', 'yaffo' ), $home_link )
			);

			self::render_before();

			foreach ( $links_data as $link_data ) {
				list( $text, $url, $is_current ) = $link_data;

				$attrs = [];

				if ( $url ) {
					$attrs['href'] = $url;
					$tag = 'a';
				} else {
					$tag = 'span';
				}

				if ( $is_current ) {
					$attrs['class'] = 'current';
				}

				echo '<' . tag_escape( $tag ) . ' ' . nebotheme_get_render_attrs( $attrs ) . '>' .
					nebotheme_get_kses_content( $text ) .
				'</' . tag_escape( $tag ) . '>';

				if ( next( $links_data ) ) {
					echo nebotheme_get_render_breadcrumbs_separator();
				}
			}

			self::render_after();
		}
	}

	/**
	 * Echoes html before.
	 */
	public static function render_before() {
		?>
		<div class="nebotheme-breadcrumbs" id="breadcrumbs">
			<div class="nebotheme-breadcrumbs__inner">
		<?php
	}

	/**
	 * Echoes html after.
	 */
	public static function render_after() {
		?>
			</div>
		</div>
		<?php
	}
}
