<?php
/**
 * Homepage class.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo;

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Blog_Main;
use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Traits\Singleton;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Homepage class.
 */
final class Homepage {
	use Singleton;

	/**
	 * The WordPress Query class.
	 *
	 * @var \WP_Query
	 */
	protected static $wp_query_slider = [];

	/**
	 * Slider status.
	 *
	 * @var bool
	 */
	protected static $is_slider_rendering;

	/**
	 * Check slider status.
	 *
	 * @return bool
	 */
	public static function is_slider_rendering() {
		return self::$is_slider_rendering;
	}

	/**
	 * Check enable blog.
	 *
	 * @return bool
	 */
	public static function is_blog_enable() {
		return is_home() || ( self::is_blog_on_page_enable() );
	}

	/**
	 * Check enable blog.
	 *
	 * @return bool
	 */
	public static function is_blog_on_page_enable() {
		return ( is_page() && Meta_Boxes::get_pm_safe( 'homepage_blog_enable' ) );
	}

	/**
	 * Check for hide  page content.
	 *
	 * @return bool
	 */
	public static function is_hide_content() {
		$blog_enable = static::is_blog_enable();
		$hide_content = Meta_Boxes::get_pm_safe( 'homepage_blog_hide_content' );

		if ( ! $blog_enable ) {
			return false;
		} else {
			return $hide_content;
		}
	}

	/**
	 * The class constructor
	 */
	protected function __construct() {
		$this->add_hooks();
	}

	/**
	 * Add Hooks.
	 */
	protected function add_hooks() {
		add_action( 'customize_register', [ $this, 'reorder_static_front_page' ], 10 );
		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
		add_action( 'nebotheme_render_main_start', [ $this, 'render_featured_images' ], 2 );
		add_action( 'nebotheme_render_main_start', [ $this, 'render_slider' ], 1 );
		add_action( 'pre_get_posts', [ $this, 'exclude_posts_of_slider' ] );

		add_filter( 'nebotheme_meta_boxes_fields', function( array $field ) {
			return array_merge( $field, static::get_meta_boxes_fields() );
		} );
		add_filter( 'nebotheme_meta_boxes_tabs', function( array $tabs ) {
			return array_merge( $tabs, static::get_meta_boxes_tabs() );
		} );

		add_filter( 'body_class', [ $this, 'body_class' ] );
		add_filter( 'nebotheme_is_active_header_divider', [ $this, 'is_active_header_divider' ] );
	}

	/**
	 * Check if header divider need.
	 *
	 * @param bool $is_active
	 */
	public function is_active_header_divider( $is_active ) {
		if ( $is_active && static::is_enable_slider_settings() ) {
			if ( static::is_enable_slider() && ! static::get_settings( 'homepage_slider_space_header' ) ) {
				$is_active = false;
			}
		}

		return $is_active;
	}

	/**
	 * Filter for body classes.
	 */
	public function body_class( array $body_classes ) {
		if ( static::is_enable_slider() ) {
			$body_classes[] = 'nebotheme--with-slider';

			if ( static::get_settings( 'homepage_slider_space_header' ) ) {
				$body_classes[] = 'nebotheme--slider-space-header';
			}

			if ( static::get_settings( 'homepage_slider_custom_height' ) ) {
				$body_classes[] = 'nebotheme--slider-custom-height';
			}
		}

		return $body_classes;
	}

	/**
	 * Removes default WordPress Static Front Page section.
	 */
	public function reorder_static_front_page( \WP_Customize_Manager $wp_customize ) {
		$static_front_page = $wp_customize->get_section( 'static_front_page' );

		$wp_customize->remove_section( 'static_front_page' );

		Customize_Manager::add_section(
			'static_front_page',
			[
				'title' => esc_html__( 'Static Front Page', 'yaffo' ),
				'description' => $static_front_page->description,
				'section' => 'homepage',
				'active_callback' => $static_front_page->active_callback,
			]
		);
	}

	/**
	 * Add fields to customize.
	 */
	public function customize_add_fields() {
		$partial_refresh_featured_boxes = [
			'container_inclusive' => true,
			'selector' => 'body.home .nebotheme-featured-boxes',
			'render_callback' => [ $this, 'render_featured_images' ],
		];
		$partial_refresh_slider = [
			'container_inclusive' => true,
			'selector' => '.nebotheme-homepage-slider',
			'render_callback' => [ $this, 'render_slider' ],
		];
		$partial_refresh_blog = [
			'container_inclusive' => true,
			'selector' => 'body.home .nebotheme-blog',
			'render_callback' => 'nebotheme_render_main_blog',
		];
		$active_callback_slider = [
			'setting' => 'homepage_slider_enable',
			'operator' => '!=',
			'value' => '',
		];
		$active_callback_featured_images = [
			[
				'setting' => 'homepage_featured_images_enable',
				'operator' => '!=',
				'value' => '',
			],
		];

		Customize_Manager::add_section(
			'homepage',
			[
				'title' => esc_html__( 'Homepage Settings', 'yaffo' ),
				'icon' => 'dashicons-admin-home',
				'priority' => 2,
			]
		);

		Customize_Manager::add_section(
			'homepage_slider',
			[
				'title' => esc_html__( 'Slider Settings', 'yaffo' ),
				'section' => 'homepage',
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Show Slider', 'yaffo' ),
				'settings' => 'homepage_slider_enable',
				'type' => 'toggle',
				'section' => 'homepage_slider',
				'default' => false,
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Slider Layout', 'yaffo' ),
				'settings' => 'homepage_slider_layout',
				'type' => 'radio-buttonset',
				'default' => 'classic',
				'choices' => [
					'classic' => esc_html__( 'Classic', 'yaffo' ),
					'carousel' => esc_html__( 'Carousel', 'yaffo' ),
					'split' => esc_html__( 'Split', 'yaffo' ),
				],
				'section' => 'homepage_slider',
				'active_callback' => [ $active_callback_slider ],
				'partial_refresh' => [
					'homepage_slider_layout/slider' => $partial_refresh_slider,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Split Type', 'yaffo' ),
				'settings' => 'homepage_slider_split_type',
				'type' => 'radio-buttonset',
				'default' => 'double',
				'choices' => [
					'double' => esc_html__( 'Double', 'yaffo' ),
					'triple' => esc_html__( 'Triple', 'yaffo' ),
				],
				'section' => 'homepage_slider',
				'active_callback' => [
					$active_callback_slider,
					[
						'setting' => 'homepage_slider_layout',
						'operator' => '==',
						'value' => 'split',
					],
				],
				'partial_refresh' => [
					'homepage_slider_split_type/slider' => $partial_refresh_slider,
				],
			]
		);

		$post_types = array_intersect_key( wp_list_pluck( get_post_types(
			[
				'show_in_nav_menus' => true,
				'public' => true,
			],
			'object'
		), 'label' ), array_flip( [ 'post', 'product', 'page' ] ) );
		$has_product = isset( $post_types['product'] );
		$active_callback_post = [ $active_callback_slider ];

		if ( $has_product ) {
			$active_callback_post[] = [
				'setting' => 'homepage_slider_post_type',
				'operator' => '==',
				'value' => 'post',
			];
			$active_callback_product = [
				$active_callback_slider,
				[
					'setting' => 'homepage_slider_post_type',
					'operator' => '==',
					'value' => 'product',
				],
			];
			$active_callback_page = [
				$active_callback_slider,
				[
					'setting' => 'homepage_slider_post_type',
					'operator' => '==',
					'value' => 'page',
				],
			];

			Customize_Manager::add_field(
				[
					'type' => 'radio',
					'settings' => 'homepage_slider_post_type',
					'label' => esc_html__( 'Slider Post Type', 'yaffo' ),
					'section' => 'homepage_slider',
					'default' => 'post',
					'choices' => $post_types,
					'active_callback' => [ $active_callback_slider ],
					'partial_refresh' => [
						'homepage_slider_post_type/slider' => $partial_refresh_slider,
						'homepage_slider_post_type/blog' => $partial_refresh_blog,
					],
				]
			);
		}

		Customize_Manager::add_field(
			[
				'type' => 'radio',
				'settings' => 'homepage_slider_content_type',
				'label' => esc_html__( 'Slider Content Type', 'yaffo' ),
				'description' => esc_html__( 'Select how you want to get the Slider content', 'yaffo' ),
				'section' => 'homepage_slider',
				'default' => 'latest',
				'choices' => [
					'latest' => esc_html__( 'Latest Posts', 'yaffo' ),
					'category' => esc_html__( 'Posts by Categories', 'yaffo' ),
					'tag' => esc_html__( 'Posts by Tags', 'yaffo' ),
					'specific' => esc_html__( 'Specific Posts', 'yaffo' ),
				],
				'active_callback' => $active_callback_post,
				'partial_refresh' => [
					'homepage_slider_content_type/slider' => $partial_refresh_slider,
				],
			]
		);

		if ( $has_product ) {
			Customize_Manager::add_field(
				[
					'type' => 'radio',
					'settings' => 'homepage_slider_product_content_type',
					'label' => esc_html__( 'Slider Content Type', 'yaffo' ),
					'description' => esc_html__( 'Select how you want to get the Slider content', 'yaffo' ),
					'section' => 'homepage_slider',
					'default' => 'latest',
					'choices' => [
						'latest' => esc_html__( 'Latest Products', 'yaffo' ),
						'featured' => esc_html__( 'Featured Products', 'yaffo' ),
						'category' => esc_html__( 'Products by Categories', 'yaffo' ),
						'tag' => esc_html__( 'Products by Tags', 'yaffo' ),
						'specific' => esc_html__( 'Specific Products', 'yaffo' ),
					],
					'active_callback' => $active_callback_product,
					'partial_refresh' => [
						'homepage_slider_product_content_type/slider' => $partial_refresh_slider,
					],
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'nebotheme-ajax-select',
					'settings' => 'homepage_slider_product_content_category',
					'label' => esc_attr__( 'Select Category/Categories', 'yaffo' ),
					'section' => 'homepage_slider',
					'multiple' => true,
					'ajax_select' => [
						'type' => 'taxonomy',
						'sub_type' => 'product_cat',
					],
					'active_callback' => array_merge(
						$active_callback_product,
						[
							[
								'setting' => 'homepage_slider_product_content_type',
								'operator' => '==',
								'value' => 'category',
							],
						]
					),
					'partial_refresh' => [
						'homepage_slider_product_content_category/slider' => $partial_refresh_slider,
					],
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'nebotheme-ajax-select',
					'settings' => 'homepage_slider_product_content_tag',
					'label' => esc_attr__( 'Select Tag', 'yaffo' ),
					'section' => 'homepage_slider',
					'multiple' => true,
					'ajax_select' => [
						'type' => 'taxonomy',
						'sub_type' => 'product_tag',
					],
					'active_callback' => array_merge(
						$active_callback_product,
						[
							[
								'setting' => 'homepage_slider_product_content_type',
								'operator' => '==',
								'value' => 'tag',
							],
						]
					),
					'partial_refresh' => [
						'homepage_slider_product_content_tag/slider' => $partial_refresh_slider,
					],
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'nebotheme-ajax-select',
					'settings' => 'homepage_slider_product_content_posts',
					'label' => esc_html__( 'Enter Product', 'yaffo' ),
					'section' => 'homepage_slider',
					'multiple' => true,
					'ajax_select' => [
						'type' => 'post',
						'sub_type' => 'product',
					],
					'active_callback' => array_merge(
						$active_callback_product,
						[
							[
								'setting' => 'homepage_slider_product_content_type',
								'operator' => '==',
								'value' => 'specific',
							],
						]
					),
					'partial_refresh' => [
						'homepage_slider_product_content_posts/slider' => $partial_refresh_slider,
					],
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'nebotheme-ajax-select',
					'settings' => 'homepage_slider_page_content_posts',
					'label' => esc_html__( 'Enter Page', 'yaffo' ),
					'section' => 'homepage_slider',
					'multiple' => true,
					'ajax_select' => [
						'type' => 'post',
						'sub_type' => 'page',
					],
					'active_callback' => $active_callback_page,
					'partial_refresh' => [
						'homepage_slider_page_content_posts/slider' => $partial_refresh_slider,
						'homepage_slider_page_content_posts/blog' => $partial_refresh_blog,
					],
				]
			);

			Customize_Manager::add_field(
				[
					'label' => esc_html__( 'Custom Ratio', 'yaffo' ),
					'settings' => 'homepage_slider_custom_height',
					'type' => 'toggle',
					'section' => 'homepage_slider',
					'default' => false,
					'transport' => 'refresh',
					'active_callback' => array_merge(
						[ $active_callback_slider ]
					),
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'slider',
					'settings' => 'homepage_slider_ratio',
					'label' => esc_html__( 'Slider Ratio', 'yaffo' ),
					'description' => esc_html__( 'Desktop Device', 'yaffo' ),
					'section' => 'homepage_slider',
					'default' => 0.45,
					'stick' => true,
					'choices' => [
						'min' => 0.2,
						'max' => 2,
						'step' => 0.01,
					],
					'output' => [
						[
							'element' => '.nebotheme--slider-custom-height .nebotheme-homepage-slider',
							'property' => '--nebotheme-slider-image-ratio',
							'value_pattern' => '$ !important',
						],
					],
					'theme_export' => true,
					'active_callback' => array_merge(
						[ $active_callback_slider ],
						[
							[
								'setting' => 'homepage_slider_custom_height',
								'operator' => '!=',
								'value' => '',
							],
						]
					),
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'slider',
					'settings' => 'homepage_slider_ratio_mobile',
					'description' => esc_html__( 'Tablet/Mobile Device', 'yaffo' ),
					'section' => 'homepage_slider',
					'default' => nebotheme_get_tm( 'homepage_slider_ratio', 0.45 ) ? nebotheme_get_tm( 'homepage_slider_ratio', 0.45 ) : 0.45,
					'choices' => [
						'min' => 0.2,
						'max' => 2,
						'step' => 0.01,
					],
					'output' => [
						[
							'element' => '.nebotheme--slider-custom-height .nebotheme-homepage-slider',
							'media_query' => '@media (max-width: ' . yaffo_get_environment()['monitor_tablet'] . 'px)',
							'property' => '--nebotheme-slider-image-ratio',
							'value_pattern' => '$ !important',
						],
					],
					'theme_export' => true,
					'active_callback' => array_merge(
						[ $active_callback_slider ],
						[
							[
								'setting' => 'homepage_slider_custom_height',
								'operator' => '!=',
								'value' => '',
							],
						]
					),
				]
			);

			// Feat amount.
			Customize_Manager::add_field(
				[
					'type' => 'number',
					'settings' => 'homepage_slider_product_amount',
					'label' => esc_html__( 'Number of Slider Items', 'yaffo' ),
					'section' => 'homepage_slider',
					'default' => 3,
					'min' => 1,
					'max' => 10,
					'step' => 1,
					'active_callback' => array_merge(
						$active_callback_product,
						[
							[
								'setting' => 'homepage_slider_product_content_type',
								'operator' => '!=',
								'value' => 'specific',
							],
						]
					),
					'partial_refresh' => [
						'homepage_slider_product_amount/slider' => $partial_refresh_slider,
					],
				]
			);
		}

		Customize_Manager::add_field(
			[
				'type' => 'nebotheme-ajax-select',
				'settings' => 'homepage_slider_content_category',
				'label' => esc_attr__( 'Select Category/Categories', 'yaffo' ),
				'section' => 'homepage_slider',
				'multiple' => true,
				'ajax_select' => [
					'type' => 'taxonomy',
					'sub_type' => 'category',
				],
				'active_callback' => array_merge(
					$active_callback_post,
					[
						[
							'setting' => 'homepage_slider_content_type',
							'operator' => '==',
							'value' => 'category',
						],
					]
				),
				'partial_refresh' => [
					'homepage_slider_content_category/slider' => $partial_refresh_slider,
					'homepage_slider_content_category/blog' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'nebotheme-ajax-select',
				'settings' => 'homepage_slider_content_tag',
				'label' => esc_attr__( 'Select Tag', 'yaffo' ),
				'section' => 'homepage_slider',
				'multiple' => true,
				'ajax_select' => [
					'type' => 'taxonomy',
					'sub_type' => 'post_tag',
				],
				'active_callback' => array_merge(
					$active_callback_post,
					[
						[
							'setting' => 'homepage_slider_content_type',
							'operator' => '==',
							'value' => 'tag',
						],
					]
				),
				'partial_refresh' => [
					'homepage_slider_content_tag/slider' => $partial_refresh_slider,
					'homepage_slider_content_tag/blog' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'nebotheme-ajax-select',
				'settings' => 'homepage_slider_content_posts',
				'label' => esc_html__( 'Enter Post', 'yaffo' ),
				'section' => 'homepage_slider',
				'multiple' => true,
				'ajax_select' => [
					'type' => 'post',
					'sub_type' => 'post',
				],
				'active_callback' => array_merge(
					$active_callback_post,
					[
						[
							'setting' => 'homepage_slider_content_type',
							'operator' => '==',
							'value' => 'specific',
						],
					]
				),
				'partial_refresh' => [
					'homepage_slider_content_posts/slider' => $partial_refresh_slider,
					'homepage_slider_content_posts/blog' => $partial_refresh_blog,
				],
			]
		);

		// Feat amount.
		Customize_Manager::add_field(
			[
				'type' => 'number',
				'settings' => 'homepage_slider_amount',
				'label' => esc_html__( 'Number of Slider Items', 'yaffo' ),
				'section' => 'homepage_slider',
				'default' => 3,
				'min' => 1,
				'max' => 10,
				'step' => 1,
				'active_callback' => array_merge(
					$active_callback_post,
					[
						[
							'setting' => 'homepage_slider_content_type',
							'operator' => '!=',
							'value' => 'specific',
						],
					]
				),
				'partial_refresh' => [
					'homepage_slider_amount/slider' => $partial_refresh_slider,
					'homepage_slider_amount/blog' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_slider_date',
				'label' => esc_html__( 'Display Post Date', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage_slider',
				'default' => true,
				'theme_export' => true,
				'active_callback' => $active_callback_post,
				'partial_refresh' => [
					'homepage_slider_date/slider' => $partial_refresh_slider,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_slider_read_time',
				'label' => esc_html__( 'Display Post Read Time', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage_slider',
				'default' => true,
				'theme_export' => true,
				'active_callback' => $active_callback_post,
				'partial_refresh' => [
					'homepage_slider_read_time/slider' => $partial_refresh_slider,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_slider_space_header',
				'label' => esc_html__( 'Space Between Header', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage_slider',
				'default' => false,
				'theme_export' => true,
				'active_callback' => [ $active_callback_slider ],
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'radio',
				'settings' => 'homepage_blog_layout',
				'label' => esc_html__( 'Homepage Blog Layout', 'yaffo' ),
				'section' => 'homepage',
				'default' => 'classic',
				'choices' => [
					'classic' => esc_html__( 'Classic', 'yaffo' ),
					'classic/grid' => esc_html__( 'Classic / Grid', 'yaffo' ),
					'grid' => esc_html__( 'Grid', 'yaffo' ),
					'classic/list' => esc_html__( 'Classic / List', 'yaffo' ),
					'list' => esc_html__( 'List', 'yaffo' ),
				],
				'partial_refresh' => [
					'homepage_blog_layout' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_blog_date',
				'label' => esc_html__( 'Display Post Date', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage',
				'default' => true,
				'theme_export' => true,
				'partial_refresh' => [
					'homepage_blog_date' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_blog_read_time',
				'label' => esc_html__( 'Display Post Read Time', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage',
				'default' => true,
				'theme_export' => true,
				'partial_refresh' => [
					'homepage_blog_read_time' => $partial_refresh_blog,
				],
				'active_callback' => [
					[
						'setting' => 'homepage_blog_layout',
						'operator' => 'in',
						'value' => [ 'classic', 'classic/grid', 'classic/list', 'grid' ],
					],
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_blog_excerpt',
				'label' => esc_html__( 'Display Post Excerpt', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage',
				'default' => false,
				'theme_export' => true,
				'partial_refresh' => [
					'homepage_blog_excerpt' => $partial_refresh_blog,
				],
				'active_callback' => [
					[
						'setting' => 'homepage_blog_layout',
						'operator' => 'in',
						'value' => [ 'classic/grid', 'classic/list', 'grid' ],
					],
				],
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'homepage_blog_author',
				'label' => esc_html__( 'Display Post Author', 'yaffo' ),
				'type' => 'toggle',
				'section' => 'homepage',
				'default' => true,
				'theme_export' => true,
				'partial_refresh' => [
					'homepage_blog_author' => $partial_refresh_blog,
				],
				'active_callback' => [
					[
						'setting' => 'homepage_blog_layout',
						'operator' => 'in',
						'value' => [ 'list', 'classic/list' ],
					],
				],
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Masonry', 'yaffo' ),
				'settings' => 'homepage_blog_masonry',
				'type' => 'toggle',
				'section' => 'homepage',
				'default' => false,
				'active_callback' => [
					[
						'setting' => 'homepage_blog_layout',
						'operator' => 'in',
						'value' => [ 'grid', 'classic/grid' ],
					],
				],
				'partial_refresh' => [
					'homepage_blog_masonry' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Load More Button', 'yaffo' ),
				'settings' => 'homepage_blog_loadmore',
				'type' => 'toggle',
				'section' => 'homepage',
				'default' => false,
				'partial_refresh' => [
					'homepage_blog_loadmore' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Load More Text', 'yaffo' ),
				'settings' => 'homepage_blog_loadmore_text',
				'type' => 'text',
				'section' => 'homepage',
				'input_attrs' => [
					'placeholder' => esc_html__( 'Load More', 'yaffo' ),
				],
				'active_callback' => [
					[
						'setting' => 'homepage_blog_loadmore',
						'operator' => '!=',
						'value' => '',
					],
				],
				'partial_refresh' => [
					'homepage_blog_loadmore_text' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Grid Columns', 'yaffo' ) . ' <small>(' . esc_html__( 'without sidebar', 'yaffo' ) . ')</small>',
				'settings' => 'homepage_blog_grid_columns',
				'section' => 'homepage',
				'type' => 'radio',
				'default' => '2',
				'choices' => [
					'2' => esc_html__( '2 Columns', 'yaffo' ),
					'3' => esc_html__( '3 Columns', 'yaffo' ),
				],
				'active_callback' => [
					[
						'setting' => 'homepage_blog_layout',
						'operator' => 'in',
						'value' => [ 'grid', 'classic/grid' ],
					],
					[
						'setting' => 'homepage_blog_masonry',
						'operator' => '==',
						'value' => '',
					],
				],
				'output' => [
					[
						'element' => '.home .nebotheme-var-block:not(.sidebar-primary--exists) .nebotheme-blog__posts[data-layout="grid"]',
						'property' => '--nebotheme-blog-columns',
					],
				],
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Post Image', 'yaffo' ),
				'settings' => 'homepage_image_cropped',
				'type' => 'radio',
				'section' => 'homepage',
				'default' => 'cropped',
				'choices'  => [
					'cropped' => esc_attr__( 'Display Cropped Image', 'yaffo' ),
					'uncropped' => esc_attr__( 'Display in Original Ratio', 'yaffo' ),
				],
				'active_callback' => [
					[
						'setting' => 'homepage_blog_layout',
						'operator' => '==',
						'value' => 'grid',
					],
					[
						'setting' => 'homepage_blog_masonry',
						'operator' => '==',
						'value' => '',
					],
				],
				'partial_refresh' => [
					'homepage_image_cropped' => $partial_refresh_blog,
				],
			]
		);

		Customize_Manager::add_section(
			'homepage_featured_images',
			[
				'title' => esc_html__( 'Featured Images', 'yaffo' ),
				'section' => 'homepage',
			]
		);

		Customize_Manager::add_field(
			[
				'label' => esc_html__( 'Show Featured Images', 'yaffo' ),
				'settings' => 'homepage_featured_images_enable',
				'type' => 'toggle',
				'section' => 'homepage_featured_images',
				'default' => false,
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'slider',
				'settings' => 'homepage_featured_images_ratio',
				'label' => esc_html__( 'Featured Images Ratio', 'yaffo' ),
				'section' => 'homepage_featured_images',
				'default' => 0.55,
				'choices' => [
					'min' => 0.3,
					'max' => 1.5,
					'step' => 0.01,
				],
				'output' => [
					[
						'element' => 'body.home .nebotheme-featured-boxes .nebotheme-featured-image',
						'property' => '--nebotheme-featured-image-ratio',
					],
				],
				'theme_export' => true,
				'active_callback' => $active_callback_featured_images,
			]
		);

		static::loop_homepage_featured_image(
			function( $index ) use ( $partial_refresh_featured_boxes, $active_callback_featured_images ) {
				$label = sprintf(
					/* translators: number featured image */
					esc_html__( '%d. Featured Image', 'yaffo' ),
					$index + 1
				);

				Customize_Manager::add_field(
					[
						'label' => $label,
						'type' => 'nebotheme-popover-start',
						'section' => 'homepage_featured_images',
						'active_callback' => $active_callback_featured_images,
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => "homepage_featured_image_image_id_{$index}",
						'label' => esc_html__( 'Image', 'yaffo' ),
						'type' => 'image',
						'section' => 'homepage_featured_images',
						'choices' => [
							'save_as' => 'id',
						],
						'partial_refresh' => [
							"homepage_featured_image_image_id_{$index}" => $partial_refresh_featured_boxes,
						],
						'active_callback' => $active_callback_featured_images,
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => "homepage_featured_image_label_{$index}",
						'label' => esc_html__( 'Label', 'yaffo' ),
						'default' => $label,
						'type' => 'text',
						'section' => 'homepage_featured_images',
						'partial_refresh' => [
							"homepage_featured_image_label_{$index}" => $partial_refresh_featured_boxes,
						],
						'active_callback' => $active_callback_featured_images,
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => "homepage_featured_image_url_{$index}",
						'label' => esc_html__( 'URL', 'yaffo' ),
						'type' => 'url',
						'default' => '#',
						'section' => 'homepage_featured_images',
						'sanitize_callback' => 'esc_url',
						'partial_refresh' => [
							"homepage_featured_image_url_{$index}" => $partial_refresh_featured_boxes,
						],
						'active_callback' => $active_callback_featured_images,
					]
				);

				Customize_Manager::add_field(
					[
						'type' => 'nebotheme-popover-end',
						'section' => 'homepage_featured_images',
						'active_callback' => $active_callback_featured_images,
					]
				);
			}
		);
	}

	/**
	 * Exclude posts from blog by slider posts
	 */
	public function exclude_posts_of_slider( $query ) {
		if (
			! $query->is_home() ||
			! $query->is_main_query()
		) {
			return;
		}

		foreach ( Blog_Main::get_default_query_vars() as $query_var => $value ) {
			$query->set( $query_var, $value );
		}
	}

	/**
	 * Loop for each featured image.
	 */
	public static function loop_homepage_featured_image( callable $callback ) {
		$featured_image_count = 4;

		for ( $index = 0; $index < $featured_image_count; $index++ ) {
			call_user_func( $callback, $index );
		}
	}

	/**
	 * Check featured images.
	 */
	public static function is_enable_featured_images() {
		return (
			static::is_first_page() &&
			static::get_settings( 'homepage_featured_images_enable' ) &&
			(bool) static::get_featured_images()
		);
	}

	/**
	 * Add fields to Meta Boxes.
	 */
	public static function get_meta_boxes_tabs() {
		$tabs = [
			'homepage_blog' => [
				'label' => esc_html__( 'Blog Settings', 'yaffo' ),
				'post_type' => [ 'page' ],
			],
			'homepage_slider' => [
				'label' => esc_html__( 'Slider Settings', 'yaffo' ),
				'post_type' => [ 'page' ],
			],
			'homepage_featured_images' => [
				'label' => esc_html__( 'Featured Images', 'yaffo' ),
				'post_type' => [ 'page' ],
			],
		];

		return $tabs;
	}

	/**
	 * Add tabs to Meta Boxes.
	 */
	public static function get_meta_boxes_fields() {
		$fields = [
			/* < Slider Settings */
			'homepage_slider_enable' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_enable', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_slider',
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
			],
			'homepage_slider_layout' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_layout', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'default' => Customize_Manager::get_field( 'homepage_slider_layout', 'default' ),
				'options' => Customize_Manager::get_field( 'homepage_slider_layout', 'choices', [] ),
				'tab' => 'homepage_slider',
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_layout', 'active_callback', [] )
				),
			],
			'homepage_slider_split_type' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_split_type', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'default' => Customize_Manager::get_field( 'homepage_slider_split_type', 'default' ),
				'options' => Customize_Manager::get_field( 'homepage_slider_split_type', 'choices', [] ),
				'tab' => 'homepage_slider',
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_split_type', 'active_callback', [] )
				),
			],
		];

		if ( post_type_exists( 'product' ) ) {
			$fields += [
				'homepage_slider_post_type' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_post_type', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'default' => Customize_Manager::get_field( 'homepage_slider_post_type', 'default' ),
					'options' => Customize_Manager::get_field( 'homepage_slider_post_type', 'choices', [] ),
					'tab' => 'homepage_slider',
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_post_type', 'active_callback', [] )
					),
				],
				'homepage_slider_product_content_type' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_product_content_type', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'default' => Customize_Manager::get_field( 'homepage_slider_product_content_type', 'default' ),
					'options' => Customize_Manager::get_field( 'homepage_slider_product_content_type', 'choices', [] ),
					'tab' => 'homepage_slider',
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_product_content_type', 'active_callback', [] )
					),
				],
				'homepage_slider_product_content_category' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_product_content_category', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'default' => Customize_Manager::get_field( 'homepage_slider_product_content_category', 'default' ),
					'ajax_select' => Customize_Manager::get_field( 'homepage_slider_product_content_category', 'ajax_select' ),
					'tab' => 'homepage_slider',
					'multiple' => true,
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_product_content_category', 'active_callback', [] )
					),
				],
				'homepage_slider_product_content_tag' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_product_content_tag', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'ajax_select' => Customize_Manager::get_field( 'homepage_slider_product_content_tag', 'ajax_select' ),
					'tab' => 'homepage_slider',
					'multiple' => true,
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_product_content_tag', 'active_callback', [] )
					),
				],
				'homepage_slider_product_content_posts' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_product_content_posts', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'tab' => 'homepage_slider',
					'multiple' => true,
					'ajax_select' => Customize_Manager::get_field( 'homepage_slider_product_content_posts', 'ajax_select' ),
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_product_content_posts', 'active_callback', [] )
					),
				],
				'homepage_slider_product_amount' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_product_amount', 'label' ),
					'type' => Meta_Boxes::CONTROL_NUMBER,
					'tab' => 'homepage_slider',
					'description' => Customize_Manager::get_field( 'homepage_slider_product_amount', 'description' ),
					'default' => Customize_Manager::get_field( 'homepage_slider_product_amount', 'default' ),
					'min' => Customize_Manager::get_field( 'homepage_slider_product_amount', 'min' ),
					'max' => Customize_Manager::get_field( 'homepage_slider_product_amount', 'max' ),
					'step' => Customize_Manager::get_field( 'homepage_slider_product_amount', 'step' ),
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_product_amount', 'active_callback', [] )
					),
				],
				'homepage_slider_page_content_posts' => [
					'label' => Customize_Manager::get_field( 'homepage_slider_page_content_posts', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'tab' => 'homepage_slider',
					'multiple' => true,
					'ajax_select' => Customize_Manager::get_field( 'homepage_slider_page_content_posts', 'ajax_select' ),
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_slider_page_content_posts', 'active_callback', [] )
					),
				],
			];
		}

		$fields += [
			'homepage_slider_content_type' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_content_type', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'tab' => 'homepage_slider',
				'description' => Customize_Manager::get_field( 'homepage_slider_content_type', 'description' ),
				'default' => Customize_Manager::get_field( 'homepage_slider_content_type', 'default' ),
				'options' => Customize_Manager::get_field( 'homepage_slider_content_type', 'choices', [] ),
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_content_type', 'active_callback', [] )
				),
			],
			'homepage_slider_content_category' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_content_category', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'default' => Customize_Manager::get_field( 'homepage_slider_content_category', 'default' ),
				'ajax_select' => Customize_Manager::get_field( 'homepage_slider_content_category', 'ajax_select' ),
				'tab' => 'homepage_slider',
				'multiple' => true,
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_content_category', 'active_callback', [] )
				),
			],
			'homepage_slider_content_tag' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_content_tag', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'ajax_select' => Customize_Manager::get_field( 'homepage_slider_content_tag', 'ajax_select' ),
				'tab' => 'homepage_slider',
				'multiple' => true,
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_content_tag', 'active_callback', [] )
				),
			],
			'homepage_slider_content_posts' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_content_posts', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'tab' => 'homepage_slider',
				'multiple' => true,
				'ajax_select' => Customize_Manager::get_field( 'homepage_slider_content_posts', 'ajax_select' ),
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_content_posts', 'active_callback', [] )
				),
			],
			'homepage_slider_amount' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_amount', 'label' ),
				'type' => Meta_Boxes::CONTROL_NUMBER,
				'tab' => 'homepage_slider',
				'description' => Customize_Manager::get_field( 'homepage_slider_amount', 'description' ),
				'default' => Customize_Manager::get_field( 'homepage_slider_amount', 'default' ),
				'min' => Customize_Manager::get_field( 'homepage_slider_amount', 'min' ),
				'max' => Customize_Manager::get_field( 'homepage_slider_amount', 'max' ),
				'step' => Customize_Manager::get_field( 'homepage_slider_amount', 'step' ),
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_amount', 'active_callback', [] )
				),
			],
			'homepage_slider_space_header' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_space_header', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_slider',
				'default' => Customize_Manager::get_field( 'homepage_slider_space_header', 'default' ),
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_space_header', 'active_callback', [] )
				),
			],
			'homepage_slider_date' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_date', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_slider',
				'default' => Customize_Manager::get_field( 'homepage_slider_date', 'default' ),
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_date', 'active_callback', [] )
				),
			],
			'homepage_slider_read_time' => [
				'label' => Customize_Manager::get_field( 'homepage_slider_read_time', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_slider',
				'default' => Customize_Manager::get_field( 'homepage_slider_read_time', 'default' ),
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_slider_read_time', 'active_callback', [] )
				),
			],
			/* Slider Settings > */

			/* Blog Settings < */
			'homepage_blog_enable' => [
				'label' => esc_html__( 'Blog on this page?', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'tab' => 'homepage_blog',
			],
			'homepage_blog_layout' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_layout', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_blog_layout', 'default' ),
				'options' => Customize_Manager::get_field( 'homepage_blog_layout', 'choices', [] ),
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			'homepage_blog_date' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_date', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_blog_date', 'default' ),
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			'homepage_blog_read_time' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_read_time', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_blog_read_time', 'default' ),
				'condition' => array_merge(
					Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_blog_read_time', 'active_callback', [] )
					),
					[
						'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					]
				),
			],
			'homepage_blog_excerpt' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_excerpt', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_blog_excerpt', 'default' ),
				'condition' => array_merge(
					Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_blog_excerpt', 'active_callback', [] )
					),
					[
						'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					]
				),
			],
			'homepage_blog_author' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_author', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_blog_author', 'default' ),
				'condition' => array_merge(
					Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_blog_author', 'active_callback', [] )
					),
					[
						'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					]
				),
			],
			'homepage_blog_masonry' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_masonry', 'label' ),
				'default' => Customize_Manager::get_field( 'homepage_blog_masonry', 'default' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_blog',
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'condition' => array_merge(
					Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_blog_masonry', 'active_callback', [] )
					),
					[
						'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					]
				),
			],
			'homepage_blog_grid_columns' => [
				'label' => Customize_Manager::get_field( 'homepage_blog_grid_columns', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_blog_grid_columns', 'default' ),
				'options' => Customize_Manager::get_field( 'homepage_blog_grid_columns', 'choices', [] ),
				'selectors' => [
					'.nebotheme-var-block:not(.sidebar-primary--exists) .nebotheme-blog__posts[data-layout="grid"]' => '--nebotheme-blog-columns: {{VALUE}};',
				],
				'condition' => array_merge(
					Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_blog_grid_columns', 'active_callback', [] )
					),
					[
						'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					]
				),
			],
			'homepage_image_cropped' => [
				'label' => Customize_Manager::get_field( 'homepage_image_cropped', 'label' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'tab' => 'homepage_blog',
				'default' => Customize_Manager::get_field( 'homepage_image_cropped', 'default' ),
				'options' => Customize_Manager::get_field( 'homepage_image_cropped', 'choices', [] ),
				'condition' => array_merge(
					Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( 'homepage_image_cropped', 'active_callback', [] )
					),
					[
						'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					]
				),
			],
			'homepage_blog_posts_per_page' => [
				'label' => esc_html__( 'Posts Per Page', 'yaffo' ),
				'description' => esc_html__( 'Not including sticky post', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_NUMBER,
				'min' => 1,
				'tab' => 'homepage_blog',
				'default' => get_option( 'posts_per_page' ),
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			'homepage_blog_category' => [
				'label' => esc_html__( 'Filter by Categories', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'multiple' => true,
				'ajax_select' => [
					'type' => 'taxonomy',
					'sub_type' => 'category',
				],
				'tab' => 'homepage_blog',
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			'homepage_blog_tag' => [
				'label' => esc_html__( 'Filter by Tags', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'multiple' => true,
				'ajax_select' => [
					'type' => 'taxonomy',
					'sub_type' => 'post_tag',
				],
				'tab' => 'homepage_blog',
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			'homepage_blog_relation' => [
				'label' => esc_html__( 'Filter Relation', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_SELECT,
				'default' => 'or',
				'options' => [
					'or' => esc_html__( 'Categories or Tags', 'yaffo' ),
					'and' => esc_html__( 'Categories and Tags', 'yaffo' ),
				],
				'tab' => 'homepage_blog',
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
					'homepage_blog_category!' => YAFFO_VALUE_NEGATIVE,
					'homepage_blog_tag!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			'homepage_blog_hide_content' => [
				'label' => esc_html__( 'Hide Page Content?', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_blog',
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
				'condition' => [
					'homepage_blog_enable!' => YAFFO_VALUE_NEGATIVE,
				],
			],
			/* Blog Settings > */

			/* Featured Image Settings < */
			'homepage_featured_images_enable' => [
				'label' => Customize_Manager::get_field( 'homepage_featured_images_enable', 'label' ),
				'type' => Meta_Boxes::CONTROL_SWITCHER,
				'tab' => 'homepage_featured_images',
				'label_enable' => esc_html__( 'Yes', 'yaffo' ),
				'label_disabled' => esc_html__( 'No', 'yaffo' ),
			],
			'homepage_featured_images_ratio' => [
				'label' => Customize_Manager::get_field( 'homepage_featured_images_ratio', 'label' ),
				'type' => Meta_Boxes::CONTROL_SLIDER,
				'default' => Customize_Manager::get_field( 'homepage_featured_images_ratio', 'default' ),
				'min' => Customize_Manager::get_field( 'homepage_featured_images_ratio', 'choices' )['min'],
				'max' => Customize_Manager::get_field( 'homepage_featured_images_ratio', 'choices' )['max'],
				'step' => Customize_Manager::get_field( 'homepage_featured_images_ratio', 'choices' )['step'],
				'tab' => 'homepage_featured_images',
				'selectors' => [
					'.nebotheme-featured-boxes .nebotheme-featured-image' => '--nebotheme-featured-image-ratio: {{VALUE}};',
				],
				'condition' => Meta_Boxes::conversion_condition(
					Customize_Manager::get_field( 'homepage_featured_images_ratio', 'active_callback', [] )
				),
			],
			/* Featured Image Settings > */
		];

		/* Featured Image Settings < */
		static::loop_homepage_featured_image(
			function( $index ) use ( &$fields ) {
				$fields[ "homepage_featured_image_heading_{$index}" ] = [
					'label' => sprintf(
						/* translators: number featured image */
						esc_html__( '%d. Featured Image', 'yaffo' ),
						$index + 1
					),
					'type' => Meta_Boxes::CONTROL_HEADING,
					'tab' => 'homepage_featured_images',
					'tag' => 'h3',
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( "homepage_featured_image_image_id_{$index}", 'active_callback', [] )
					),
				];

				$fields[ "homepage_featured_image_image_id_{$index}" ] = [
					'label' => Customize_Manager::get_field( "homepage_featured_image_image_id_{$index}", 'label' ),
					'type' => Meta_Boxes::CONTROL_IMAGE,
					'tab' => 'homepage_featured_images',
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( "homepage_featured_image_image_id_{$index}", 'active_callback', [] )
					),
				];

				$fields[ "homepage_featured_image_label_{$index}" ] = [
					'label' => Customize_Manager::get_field( "homepage_featured_image_label_{$index}", 'label' ),
					'type' => Meta_Boxes::CONTROL_TEXT,
					'default' => Customize_Manager::get_field( "homepage_featured_image_label_{$index}", 'default' ),
					'tab' => 'homepage_featured_images',
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( "homepage_featured_image_label_{$index}", 'active_callback', [] )
					),
				];

				$fields[ "homepage_featured_image_url_{$index}" ] = [
					'label' => Customize_Manager::get_field( "homepage_featured_image_url_{$index}", 'label' ),
					'type' => Meta_Boxes::CONTROL_TEXT,
					'default' => Customize_Manager::get_field( "homepage_featured_image_url_{$index}", 'default' ),
					'placeholder' => esc_html__( 'Enter URL', 'yaffo' ),
					'tab' => 'homepage_featured_images',
					'condition' => Meta_Boxes::conversion_condition(
						Customize_Manager::get_field( "homepage_featured_image_url_{$index}", 'active_callback', [] )
					),
				];
			}
		);
		/* Featured Image Settings > */

		return $fields;
	}

	/**
	 * Check if first page of blog.
	 *
	 * @return bool
	 */
	public static function is_first_page() {
		return 1 === Blog_Main::get_paged();
	}

	/**
	 * Check slider for display.
	 */
	public static function is_enable_slider_settings() {
		return static::get_settings( 'homepage_slider_enable' );
	}

	/**
	 * Check slider for display.
	 */
	public static function is_enable_slider() {
		$wp_query = static::get_wp_query_slider();

		return ( $wp_query->post_count && static::is_first_page() && static::is_enable_slider_settings() );
	}

	/**
	 * Check if slider by products.
	 *
	 * @return bool
	 */
	public static function is_products_slider() {
		return post_type_exists( 'product' ) && 'product' === static::get_settings( 'homepage_slider_post_type' );
	}

	/**
	 * Get query vars for slider.
	 */
	public static function get_query_vars_slider() {
		$is_products_slider = static::is_products_slider();
		$is_pages_slider = 'page' === static::get_settings( 'homepage_slider_post_type' );
		$prefix_field = 'homepage_slider';
		$query_vars = [
			'post_status' => 'publish',
		];

		if ( $is_pages_slider ) {
			$prefix_field .= '_page';
			$query_vars += [
				'post_type' => [ 'page' ],
			];
		} elseif ( $is_products_slider ) {
			$prefix_field .= '_product';
			$query_vars += [
				'post_type' => [ 'product' ],
			];
		} else {
			$sticky_posts = get_option( 'sticky_posts' );

			$query_vars['post_type'] = [ 'post' ];
			$query_vars['ignore_sticky_posts'] = true;

			if ( $sticky_posts ) {
				$query_vars['post__not_in'] = $sticky_posts;
			}
		}

		if ( $is_pages_slider ) {
			$content_type = 'specific';
		} else {
			$content_type = static::get_settings( "{$prefix_field}_content_type" );
		}

		if ( 'specific' !== $content_type ) {
			$query_vars['posts_per_page'] = static::get_settings( "{$prefix_field}_amount" );
		}

		switch ( $content_type ) {
			case 'category':
				$cat_ids = static::get_settings( "{$prefix_field}_content_category" );

				if ( $is_products_slider ) {
					$query_vars['tax_query'] = [ //phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_tax_query
						[
							'taxonomy' => 'product_cat',
							'field' => 'id',
							'terms' => $cat_ids,
						],
					];
				} else {
					$query_vars['cat'] = $cat_ids;
				}
				break;
			case 'tag':
				$tag_ids = static::get_settings( "{$prefix_field}_content_tag" );

				if ( $is_products_slider ) {
					$query_vars['tax_query'] = [ //phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_tax_query
						[
							'taxonomy' => 'product_tag',
							'field' => 'id',
							'terms' => $tag_ids,
						],
					];
				} else {
					$query_vars['tag__in'] = $tag_ids;
				}
				break;
			case 'specific':
				$content_posts = static::get_settings( "{$prefix_field}_content_posts" );

				if ( $content_posts ) {
					if ( is_array( $content_posts ) ) {
						$query_vars['post__in'] = $content_posts;
					} elseif ( is_string( $content_posts ) ) {
						$query_vars['post__in'] = explode( ',', $content_posts );
					}

					$query_vars['orderby'] = 'post__in';
				}

				break;
		}

		if ( $is_products_slider ) {
			switch ( $content_type ) {
				case 'latest':
					$query_vars += [
						'orderby' => 'menu_order title',
						'order' => 'ASC',
					];
					break;
				case 'featured':
					$query_vars['tax_query'] = [ //phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_tax_query
						[
							'taxonomy' => 'product_visibility',
							'terms' => 'featured',
							'field' => 'name',
							'operator' => 'IN',
						],
					];
					break;
			}
		}

		return $query_vars;
	}

	/**
	 * Get current settings.
	 *
	 * @param string $settings_name
	 */
	public static function get_settings( $settings_name ) {
		if ( is_home() ) {
			return nebotheme_get_tm( $settings_name );
		} elseif ( is_page() ) {
			global $wp_query;

			return Meta_Boxes::get_pm_safe( $settings_name, $wp_query->post );
		}

		return false;
	}

	/**
	 * Get all featured images.
	 */
	public static function get_featured_images() {
		$featured_images = [];

		static::loop_homepage_featured_image(
			function( $index ) use ( &$featured_images ) {
				$featured_image = [
					'label' => static::get_settings(
						"homepage_featured_image_label_{$index}",
						sprintf(
							/* translators: number featured image */
							esc_html__( '%d. Featured Image', 'yaffo' ),
							$index + 1
						)
					),
					'image_size' => 'medium_large',
					'image_id' => static::get_settings( "homepage_featured_image_image_id_{$index}" ),
					'url' => static::get_settings( "homepage_featured_image_url_{$index}" ),
					'src' => '',
				];

				if ( (bool) array_filter( $featured_image ) ) {
					$featured_images[] = $featured_image;
				}
			}
		);

		return $featured_images;
	}

	/**
	 * Get wp_query from slider
	 */
	public static function get_wp_query_slider() {
		if ( ! static::$wp_query_slider ) {
			static::$wp_query_slider = new \WP_Query( static::get_query_vars_slider() );
		}

		return static::$wp_query_slider;
	}

	/**
	 * Echoes slider.
	 */
	public function render_slider() {
		if ( ! static::is_enable_slider() ) {
			return;
		}

		self::$is_slider_rendering = true;

		$wp_query = static::get_wp_query_slider();

		if ( ! $wp_query->have_posts() ) {
			return;
		}

		$slider_layout = static::get_settings( 'homepage_slider_layout' );
		$slider_options = [
			'adaptiveHeight' => true,
			'arrows' => true,
			'autoplay' => true,
			'autoplaySpeed' => 5000,
			'autoplaySpeed' => 5000,
		];

		if ( 'classic' === $slider_layout ) {
			$slider_options['fade'] = true;
		} elseif ( 'carousel' === $slider_layout ) {
			$slider_options['slidesToScroll'] = 3;
			$slider_options['slidesToShow'] = 3;
			$slider_options['responsive'] = [
				[
					'breakpoint' => yaffo_get_environment()['monitor_tablet'],
					'settings' => [
						'slidesToShow' => 2,
						'slidesToScroll' => 2,
					],
				],
				[
					'breakpoint' => yaffo_get_environment()['monitor_mobile'],
					'settings' => [
						'slidesToShow' => 1,
						'slidesToScroll' => 1,
					],
				],
			];
		} elseif ( 'split' === $slider_layout ) {
			$split_type = static::get_settings( 'homepage_slider_split_type' );

			$slider_options['slidesToScroll'] = 1;
			$slider_options['responsive'] = [
				[
					'breakpoint' => yaffo_get_environment()['monitor_tablet'],
					'settings' => [
						'slidesToShow' => 1,
						'slidesToScroll' => 1,
					],
				],
			];

			if ( 'triple' === $split_type ) {
				$slider_options['slidesToShow'] = 3;
			} else {
				$slider_options['slidesToShow'] = 2;
			}
		}

		$meta_data_bottom = [];

		if ( static::is_products_slider() ) {
			$meta_data_bottom[] = 'product_price';
		} else {
			if ( static::get_settings( 'homepage_slider_read_time' ) ) {
				$meta_data_bottom[] = 'read_time';
			}

			if ( static::get_settings( 'homepage_slider_date' ) ) {
				$meta_data_bottom[] = 'date';
			}
		}

		$grid_classes = 'grid-container';
		$homepage_slider_classes = "nebotheme-homepage-slider nebotheme-homepage-slider--{$slider_layout}";

		if ( 'split' === $slider_layout ) {
			$grid_classes .= ' full-width';

			if ( 'triple' === static::get_settings( 'homepage_slider_split_type' ) ) {
				$homepage_slider_classes .= ' nebotheme-homepage-slider--split-triple';
			}
		} else {
			$grid_classes .= ' wide-adaptive';
		}

		?>
		<div class="<?php echo esc_attr( $homepage_slider_classes ); ?>">
			<div class="<?php echo esc_attr( $grid_classes ); ?>">
				<?php
				nebotheme_render_post_slider(
					$wp_query,
					function() use ( $wp_query, $slider_layout, $meta_data_bottom ) {
						?>
						<div class="nebotheme-homepage-slider__item">
							<?php
							nebotheme_render_post_open( '', false );

							if ( 'classic' === $slider_layout ) {
								$thumbnail_args = [
									'size' => 'full',
									'placeholder' => true,
									'bg_image' => true,
								];

								nebotheme_render_thumbnail( $thumbnail_args );

								?>
								<div class="content-wrap nebotheme-padding-children">
									<?php
									nebotheme_render_post_meta( [ 'category' ] );

									nebotheme_render_post_title(
										[
											'title_attr' => true,
											'tag' => 'h2',
										]
									);

									nebotheme_render_post_meta( $meta_data_bottom );
									?>
								</div>
								<?php
							} elseif ( 'carousel' === $slider_layout ) {
								nebotheme_render_thumbnail(
									[
										'size' => 'yaffo-medium',
										'placeholder' => true,
										'bg_image' => true,
									]
								);

								nebotheme_render_post_meta( [ 'category' ] );

								nebotheme_render_post_title(
									[
										'tag' => 'h3',
									]
								);

								nebotheme_render_post_meta( $meta_data_bottom );
							} elseif ( 'split' === $slider_layout ) {
								nebotheme_render_thumbnail(
									[
										'size' => 'large',
										'placeholder' => true,
										'bg_image' => true,
									]
								);

								echo '<div class="content-wrap nebotheme-padding-children">';

								nebotheme_render_post_meta( [ 'category' ] );

								nebotheme_render_post_title(
									[
										'tag' => 'h2',
									]
								);

								nebotheme_render_post_meta( $meta_data_bottom );

								echo '</div>';
							}

							nebotheme_render_post_close();
							?>
						</div>
						<?php
					},
					$slider_options
				);
		?>
			</div>
		</div><!-- .nebotheme-homepage-slider -->
		<?php

		self::$is_slider_rendering = false;
	}

	/**
	 * Echoes each featured image.
	 */
	public function render_featured_images() {
		if ( ! static::is_enable_featured_images() ) {
			return;
		}

		$featured_images = static::get_featured_images();

		?>
		<section class="nebotheme-featured-boxes">
			<div class="grid-container">
				<div class="nebotheme-featured-boxes__inner">
				<?php
				foreach ( $featured_images as $featured_box ) {
					nebotheme_render_featured_image( $featured_box );
				}
				?>
				</div>
			</div>
		</section>
		<?php
	}
}
