<?php
/**
 * Logo class.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo;

use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Utils;
use NeboTheme\Yaffo\Traits\Singleton;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Logo class.
 */
final class Logos {
	const CUSTOM_LOGO_TM = 'custom_logo';

	use Singleton;

	/**
	 * Check if custom.
	 *
	 * @param string $logo_type
	 *
	 * @return bool
	 */
	public static function is_custom_logo( $logo_type ) {
		$logo_data = Utils::get_if_isset( self::get_logo_types(), $logo_type );

		if ( ! $logo_data ) {
			return;
		}

		$logo_id = (int) nebotheme_get_tm( $logo_data['theme_mod'] );

		return $logo_id && wp_attachment_is_image( $logo_id );
	}

	/**
	 * Get logo url by name.
	 *
	 * @param string $logo_tm
	 *
	 * @return string|false
	 */
	public static function get_logo_url( $logo_tm ) {
		$logo_id = static::get_logo_id( $logo_tm );

		if ( $logo_id && wp_attachment_is_image( $logo_id ) ) {
			$logo_url = wp_get_attachment_image_url( $logo_id, 'full' );
		} else {
			$logo_url = Utils::get_if_isset( nebotheme_get_theme_vars( 'logo_urls' ), $logo_tm );
		}

		return $logo_url;
	}

	/**
	 * Get logo id by name.
	 *
	 * @param string $logo_tm
	 *
	 * @return int|false
	 */
	public static function get_logo_id( $logo_tm ) {
		$logo_id = nebotheme_get_tm( $logo_tm );

		if (
			(
				'logo_header_mobile' === $logo_tm ||
				'logo_header_mobile_retina' === $logo_tm
			) &&
			! $logo_id &&
			self::is_custom_logo( 'header' )
		) {
			$logo_header_tm = self::CUSTOM_LOGO_TM;

			if ( 'logo_header_mobile_retina' === $logo_tm ) {
				$logo_header_tm .= '_retina';
			}

			return self::get_logo_id( $logo_header_tm );
		}

		return $logo_id;
	}

	/**
	 * Get labels for logo button.
	 */
	public static function get_custom_logo_button_labels() {
		return [
			'change' => esc_html__( 'Change custom logo', 'yaffo' ),
			'default' => esc_html__( 'Default', 'yaffo' ),
			'frame_button' => esc_html__( 'Choose custom logo', 'yaffo' ),
			'frame_title' => esc_html__( 'Select custom logo', 'yaffo' ),
			'placeholder' => esc_html__( 'No custom logo selected', 'yaffo' ),
			'remove' => esc_html__( 'Remove', 'yaffo' ),
			'select' => esc_html__( 'Select custom logo', 'yaffo' ),
		];
	}

	/**
	 * Get logo types.
	 */
	public static function get_logo_types() {
		return [
			'header' => [
				'theme_mod' => self::CUSTOM_LOGO_TM,
				'label' => esc_html__( 'Logo Header', 'yaffo' ),
			],
			'header_mobile' => [
				'theme_mod' => 'logo_header_mobile',
				'label' => esc_html__( 'Logo Header Mobile', 'yaffo' ),
			],
			'footer' => [
				'theme_mod' => 'logo_footer',
				'label' => esc_html__( 'Logo Footer', 'yaffo' ),
			],
			'off_canvas' => [
				'theme_mod' => 'logo_off_canvas',
				'label' => esc_html__( 'Logo Off-Canvas', 'yaffo' ),
			],
		];
	}

	/**
	 * Get srcset attribute.
	 *
	 * @param string $logo_tm
	 */
	public static function get_srcset( $logo_tm ) {
		$logo_tm_retina = "{$logo_tm }_retina";

		if ( ! nebotheme_get_tm( $logo_tm_retina ) ) {
			return;
		}

		$logo_src = self::get_logo_url( $logo_tm );
		$logo_retina_src = self::get_logo_url( $logo_tm_retina );
		$srcset = '';

		if ( $logo_src && $logo_retina_src ) {
			$srcset = "{$logo_src} 1x, {$logo_retina_src} 2x";
		}

		return $srcset;
	}

	/**
	 * Echoes Logo/Retina by theme-mod name
	 *
	 * @param string $logo_type
	 *
	 * @return void
	 */
	public static function render_logo( $logo_type ) {
		$logo_data = Utils::get_if_isset( self::get_logo_types(), $logo_type );

		if ( ! $logo_data ) {
			return;
		}

		$home_url = home_url();
		$alt = get_bloginfo( 'description' );

		if ( ! $alt ) {
			$alt = esc_html__( 'Logo', 'yaffo' );
		}

		$attrs = [
			'src' => self::get_logo_url( $logo_data['theme_mod'] ),
			'alt' => $alt,
			'title' => $alt,
			'loading' => 'lazy',
		];

		$srcset = self::get_srcset( $logo_data['theme_mod'] );

		if ( $srcset ) {
			$attrs['srcset'] = $srcset;
		}

		$classes = [ 'nebotheme-logo' ];

		if ( is_customize_preview() ) {
			$classes[] = "nebotheme-logo--{$logo_type}";
		}

		?>
		<div class="<?php echo esc_attr( implode( ' ', $classes ) ); ?>">
			<a href="<?php echo esc_attr( $home_url ); ?>" rel="home">
				<img <?php nebotheme_render_attrs( $attrs ); ?>>
			</a>
		</div>
		<?php
	}

	/**
	 * The class constructor
	 */
	protected function __construct() {
		$this->add_hooks();
	}

	/**
	 * Add Hooks.
	 */
	protected function add_hooks() {
		add_action( 'customize_register', [ $this, 'change_section_custom_logo' ], 999 );
		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
	}

	/**
	 * Custom logo image attributes.
	 */
	public function custom_logo_image_attrs( $attrs ) {
		$srcset = self::get_srcset( self::get_logo_types()['header']['theme_mod'] );

		if ( $srcset ) {
			$attrs['srcset'] = $srcset;
		}

		return $attrs;
	}

	/**
	 * Move custom_logo field.
	 */
	public function change_section_custom_logo( \WP_Customize_Manager $wp_customize ) {
		$control = $wp_customize->get_control( self::CUSTOM_LOGO_TM );
		$control_retina = $wp_customize->get_control( self::CUSTOM_LOGO_TM . '_retina' );

		if ( ! $control || ! $control_retina ) {
			return;
		}

		$control->label = esc_html__( 'Logo', 'yaffo' );
		$control->section = $control_retina->section;
		$control->priority = $control_retina->priority;
		$control->instance_number = $control_retina->instance_number;
		$control->button_labels = self::get_custom_logo_button_labels();

		$control_retina->instance_number++;

		$wp_customize->selective_refresh->remove_partial( self::CUSTOM_LOGO_TM );

		$wp_customize->selective_refresh->add_partial(
			self::CUSTOM_LOGO_TM,
			[
				'container_inclusive' => true,
				'render_callback' => function() {
					self::render_logo( 'header' );
				},
				'selector' => '.nebotheme-logo--header',
				'settings' => [ self::CUSTOM_LOGO_TM ],
			]
		);
	}

	/**
	 * Add fields to customize.
	 */
	public function customize_add_fields() {
		$custom_logo_args = get_theme_support( 'custom-logo' );
		$field_args = [
			'height' => isset( $custom_logo_args[0]['height'] ) ? $custom_logo_args[0]['height'] : null,
			'width' => isset( $custom_logo_args[0]['width'] ) ? $custom_logo_args[0]['width'] : null,
			'flex_height' => isset( $custom_logo_args[0]['flex-height'] ) ? $custom_logo_args[0]['flex-height'] : null,
			'flex_width' => isset( $custom_logo_args[0]['flex-width'] ) ? $custom_logo_args[0]['flex-width'] : null,
			'button_labels' => self::get_custom_logo_button_labels(),
		];

		Customize_Manager::add_section(
			'logos',
			[
				'title' => esc_html__( 'Logos', 'yaffo' ),
				'priority' => 5,
				'icon' => 'dashicons-art',
			]
		);

		foreach ( self::get_logo_types() as $logo_type => $logo_data ) {
			$partial_refresh = [
				'selector' => ".nebotheme-logo--{$logo_type}",
				'container_inclusive' => true,
				'render_callback' => function() use ( $logo_type ) {
					self::render_logo( $logo_type );
				},
			];

			Customize_Manager::add_field(
				[
					'label' => $logo_data['label'],
					'type' => 'nebotheme-popover-start',
					'section' => 'logos',
				]
			);

			if ( self::CUSTOM_LOGO_TM !== $logo_data['theme_mod'] ) {
				Customize_Manager::add_field(
					array_merge(
						$field_args,
						[
							'type' => 'cropped_image',
							'label' => esc_html__( 'Logo', 'yaffo' ),
							'settings' => $logo_data['theme_mod'],
							'section' => 'logos',
							'partial_refresh' => [
								$logo_data['theme_mod'] => $partial_refresh,
							],
						]
					)
				);
			}

			Customize_Manager::add_field(
				array_merge(
					$field_args,
					[
						'type' => 'cropped_image',
						'label' => esc_html__( 'Logo Retina', 'yaffo' ),
						'settings' => "{$logo_data['theme_mod']}_retina",
						'section' => 'logos',
						'partial_refresh' => [
							"{$logo_data['theme_mod']}_retina" => $partial_refresh,
						],
					]
				)
			);

			Customize_Manager::add_field(
				[
					'type' => 'nebotheme-popover-end',
					'section' => 'logos',
				]
			);
		}
	}
}
