<?php
/**
 * General theme class.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo;

use NeboTheme\Yaffo\Admin\Admin;
use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Customize\Customize;
use NeboTheme\Yaffo\Customize\Theme_Fonts;
use NeboTheme\Yaffo\Customize\Font_Printer;
use NeboTheme\Yaffo\Headline;
use NeboTheme\Yaffo\Blog_Main;
use NeboTheme\Yaffo\Homepage;
use NeboTheme\Yaffo\Logos;
use NeboTheme\Yaffo\Menu\Menu_Manager;
use NeboTheme\Yaffo\Post\Post;
use NeboTheme\Yaffo\Sidebars\Sidebar_Manager;
use NeboTheme\Yaffo\Sidebars\Widgets_Manager;
use NeboTheme\Yaffo\ThemeData\Default_Manager;
use NeboTheme\Yaffo\Traits\Singleton;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * General theme class
 */
final class Theme {
	use Singleton;

	const URI_JS = YAFFO_URI_ASSETS . '/js';
	const URI_CSS = YAFFO_URI_ASSETS . '/css';
	const URI_LIB = YAFFO_URI_ASSETS . '/lib';

	/**
	 * If set 'SCRIPT_DEBUG' then is '.min'.
	 *
	 * @var string
	 */
	private static $suffix;

	/**
	 * Sidebar Manager.
	 *
	 * @var Sidebar_Manager
	 */
	public $sidebar_manager;

	/**
	 * Get common data (etc frontend, admin, customize)
	 */
	public static function get_config_common() {
		return apply_filters( 'nebotheme_config_common', [
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		] );
	}

	/**
	 * Get suffix value.
	 */
	public static function get_suffix_assets() {
		return static::$suffix;
	}

	/**
	 * Get url to css file.
	 *
	 * Considering rtl
	 */
	public static function get_style_url( $dirname, $url = self::URI_CSS, $filename = 'style' ) {
		$suffix = static::get_suffix_assets();
		$direction_suffix = is_rtl() ? '-rtl' : '';

		$url = trailingslashit( $url );

		return "{$url}{$dirname}{$direction_suffix}/{$filename}{$suffix}.css";
	}

	/**
	 * The class constructor
	 */
	private function __construct() {
		$this->init_suffix();
		$this->require_files();
		$this->add_hooks();
		$this->init_class();
	}

	/**
	 * Set suffix property
	 */
	public function init_suffix() {
		self::$suffix = defined( 'SCRIPT_DEBUG' ) && constant( 'SCRIPT_DEBUG' ) ? '' : '.min';
	}

	/**
	 * Method for activation actions and filters.
	 */
	private function add_hooks() {
		add_action( 'wp_enqueue_scripts', [ $this, 'wp_enqueue_scripts' ] );
		add_action( 'after_setup_theme', [ $this, 'after_setup_theme' ] );
		add_action( 'wp_head', [ $this, 'wp_head_icon_preloader' ] );

		add_filter( 'nebotheme_config_frontend', [ $this, 'get_values_for_frontend' ] );
		add_filter( 'wxr_export_skip_postmeta', [ $this, 'on_export_post_meta' ], 10, 2 );
		add_filter( 'image_size_names_choose', [ $this, 'custom_image_size_names_choose' ] );

		// Disables the block editor from managing widgets in the Gutenberg plugin.
		add_filter( 'gutenberg_use_widgets_block_editor', '__return_false', 100 );
		// Disables the block editor from managing widgets.
		add_filter( 'use_widgets_block_editor', '__return_false' );
	}

	/**
	 * Add custom image sizes.
	 */
	public function custom_image_size_names_choose( array $sizes ) {
		$custom_image_choose = nebotheme_wp_list_pluck( nebotheme_get_theme_image_sizes(), 'label' );

		return array_merge( $sizes, $custom_image_choose );
	}

	/**
	 * Skip metabox field for export.
	 */
	public function on_export_post_meta( $skip, $meta_key ) {
		$to_skip = apply_filters( 'nebotheme_export_skip_post_meta', [] );

		if ( in_array( $meta_key, $to_skip, true ) ) {
			$skip = true;
		}

		return $skip;
	}

	/**
	 * Get data for frontend.
	 */
	public function get_values_for_frontend( $config ) {
		$theme_mods = [];

		foreach ( Customize_Manager::get_fields() as $field ) {
			if ( ! Utils::get_if_isset( $field, 'on_frontend' ) ) {
				continue;
			}

			$theme_mod = nebotheme_get_tm( $field['settings'] );

			if ( $theme_mod ) {
				$theme_mods[ $field['settings'] ] = $theme_mod;
			}
		}

		if ( ! empty( $theme_mods ) ) {
			$config['theme_mods'] = $theme_mods;
		}

		return $config;
	}

	/**
	 * Register support for various WordPress features.
	 */
	public function after_setup_theme() {
		global $content_width;

		if ( ! isset( $content_width ) ) {
			$content_width = yaffo_get_environment()['width_content'];
		}

		load_theme_textdomain( 'yaffo', YAFFO_DIR_ROOT . '/languages' );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		// Add theme support for selective refresh for widgets.
		add_theme_support( 'customize-selective-refresh-widgets' );
		add_theme_support( 'post-formats', [ 'gallery', 'video' ] );

		// Gutenberg responsive embed support.
		add_theme_support( 'responsive-embeds' );
		/**
		 * Let WordPress manage the document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in the document head, and expect WordPress to
		 * provide it for us.
		 */
		add_theme_support( 'title-tag' );

		// Custom logo support.
		add_theme_support(
			'custom-logo',
			[
				'flex-width' => true,
				'flex-height' => true,
			]
		);

		/**
		 * Switch default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 */
		add_theme_support(
			'html5', [
				'caption',
				'comment-form',
				'comment-list',
				'gallery',
				'script',
				'search-form',
				'style',
			]
		);

		add_theme_support( 'post-thumbnails' );

		foreach ( nebotheme_get_theme_image_sizes() as $image_size_name => $image_size_data ) {
			add_image_size(
				$image_size_name,
				Utils::get_if_isset( $image_size_data, 'width', $content_width ),
				Utils::get_if_isset( $image_size_data, 'height', 0 ),
				Utils::get_if_isset( $image_size_data, 'crop', false )
			);
		}

		remove_theme_support( 'widgets-block-editor' );
	}

	/**
	 * Require files.
	 */
	private function require_files() {
		require_once YAFFO_DIR_FRAMEWORK . '/functions/general-functions.php';
		require_once YAFFO_DIR_FRAMEWORK . '/functions/environments.php';
	}

	/**
	 * Instance classes.
	 */
	private function init_class() {
		Default_Manager::instance();

		Admin::instance();
		Customize::instance();
		Theme_Fonts::instance();
		Font_Printer::instance();
		Headline::instance();
		Homepage::instance();
		Logos::instance();
		Menu_Manager::instance();
		Post::instance();
		Sidebar_Manager::instance();
		Widgets_Manager::instance();
		Blog_Main::instance();
	}

	/**
	 * Handles JavaScript detection.
	 */
	private function enqueue_script() {
		$suffix = static::get_suffix_assets();
		$version = wp_get_theme( 'yaffo' )->version;

		/* < Libs */
		wp_register_script(
			'yaffo-libs',
			self::URI_JS . '/frontend/libs.min.js',
			[ 'jquery' ],
			$version,
			true
		);
		/* Libs > */

		wp_register_script(
			'yaffo',
			self::URI_JS . "/frontend/script{$suffix}.js",
			[
				'jquery',
				'yaffo-libs',
			],
			$version,
			true
		);

		$config = [];

		$config = apply_filters( 'nebotheme_config_frontend', $config );
		$config_common = self::get_config_common();

		wp_localize_script( 'yaffo', 'nebothemeConfig', $config );
		wp_localize_script( 'yaffo', 'nebothemeCommonConfig', $config_common );

		wp_enqueue_script( 'yaffo' );

		if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
			wp_enqueue_script( 'comment-reply' );
		}

		wp_register_script( 'yaffo-inline', false, [], wp_get_theme()->get( 'Version' ), false );
		wp_enqueue_script( 'yaffo-inline' );
		wp_add_inline_script( 'yaffo-inline', "
			/^((?!chrome|android).)*safari/i.test( navigator.userAgent ) && document.documentElement.classList.add( 'is-browser-safari' );

			if (location.pathname.match(/\/page\//)) {
				window.addEventListener('load', function() {
					var blogElement = document.getElementById('blog');

					if ( ! blogElement ) {
						return;
					}

					var blogRect = blogElement.getBoundingClientRect();
					var viewHeight = Math.max(document.documentElement.clientHeight, window.innerHeight);

					if ( ! ( blogRect.top - viewHeight >= 0 ) ) {
						return;
					}

					blogElement.scrollIntoView({
						behavior: 'smooth',
						block: 'start',
						duration: 200
					});
				});
			}

		" );
	}

	/**
	 * Add a preload for icons.
	 */
	public function wp_head_icon_preloader() {
		echo '<link rel="preload" as="font" href="' . esc_url( self::URI_LIB . '/nebotheme-icon/font/nebotheme-icon.woff2' ) . '" crossorigin>';
	}

	/**
	 * Handles CSS detection.
	 */
	private function enqueue_style() {
		$version = wp_get_theme( 'yaffo' )->version;

		wp_enqueue_style(
			'yaffo',
			static::get_style_url( 'frontend' ),
			[],
			$version,
			'screen, print'
		);

		wp_enqueue_style(
			'nebotheme-icon',
			self::URI_LIB . '/nebotheme-icon/css/nebotheme-icon.css',
			[],
			$version,
			'screen, print'
		);

		wp_add_inline_style( 'yaffo', ':root { --nebotheme-svg-select-arrow: url(\'data:image/svg+xml;utf8,<svg width="13" height="7" viewBox="0 0 13 7" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M1 1L6.5 6L12 1" stroke="' . esc_attr( str_replace( '#', '%23', nebotheme_get_color_general( 'headings' ) ) ) . '"/></svg>\'); }' );
	}

	/**
	 * Handles JavaScript and CSS detection.
	 */
	public function wp_enqueue_scripts() {
		$this->enqueue_script();
		$this->enqueue_style();
	}
}

Theme::instance();
