<?php
/**
 * Utils class.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Helpers functions in class Utils.
 */
final class Utils {
	/**
	 * Get WP_Filesystem object
	 *
	 * @return WP_Filesystem_Base|false
	 */
	public static function get_wp_filesystem() {
		global $wp_filesystem;

		if ( empty( $wp_filesystem ) ) {
			require_once ABSPATH . '/wp-admin/includes/file.php';

			WP_Filesystem();
		}

		if ( ! $wp_filesystem ) {
			return false;
		}

		return $wp_filesystem;
	}

	/**
	 * Get merged array.
	 *
	 * Replaces values are excluded.
	 */
	public static function array_merge_safe( array $array1, array $array2 ) {
		return array_merge(
			/* For false value */
			array_merge(
				array_filter(
					$array2,
					function( $value ) {
						return ! $value;
					}
				),
				array_filter(
					$array1,
					function( $value ) {
						return ! $value;
					}
				)
			),
			/* For true value */
			array_merge(
				array_filter( $array2 ),
				array_filter( $array1 )
			)
		);
	}

	/**
	 * Check if on edit page.
	 */
	public static function is_edit() {
		global $pagenow;

		if ( ! is_admin() ) {
			return false;
		}

		return in_array( $pagenow, [ 'post.php', 'post-new.php' ], true );
	}

	/**
	 * Retrieve a piece of text
	 */
	public static function get_piece_text( $pattern, $text ) {
		preg_match_all( $pattern, $text, $matches );

		if ( ! empty( $matches[1][0] ) ) {
			return $matches[1][0];
		}

		return '';
	}

	/**
	 * Get main category.
	 */
	public static function get_default_category() {
		return (int) get_option( 'default_category', 1 );
	}

	/**
	 * Safe retrieve array element.
	 */
	public static function get_if_isset( $haystack, $key, $fail = false ) {
		if ( null !== $key && false !== $key && is_array( $haystack ) && isset( $haystack[ $key ] ) ) {
			return $haystack[ $key ];
		}

		if ( is_callable( $fail ) ) {
			return call_user_func( $fail );
		}

		return $fail;
	}

	/**
	 * Gets information about all registered image sizes.
	 *
	 * @global $_wp_additional_image_sizes
	 * @uses get_intermediate_image_sizes()
	 *
	 * @param bool $unset_disabled Remove dimensions with 0 height and width from the list?.
	 *
	 * @return array Data of all sizes.
	 */
	public static function get_image_sizes( $unset_disabled = true ) {
		$additional_image_sizes = & $GLOBALS['_wp_additional_image_sizes'];

		$sizes = [];

		foreach ( get_intermediate_image_sizes() as $size_name ) {
			if ( in_array( $size_name, [ 'thumbnail', 'medium', 'medium_large', 'large' ], true ) ) {
				$sizes[ $size_name ] = [
					'width' => get_option( "{$size_name}_size_w" ),
					'height' => get_option( "{$size_name}_size_h" ),
					'crop' => (bool) get_option( "{$size_name}_crop" ),
				];
			} elseif ( isset( $additional_image_sizes[ $size_name ] ) ) {
				$sizes[ $size_name ] = [
					'width' => $additional_image_sizes[ $size_name ]['width'],
					'height' => $additional_image_sizes[ $size_name ]['height'],
					'crop' => $additional_image_sizes[ $size_name ]['crop'],
				];
			}

			// size registered, but has 0 width and height.
			if ( $unset_disabled && 0 === $sizes[ $size_name ]['width'] && 0 === $sizes[ $size_name ]['height'] ) {
				unset( $sizes[ $size_name ] );
			}
		}

		return $sizes;
	}

	/**
	 * Gets data of a specific image size.
	 *
	 * @uses get_image_sizes()
	 * @param string $size_name Size name.
	 *
	 * @return bool|array $size_name Image size data or false if not.
	 */
	public static function get_image_size( $size_name ) {
		return static::get_if_isset( self::get_image_sizes( 0 ), $size_name );
	}

	/**
	 * Check if array is associative.
	 *
	 * @return bool.
	 */
	public static function is_array_sequential( array $array ) {
		return array_values( $array ) === $array;
	}

	/**
	 * Get generated html.
	 */
	public static function get_ob_html( callable $callback ) {
		if ( ! is_callable( $callback ) ) {
			return '';
		}

		ob_start();

		call_user_func( $callback );

		return ob_get_clean();
	}

	/**
	 * Add https to string url.
	 *
	 * @param string $url
	 */
	public static function add_https( $url ) {
		if ( ! preg_match( '~^(?:f|ht)tps?://~i', $url ) ) {
			$url = 'https://' . $url;
		}

		return $url;
	}

	/**
	 * Check if request is ajax.
	 *
	 * Whether the current request is a WordPress ajax request.
	 *
	 * @return bool True if it's a WordPress ajax request, false otherwise.
	 */
	public static function is_ajax() {
		if ( function_exists( 'wp_doing_ajax' ) ) {
			return wp_doing_ajax();
		}

		return defined( 'DOING_AJAX' ) && DOING_AJAX;
	}


	/**
	 * Array Merge.
	 *
	 * @return array
	 */
	public static function array_merge( $first, $second ) {
		$result = [];

		foreach ( $first as $key => $value ) {
			$result[ $key ] = $value;
		}

		foreach ( $second as $key => $value ) {
			$result[ $key ] = $value;
		}

		return $result;
	}

	public static function get_number_shorten( $n, $precision = 1 ) {
		if ( $n < 900 ) {
			// 0 - 900
			$n_format = number_format( $n, $precision );
			$suffix = '';
		} elseif ( $n < 900000 ) {
			// 0.9k-850k
			$n_format = number_format( $n / 1000, $precision );
			$suffix = 'K';
		} elseif ( $n < 900000000 ) {
			// 0.9m-850m
			$n_format = number_format( $n / 1000000, $precision );
			$suffix = 'M';
		} elseif ( $n < 900000000000 ) {
			// 0.9b-850b
			$n_format = number_format( $n / 1000000000, $precision );
			$suffix = 'B';
		} else {
			// 0.9t+
			$n_format = number_format( $n / 1000000000000, $precision );
			$suffix = 'T';
		}
		// Remove unecessary zeroes after decimal. "1.0" -> "1"; "1.00" -> "1"
		// Intentionally does not affect partials, eg "1.50" -> "1.50"
		if ( $precision > 0 ) {
			$dotzero = '.' . str_repeat( '0', $precision );
			$n_format = str_replace( $dotzero, '', $n_format );
		}
		return $n_format . $suffix;
	}
}
