<?php
/**
 * Wrapper Class for kirki which also acts as a fallback for CSS generation
 * when the kirki plugin is disabled.
 *
 * @package Yaffo WordPress theme
 */

namespace NeboTheme\Yaffo\Customize;

use NeboTheme\Yaffo\Customize\Customize;
use NeboTheme\Yaffo\Customize\Theme_Fonts;
use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Utils;
use NeboTheme\Yaffo\Customize\Font_Printer;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * This is a wrapper class for Kirki.
 * If the Kirki plugin is installed, then all CSS & Google fonts
 * will be handled by the plugin.
 * In case the plugin is not installed, this acts as a fallback
 * ensuring that all CSS & fonts still work.
 * It does not handle the customize options, simply the frontend CSS.
 */
final class Customize_Manager {
	use Singleton;

	const CONTROLS_READONLY_SETTINGS = [
		'nebotheme-popover-start',
		'nebotheme-popover-end',
		'nebotheme-button',
	];

	/**
	 * The config ID.
	 *
	 * @static
	 * @access protected
	 * @var array
	 */
	protected static $config = [];

	/**
	 * An array of all our fields.
	 *
	 * @static
	 * @access protected
	 * @var array
	 */
	protected static $fields = [];

	/**
	 * The class constructor
	 */
	private function __construct() {
		// If Kirki exists then there's no reason to proceed.
		if ( class_exists( '\Kirki' ) ) {
			return;
		}

		// Add our CSS.
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_styles' ], 20 );
		add_filter( 'admin_enqueue_scripts', [ $this, 'gutenberg_editor' ], 20 );

		// Add google fonts.
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_fonts' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_fonts' ] );
	}

	/**
	 * Get the value of an option from the db.
	 *
	 * @param string $config_id The ID of the configuration corresponding to this field.
	 * @param string $field_id  The field_id (defined as 'settings' in the field arguments).
	 * @return mixed            The saved value of the field.
	 */
	public static function get_option( $config_id = '', $field_id = '', $default_fail = null ) {
		$field = self::get_field( $field_id );
		$field_default = Utils::get_if_isset( $field, 'default' );
		$return_value = $field_default;

		// if Kirki exists, use it.
		if ( class_exists( '\Kirki' ) ) {
			$return_value = \Kirki::get_option( $config_id, $field_id );
		} elseif ( $field && null === $default_fail && $field_default ) {
			// Kirki does not exist, continue with our custom implementation.
			// Get the default value of the field.
			$default_fail = $field_default;
		} elseif ( isset( self::$config[ $config_id ] ) ) {
			// Make sure the config is defined.
			if ( 'option' === self::$config[ $config_id ]['option_type'] ) {

				// check if we're using serialized options.
				if ( isset( self::$config[ $config_id ]['option_name'] ) && ! empty( self::$config[ $config_id ]['option_name'] ) ) {

					// Get all our options.
					$all_options = get_option( self::$config[ $config_id ]['option_name'], [] );

					// If our option is not saved, return the default value.
					// If option was set, return its value unserialized.
					$return_value = ( ! isset( $all_options[ $field_id ] ) ) ? $default_fail : maybe_unserialize( $all_options[ $field_id ] );
				}

				// If we're not using serialized options, get the value and return it.
				// We'll be using a dummy default here to check if the option has been set or not.
				// We'll be using md5 to make sure it's randomish and impossible to be actually set by a user.
				$dummy = md5( $config_id . '_UNDEFINED_VALUE' );
				$value = get_option( $field_id, $dummy );

				// setting has not been set, return default.
				$return_value = ( $dummy === $value ) ? $default_fail : $value;
			}

			// We're not using options so fallback to theme_mod.
			$return_value = get_theme_mod( $field_id, $default_fail );
		}

		if (
			$field &&
			'typography' === $field['type'] &&
			is_array( $return_value ) &&
			is_array( $field_default )
		) {
			$return_value = array_merge( $field_default, $return_value );
		}

		return $return_value;
	}

	/**
	 * Create a new panel.
	 *
	 * @param string $id   The ID for this panel.
	 * @param array  $args The panel arguments.
	 */
	public static function add_panel( $id = '', $args = [] ) {
		if ( class_exists( '\Kirki' ) ) {
			\Kirki::add_panel( $id, $args );
		}
		/* If Kirki does not exist then there's no reason to add any panels. */
	}

	/**
	 * Create a new section.
	 *
	 * @param string $id   The ID for this section.
	 * @param array  $args The section arguments.
	 */
	public static function add_section( $id, array $args ) {
		if ( class_exists( '\Kirki' ) ) {
			\Kirki::add_section( $id, $args );
		}
		/* If Kirki does not exist then there's no reason to add any sections. */
	}

	/**
	 * Sets the configuration options.
	 *
	 * @param string $config_id The configuration ID.
	 * @param array  $args      The configuration arguments.
	 */
	public static function add_config( $config_id, $args = [] ) {

		// if Kirki exists, use it.
		if ( class_exists( '\Kirki' ) ) {
			\Kirki::add_config( $config_id, $args );
			return;
		}

		// Kirki does not exist, set the config arguments.
		self::$config[ $config_id ] = $args;

		// Make sure an option_type is defined.
		if ( ! isset( self::$config[ $config_id ]['option_type'] ) ) {
			self::$config[ $config_id ]['option_type'] = 'theme_mod';
		}
	}

	/**
	 * Create a new field
	 *
	 * @param array $args The field's arguments.
	 *
	 * @return void
	 */
	public static function add_field( $args ) {
		// < Filter.
		/* < field without settings < */
		if ( 'nebotheme-button' === $args['type'] && empty( $args['settings'] ) ) {
			$args['settings'] = $args['event'];
		}

		if ( // For control readonly.
			in_array( $args['type'], self::CONTROLS_READONLY_SETTINGS, true ) &&
			empty( $args['settings'] )
		) {
			static $settings_dynamic = 0;

			$args['settings'] = join( '_', [ $args['type'], $settings_dynamic ] );

			$settings_dynamic++;
		}
		/* > field without settings > */

		// Set default by name from Default Data.
		if ( empty( $args['default'] ) && ! empty( $args['settings'] ) ) {
			$default = nebotheme_get_tm_default( $args['settings'], $args['type'] );

			if ( null !== $default ) {
				$args['default'] = $default;
			}
		}

		if ( 'color' === $args['type'] && empty( $args['choices'] ) ) {
			$args['choices'] = [
				'palettes' => nebotheme_get_color_general_default_palette(),
				'alpha' => true,
			];
		} elseif ( 'multicolor' === $args['type'] ) {
			// 'default' can not be empty
			if (
				! empty( $args['choices'] ) &&
				(
					empty( $args['default'] ) ||
					(
						is_array( $args['choices'] ) &&
						is_array( $args['default'] ) &&
						count( array_keys( $args['choices'] ) ) !== count( array_keys( $args['default'] ) )
					)
				)
			) {
				$default_empty = array_combine(
					array_keys( $args['choices'] ),
					array_fill( 0, count( $args['choices'] ), '' )
				);
				$default = Utils::get_if_isset( $args, 'default', [] );

				$args['default'] = array_merge( $default_empty );
			}

			// Set palettes.
			if (
				! empty( $args['default'] ) &&
				empty( $args['choices']['irisArgs'] )
			) {
				$palettes = array_values( array_filter( $args['default'] ) );

				if ( empty( $palettes ) || count( $palettes ) < 4 ) {
					$palettes = nebotheme_get_color_palette();
				}

				$args['choices']['irisArgs']['palettes'] = $palettes;
			}

			/* 'choices' to 'output' for css-var */
			if (
				isset( $args['output_by_choices'] ) &&
				empty( $args['output'] ) &&
				! empty( $args['choices'] )
			) {
				$choices = array_filter(
					$args['choices'],
					function( $choice ) {
						return 'irisArgs' !== $choice;
					},
					ARRAY_FILTER_USE_KEY
				);

				$args['output'] = array_map(
					function( $name ) use ( $args ) {
							$output_item = $args['output_by_choices'];

							$output_item['property'] = str_replace( '{{NAME}}', $name, $output_item['property'] );
							$output_item['choice'] = $name;

							return $output_item;
					},
					array_keys( $choices ),
					$choices
				);
			}
		} elseif ( 'typography' === $args['type'] ) {
			if ( ! empty( $args['css_var_selector'] ) ) {
				$args['transport'] = 'postMessage';

				Font_Printer::instance()->add_to_print( $args );
			}

			/* Because font fields by default field */
			if (
				! empty( $args['font_types'] ) &&
				! empty( $args['default'] ) &&
				is_array( $args['default'] )
			) {
				$args['default'] = array_filter(
					$args['default'],
					function( $key ) use ( $args ) {
						return in_array( $key, $args['font_types'], true );
					},
					ARRAY_FILTER_USE_KEY
				);
			}

			$args['choices'] = Theme_Fonts::get_fonts_choices();
		}

		/* Dynamic 'transport' */
		if (
			empty( $args['transport'] ) &&
			(
				! empty( $args['output'] ) ||
				'typography' === $args['type'] ||
				'repeater' === $args['type']
			)
		) {
			$args['transport'] = 'auto';
		}
		// > Filter
		// if Kirki exists, use it.
		if ( class_exists( '\Kirki' ) ) {
			\Kirki::add_field( Customize::KIRKI_CONFIG_ID, $args );
		}

		// Kirki was not located, so we'll need to add our fields here.
		// Check that the "settings" & "type" arguments have been defined.
		if ( isset( $args['settings'] ) && isset( $args['type'] ) ) {

			// Make sure we add the config_id to the field itself.
			// This will make it easier to get the value when generating the CSS later.
			if ( ! isset( $args['kirki_config'] ) ) {
				$args['kirki_config'] = Customize::KIRKI_CONFIG_ID;
			}

			self::$fields[ $args['settings'] ] = $args;
		}
	}

	/**
	 * Add Field trigger.
	 */
	public static function add_field_trigger( array $args ) {
		static $index = 0;

		$args_default = [
			'label' => '',
			'trigger' => '',
			'settings' => "control-trigger-{$index}",
		];

		$args = array_merge( $args_default, $args );

		$args['type'] = 'custom';

		$args['default'] = '<a data-customize-trigger="' . esc_attr( $args['trigger'] ) . '" class="nebotheme-section-trigger">' .
			'<h3 class="accordion-section-title">' .
				esc_html( $args['label'] ) .
			'</h3>' .
		'</a>';

		unset( $args['label'] );

		static::add_field( $args );

		$index++;
	}

	/**
	 * Enqueue the stylesheet.
	 *
	 * @access public
	 * @return null
	 */
	public function enqueue_styles() {
		// If Kirki exists there's no need to proceed any further.
		if ( class_exists( '\Kirki' ) || is_admin() ) {
			return;
		}

		// Get our inline styles.
		$styles = $this->get_styles();

		// If we have some styles to add, add them now.
		if ( ! empty( $styles ) ) {
			wp_register_style( 'yaffo-colors', false, [], wp_get_theme()->get( 'Version' ) );
			wp_enqueue_style( 'yaffo-colors' );
			wp_add_inline_style( 'yaffo-colors', $styles );
		}
	}

	/**
	 * Enqueue styles to Gutenberg Editor.
	 *
	 * @access public
	 * @return null
	 */
	public function gutenberg_editor() {
		// If Kirki exists there's no need to proceed any further.
		if ( class_exists( '\Kirki' ) || ! is_admin() || is_customize_preview() ) {
			return;
		}

		if ( is_admin() && ! is_customize_preview() ) {
			// Get our inline styles.
			$styles = $this->get_styles();

			// If we have some styles to add, add them now.
			if ( ! empty( $styles ) ) {
				wp_add_inline_style( 'nebotheme-gutenberg-editor', $styles );
			}
		}
	}

	/**
	 * Gets all our styles and returns them as a string.
	 *
	 * @access public
	 * @return string
	 */
	public function get_styles() {
		// Check if we need to exit early.
		if ( empty( self::$fields ) || ! is_array( self::$fields ) ) {
			return '';
		}

		// Initially we're going to format our styles as an array.
		// This is going to make processing them a lot easier
		// and make sure there are no duplicate styles etc.
		$css = [];

		// Start parsing our fields.
		foreach ( self::$fields as $field ) {

			// No need to process fields without an output, or an improperly-formatted output.
			if ( ! isset( $field['output'] ) || empty( $field['output'] ) || ! is_array( $field['output'] ) ) {
				continue;
			}

			// Get the value of this field.
			$value = static::get_option( $field['kirki_config'], $field['settings'] );

			// Start parsing the output arguments of the field.
			foreach ( $field['output'] as $output ) {
				$output = wp_parse_args(
					$output,
					[
						'element' => '',
						'property' => '',
						'media_query' => 'global',
						'prefix' => '',
						'units' => '',
						'suffix' => '',
						'value_pattern' => '$',
						'choice' => '',
					]
				);

				// If element is an array, convert it to a string.
				if ( is_array( $output['element'] ) ) {
					$output['element'] = implode( ',', $output['element'] );
				}

				// Simple fields.
				if ( ! is_array( $value ) ) {
					$value = str_replace( '$', $value, $output['value_pattern'] );

					if ( isset( $output['pattern_replace'] ) && is_array( $output['pattern_replace'] ) ) {
						foreach ( $output['pattern_replace'] as $search => $replace ) {
							$replacement = nebotheme_get_tm( $replace );

							$replacement = ( false === $replacement ) ? '' : $replacement;
							if ( is_array( $value ) ) {
								foreach ( $value as $k => $v ) {
									$_val = ( isset( $value[ $v ] ) ) ? $value[ $v ] : $v;
									$value[ $k ] = str_replace( $search, $replacement, $_val );
								}
								return $value;
							}
							$value = str_replace( $search, $replacement, $value );
						}
					}

					if ( ! empty( $output['element'] ) && ! empty( $output['property'] ) && ( $value || '0' === $value ) ) {
						$css[ $output['media_query'] ][ $output['element'] ][ $output['property'] ] = $output['prefix'] . $value . $output['units'] . $output['suffix'];
					}
				} else {
					if ( 'typography' === $field['type'] ) {

						foreach ( $value as $key => $sub_value ) {

							// Add double quotes if needed to font-families.
							if ( 'font-family' === $key && false !== strpos( $sub_value, ' ' ) && false === strpos( $sub_value, '"' ) ) {
								$css[ $output['media_query'] ][ $output['element'] ]['font-family'] = '"' . $sub_value . '"';
							}

							// Variants contain both font-weight & italics.
							if ( 'variant' === $key ) {
								$font_weight = str_replace( 'italic', '', $sub_value );
								$font_weight = ( in_array( $font_weight, [ '', 'regular' ], true ) ) ? '400' : $font_weight;
								$css[ $output['media_query'] ][ $output['element'] ]['font-weight'] = $font_weight;

								// Is this italic?
								if ( false !== strpos( $sub_value, 'italic' ) ) {
									$css[ $output['media_query'] ][ $output['element'] ]['font-style'] = 'italic';
								}
							} else {
								$css[ $output['media_query'] ][ $output['element'] ][ $key ] = $sub_value;
							}
						}
					} elseif ( 'multicolor' === $field['type'] ) {
						if ( ! empty( $output['element'] ) && ! empty( $output['property'] ) && ! empty( $output['choice'] ) ) {
							$css[ $output['media_query'] ][ $output['element'] ][ $output['property'] ] = $output['prefix'] . $value[ $output['choice'] ] . $output['units'] . $output['suffix'];
						}
					} else {

						foreach ( $value as $key => $sub_value ) {
							$property = $key;
							if ( false !== strpos( $output['property'], '%%' ) ) {
								$property = str_replace( '%%', $key, $output['property'] );
							} elseif ( ! empty( $output['property'] ) ) {
								$output['property'] = $output['property'] . '-' . $key;
							}
							if ( 'background-image' === $output['property'] && false === strpos( $sub_value, 'url(' ) ) {
								$sub_value = 'url("' . set_url_scheme( $sub_value ) . '")';
							}
							if ( $sub_value ) {
								$css[ $output['media_query'] ][ $output['element'] ][ $property ] = $sub_value;
							}
						}
					}
				}
			}
		}

		// Process the array of CSS properties and produce the final CSS.
		$final_css = '';
		if ( ! is_array( $css ) || empty( $css ) ) {
			return '';
		}

		// Parse the generated CSS array and create the CSS string for the output.
		foreach ( $css as $media_query => $styles ) {

			// Handle the media queries.
			$final_css .= ( 'global' !== $media_query ) ? $media_query . '{' : '';
			foreach ( $styles as $style => $style_array ) {

				$final_css .= $style . '{';
				foreach ( $style_array as $property => $value ) {
					if ( ! $value ) {
						continue;
					}

					$value = ( is_string( $value ) ) ? $value : '';

					// Make sure background-images are properly formatted.
					if ( 'background-image' === $property ) {
						if ( false === strrpos( $value, 'url(' ) ) {
							$value = set_url_scheme( $value );
							$value = 'url("' . esc_url_raw( $value ) . '")';
						}
					} else {
						$value = esc_textarea( $value );
					}
					$final_css .= $property . ':' . $value . ';';
				}
				$final_css .= '}';
			}
			$final_css .= ( 'global' !== $media_query ) ? '}' : '';
		}
		return $final_css;
	}

	/**
	 * Enqueue google fonts.
	 *
	 * @access public
	 * @return null
	 */
	public function enqueue_fonts() {
		// Check if we need to exit early.
		if ( empty( self::$fields ) || ! is_array( self::$fields ) ) {
			return;
		}

		foreach ( self::$fields as $field ) {

			// Process typography fields.
			if ( isset( $field['type'] ) && 'typography' === $field['type'] ) {

				// Check if we've got everything we need.
				if ( ! isset( $field['kirki_config'] ) || ! isset( $field['settings'] ) ) {
					continue;
				}

				$value = static::get_option( $field['kirki_config'], $field['settings'] );

				if (
					! empty( $value['font-family'] ) &&
					'inherit' !== $value['font-family'] &&
					! isset( Theme_Fonts::$theme_fonts[ $value['font-family'] ] )
				) {
					$url = '//fonts.googleapis.com/css?family=' . str_replace( ' ', '+', $value['font-family'] );
					$value['variant'] = ( isset( $value['variant'] ) ) ? $value['variant'] : '';
					$url             .= ( empty( $value['variant'] ) ) ? '' : ':' . $value['variant'];

					$key = md5( $value['font-family'] . $value['variant'] );

					// Check that the URL is valid. we're going to use transients to make this faster.
					$url_is_valid = get_transient( $key );

					/**
					 * Reset the cache if we're using action=kirki-reset-cache in the URL.
					 *
					 * Note to code reviewers:
					 * There's no need to check nonces or anything else, this is a simple true/false evaluation.
					 */
					if ( ! empty( $_GET['action'] ) && 'kirki-reset-cache' === $_GET['action'] ) { // phpcs:ignore WordPress.Security.NonceVerification
						$url_is_valid = false;
					}

					// If transient does not exist.
					if ( false === $url_is_valid ) {
						$response = wp_remote_get( 'https:' . $url );

						if ( ! is_array( $response ) ) {

							// The url was not properly formatted,
							// cache for 12 hours and continue to the next field.
							set_transient( $key, null, 12 * constant( 'HOUR_IN_SECONDS' ) );
							continue;
						}

						// Check the response headers.
						if ( isset( $response['response'] ) && isset( $response['response']['code'] ) ) {
							if ( 200 === $response['response']['code'] ) {

								// URL was ok. Set transient to true and cache for an hour.
								set_transient( $key, true, 7 * 24 * constant( 'HOUR_IN_SECONDS' ) );
								$url_is_valid = true;
							}
						}
					}

					// If the font-link is valid, enqueue it.
					if ( $url_is_valid ) {
						wp_enqueue_style( $key, $url, [], '1.0' );
					}
				}
			}
		}
	}

	/**
	 * Get all fields.
	 */
	public static function get_fields() {
		return self::$fields;
	}

	/**
	 * Get field by id
	 */
	public static function get_field( $field_id, $arg_key = '', $fail = false ) {
		if ( $arg_key ) {
			if ( isset( self::$fields[ $field_id ]['args'][ $arg_key ] ) ) {
				return self::$fields[ $field_id ]['args'][ $arg_key ];
			} elseif ( isset( self::$fields[ $field_id ][ $arg_key ] ) ) {
				return self::$fields[ $field_id ][ $arg_key ];
			}
		} elseif ( ! $arg_key && isset( self::$fields[ $field_id ] ) ) {
			return self::$fields[ $field_id ];
		}

		return $fail;
	}
}

Customize_Manager::instance();
