<?php
/**
 * NeboTheme Customize Class
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Customize;

use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Theme;
use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Utils;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Customize class
 */
final class Customize {
	use Singleton;

	const KIRKI_CONFIG_ID = 'yaffo_config';

	/**
	 * The class constructor
	 */
	private function __construct() {
		$this->require_inc();
		$this->init_hooks();
		$this->init_config();
	}

	/**
	 * Add hooks.
	 */
	private function init_hooks() {
		add_action( 'init', [ $this, 'require_fields' ] );
		add_action( 'wp_ajax_nebotheme_mod_default', [ $this, 'ajax_theme_mod_to_default' ] );

		add_action( 'customize_controls_enqueue_scripts', [ $this, 'customize_enqueue' ] );
		add_action( 'customize_preview_init', [ $this, 'preview_init_enqueue' ] );

		/* Kirki */
		add_filter( 'kirki_control_types', [ $this, 'kirki_control_types' ] );

		/* For mailchimp preview form */
		add_filter( 'kirki_telemetry', '__return_false' );

		add_action( 'init', function() {
			foreach ( Customize_Manager::get_fields() as $field ) {
				if ( 'image' !== $field['type'] || ! isset( $field['choices']['save_as'] ) || 'id' !== $field['choices']['save_as'] ) {
					continue;
				}

				add_filter( "theme_mod_{$field['settings']}", [ __CLASS__, 'get_fallback_theme_mod_image' ] );
			}
		} );

		add_action( 'nebotheme_customize_fields_end', function() {
			$fields = array_filter( Customize_Manager::get_fields(), function( $field ) {
				return isset( $field['type'] ) && 'multicolor' === $field['type'];
			} );

			foreach ( $fields as $field ) {
				$theme_mod_name = $field['settings'];

				add_filter( "theme_mod_{$theme_mod_name}", function( $theme_mod_value ) use ( $theme_mod_name ) {
					if ( $theme_mod_value && is_array( $theme_mod_value ) ) {
						$theme_default = nebotheme_get_tm_default( $theme_mod_name );

						if ( is_array( $theme_default ) ) {
							$theme_mod_value = Utils::array_merge_safe( $theme_mod_value, $theme_default );
						}
					} else {
						$theme_default = nebotheme_get_tm_default( $theme_mod_name );

						if ( $theme_default ) {
							$theme_mod_value = $theme_default;
						}
					}

					return $theme_mod_value;
				} );
			}
		} );

		add_action( 'admin_init', function() {
			$plugin = 'kirki/kirki.php';

			if ( ! ( in_array( $plugin, (array) get_option( 'active_plugins', [] ), true ) || is_plugin_active_for_network( $plugin ) ) ) {
				return;
			}

			deactivate_plugins( $plugin );
		});
	}

	/**
	 * Fallback value
	 */
	public static function get_fallback_theme_mod_image( $theme_mod ) {
		if ( ! $theme_mod ) {
			return '';
		}

		return $theme_mod;
	}

	/**
	 * Ajax. Restart value theme-mod.
	 */
	public function ajax_theme_mod_to_default() {
		if ( ! check_ajax_referer( 'nebotheme', false, false ) ) {
			wp_send_json_error( '', 400 );
		}

		$id = Utils::get_if_isset( $_POST, 'id' );

		if ( $id ) {
			remove_theme_mod( $id );

			wp_send_json(
				[
					'deleted' => true,
					'default' => nebotheme_get_tm( $id ),
				]
			);
		} else {
			wp_send_json_error( '', 400 );
		}
	}

	/**
	 * Add styles && scripts.
	 */
	public function customize_enqueue() {
		$version = wp_get_theme()->get( 'Version' );
		$suffix = Theme::get_suffix_assets();
		$config = apply_filters(
			'nebotheme_customize_config',
			[
				'l10n' => [
					'refresh' => esc_html__( 'Refresh', 'yaffo' ),
					'refreshing' => esc_html__( 'Refreshing', 'yaffo' ) . '...',
				],
			]
		);

		wp_enqueue_style(
			'yaffo-customize',
			Theme::get_style_url( 'customize' ),
			[],
			$version,
			false
		);

		wp_enqueue_script(
			'yaffo-customize',
			Theme::URI_JS . "/customize/script{$suffix}.js",
			[
				'jquery',
				'customize-base',
			],
			$version,
			false
		);

		wp_localize_script(
			'yaffo-customize',
			'nebothemeCustomizeConfig',
			$config
		);

		$css = $this->get_css_customize();

		if ( $css ) {
			wp_add_inline_style( 'yaffo-customize', $css );
		}
	}

	/**
	 * Get css for frontend.
	 */
	protected function get_css_customize() {
		$css = '';
		$selectors_array = [];

		foreach ( Customize_Manager::get_fields() as $field ) {
			if ( ! Utils::get_if_isset( $field, 'stick' ) ) {
				continue;
			}

			$selectors_array[] = "#customize-control-{$field['settings']} + .customize-control";
		}

		if ( $selectors_array ) {
			$css .= join( ',', $selectors_array ) . '{
				margin-top: -25px !important;
				border-top: 0;
			}';
		}

		return $css;
	}

	/**
	 * Add styles & scripts for customize preview.
	 */
	public function preview_init_enqueue() {
		$version = wp_get_theme()->get( 'Version' );
		$suffix = Theme::get_suffix_assets();

		wp_enqueue_style(
			'yaffo-customize-preview',
			Theme::get_style_url( 'customize-preview' ),
			[],
			$version,
			false
		);

		wp_enqueue_script(
			'yaffo-customize-preview',
			Theme::URI_JS . "/customize-preview/script{$suffix}.js",
			[
				'jquery',
				'customize-preview',
				'customize-selective-refresh',
			],
			$version,
			true
		);
	}

	/**
	 * Require Fields.
	 */
	public function require_fields() {
		$fields_dir = YAFFO_DIR_FRAMEWORK . '/customize/includes/fields';

		do_action( 'nebotheme_customize_added_fields_before' );

		require_once "{$fields_dir}/appearance.php";
		require_once "{$fields_dir}/footer.php";
		require_once "{$fields_dir}/header.php";
		require_once "{$fields_dir}/miscellaneous.php";
		require_once "{$fields_dir}/site-identity.php";
		require_once "{$fields_dir}/social-media.php";

		do_action( 'nebotheme_customize_fields' );
		do_action( 'nebotheme_customize_fields_end' );
	}

	/**
	 * Register custom control for kirki.
	 */
	public function kirki_control_types( $controls ) {
		$namespace = __NAMESPACE__ . '\\Controls\\';

		$controls['nebotheme-ajax-select'] = $namespace . 'Ajax_Select';
		$controls['nebotheme-button'] = $namespace . 'Button';
		$controls['nebotheme-popover-end'] = $namespace . 'Popover_End';
		$controls['nebotheme-popover-start'] = $namespace . 'Popover_Start';

		return $controls;
	}

	/**
	 * Add kirki config.
	 */
	private function init_config() {
		/* Add Main Config For Customize */
		Customize_Manager::add_config(
			self::KIRKI_CONFIG_ID,
			[
				'capability' => 'edit_theme_options',
				'option_type' => 'theme_mod',
			]
		);
	}

	/**
	 * Require files
	 */
	private function require_inc() {
		if ( nebotheme_is_core_active() ) {
			require_once YAFFO_DIR_FRAMEWORK . '/customize/includes/kirki/kirki.php';
		}
	}
}
