<?php
/**
 * Font Printer Class
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Customize;

use NeboTheme\Yaffo\Utils;
use NeboTheme\Yaffo\Traits\Singleton;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Font Printer Class
 */
final class Font_Printer {
	use Singleton;

	const TYPOGRAPHY_ITEMS = [
		'font-family' => 'ff',
		'font-size' => 'fz',
		'line-height' => 'lh',
		'letter-spacing' => 'ls',
		'font-weight' => 'fw',
		'font-style' => 'fs',
		'text-transform' => 'tt',
	];

	/**
	 * Stack with css properties.
	 *
	 * @var array
	 */
	private $stack = [];

	/**
	 * The class constructor
	 */
	public function __construct() {
		$this->init_hooks();
	}

	/**
	 * Add hooks.
	 */
	private function init_hooks() {
		add_action( 'wp_enqueue_scripts', [ $this, 'print_css' ] );
	}

	/**
	 * Add font to turn print
	 */
	public function add_to_print( array &$args ) {
		if ( class_exists( 'Kirki' ) ) {
			$this->add_kirki_js_vars( $args );
		}

		$this->add_to_frontend( $args );
	}

	/**
	 * Add font to print on frontend
	 */
	private function add_to_frontend( array $args ) {
		if ( ! isset( $this->stack[ $args['css_var_selector'] ] ) ) {
			$this->stack[ $args['css_var_selector'] ] = [];
		}

		$this->stack[ $args['css_var_selector'] ][] = $args;
	}

	/**
	 * Add font to print via kirki on customize
	 */
	private function add_kirki_js_vars( array &$args ) {
		if ( empty( $args['js_vars'] ) || ! is_array( $args['js_vars'] ) ) {
			$args['js_vars'] = [];
		}

		foreach ( self::TYPOGRAPHY_ITEMS as $property => $property_short ) {
			$css_var_prefix = Utils::get_if_isset( $args, 'css_var_prefix', $args['settings'] );

			$args['js_vars'][] = [
				'function' => 'nebotheme_typography',
				'choice' => $property,
				'element' => $args['css_var_selector'],
				'property' => self::get_css_var_name( $css_var_prefix, $property_short ),
				'settings' => $args['settings'],
				'context' => [ 'editor' ],
			];
		}
	}

	/**
	 * Print style typography
	 */
	public function print_css() {
		if ( empty( $this->stack ) ) {
			return;
		}

		$css = '';

		foreach ( $this->stack as $selector => $array_args ) {
			$css .= "$selector {";

			foreach ( $array_args as $args ) {
				$value = nebotheme_get_tm( $args['settings'] );
				$css_var_prefix = Utils::get_if_isset( $args, 'css_var_prefix', $args['settings'] );
				$values = [
					'font-style' => 'normal',
				];

				if ( empty( $value ) || ! is_array( $value ) ) {
					continue;
				}

				foreach ( $value as $key => $sub_value ) {
					if ( 'font-family' === $key && false !== strpos( $sub_value, ' ' ) && false === strpos( $sub_value, '"' ) ) {
						$values['font-family'] = "\"$sub_value\"";
					}

					// Variants contain both font-weight & italics.
					if ( 'variant' === $key ) {
						$font_weight = str_replace( 'italic', '', $sub_value );
						$font_weight = ( in_array( $font_weight, [ '', 'regular' ], true ) ) ? '400' : $font_weight;
						$values['font-weight'] = $font_weight;

						// Is this italic?
						if ( false !== strpos( $sub_value, 'italic' ) ) {
							$values['font-style'] = 'italic';
						}
					} else {
						if (
							'line-height' === $key &&
							(
								is_numeric( $sub_value ) &&
								preg_match_all( '!\d+!', $sub_value )
							)
						) {
							$sub_value .= 'em';
						}

						$values[ $key ] = $sub_value;
					}
				}

				foreach ( $values as $property => $value ) {
					if ( ! $value && '0' !== $value ) {
						continue;
					}

					$property_short = self::get_short_font_property( $property );

					// 'font-family' to 'ff'
					$css .= self::get_css_var_name( $css_var_prefix, $property_short ) . ":{$value};";
				}
			}

			$css .= '}';
		}

		wp_register_style( 'yaffo-typography', false, [], wp_get_theme()->get( 'Version' ) );
		wp_enqueue_style( 'yaffo-typography' );
		wp_add_inline_style( 'yaffo-typography', $css );
	}

	/**
	 * Generate css-var name
	 */
	public static function get_css_var_name( $prefix, $property_short ) {
		$prefix = str_replace( '_', '-', $prefix );

		return "--nebotheme-{$prefix}-{$property_short}";
	}

	/**
	 * Get short name by font properties.
	 */
	public static function get_short_font_property( $property ) {
		if ( isset( self::TYPOGRAPHY_ITEMS[ $property ] ) ) {
			return self::TYPOGRAPHY_ITEMS[ $property ];
		}

		return $property;
	}
}
