<?php
/**
 * Theme Fonts
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Customize;

use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Traits\Singleton;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Custom fonts from the theme
 */
class Theme_Fonts {
	use Singleton;

	/**
	 * The theme fonts.
	 *
	 * @var array
	 */
	public static $theme_fonts = [];

	/**
	 * The theme all variants.
	 *
	 * @var array
	 */
	public static $theme_variants = [];

	/**
	 * The class constructor
	 */
	public function __construct() {
		$this->init();

		add_action( 'wp_enqueue_scripts', [ $this, 'wp_enqueue_scripts' ], 5 );
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ], 5 );

		add_filter( 'kirki_fonts_google_fonts', [ $this, 'remove_from_google_fonts' ] );
		add_filter( 'site_transient_kirki_googlefonts_cache', [ $this, 'remove_from_google_fonts' ] );

		if ( is_customize_preview() && is_admin() ) {
			add_filter( 'nebotheme_customize_config', [ $this, 'config_customize' ] );
		}

	}

	/**
	 * Get data to customize.
	 *
	 * @param array $config
	 *
	 * @return array
	 */
	public function config_customize( $config ) {
		$config['theme_fonts_names'] = array_keys( self::$theme_fonts );

		return $config;
	}

	/**
	 * Remove from google fonts list
	 */
	public function remove_from_google_fonts( $fonts ) {
		if ( is_array( $fonts ) ) {
			$fonts = array_filter( $fonts, function ( $font_family ) {
				return ! isset( self::$theme_fonts[ $font_family ] );
			}, ARRAY_FILTER_USE_KEY );
		}

		return $fonts;
	}

	/**
	 * Initialization
	 */
	public function init() {
		self::$theme_fonts = [
			'Kaisei HarunoUmi' => [
				'slug'     => 'kaisei-harunoumi',
				'variants' => [ 'regular', '500', '700' ],
			],
			'Kaisei Opti' => [
				'slug'     => 'kaisei-opti',
				'variants' => [ 'regular', '500', '700' ],
			],
			'Noto Serif Display' => [
				'slug'     => 'noto-serif-display-v9-latin',
				'variants' => [ '100', '200', '300', 'regular', '600', '700', '800', '900', '100italic', '200italic', '300italic', 'italic', '600italic', '700italic', '800italic', '900italic' ],
			],
			'Azeret Mono' => [
				'slug'     => 'azeret-mono-v9-latin',
				'variants' => [ '100', '200', '300', 'regular', '600', '700', '800', '900', '100italic', '200italic', '300italic', 'italic', '600italic', '700italic', '800italic', '900italic' ],
			],
		];
	}

	/**
	 * Add new fonts for choices
	 */
	public static function get_fonts_choices() {
		static $fonts = [];

		if ( ! empty( $fonts ) ) {
			return $fonts;
		}

		// Add new section.
		if ( self::$theme_fonts ) {
			$fonts['fonts']['families']['theme'] = [
				'text'     => esc_html__( 'Theme Fonts', 'yaffo' ),
				'children' => [],
			];
		}

		// Add new font.
		foreach ( self::$theme_fonts as $font_family => $font ) {
			$fonts['fonts']['families']['theme']['children'][] = [
				'text' => $font_family,
				'id'   => $font_family,
			];

			$fonts['fonts']['variants'][ $font_family ] = $font['variants'];
		}

		return $fonts;
	}

	/**
	 * Get font-weight from variant.
	 *
	 * @param string $variant The variant of font.
	 */
	public static function get_font_weight( $variant = 'regular' ) {
		$font_weight = filter_var( $variant, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION );

		return ( 'regular' === $variant || 'italic' === $variant ) ? 400 : absint( $font_weight );
	}

	/**
	 * Get font-style from variant.
	 *
	 * @param string $variant The variant of font.
	 */
	public static function get_font_style( $variant = 'regular' ) {
		return ( false === strpos( $variant, 'italic' ) ) ? 'normal' : 'italic';
	}

	/**
	 * Get stack the variants that are used in the theme.
	 */
	public function init_theme_variants() {
		$fields = Customize_Manager::get_fields();

		$extra_variants = [];

		foreach ( $fields as $field ) {
			if ( ! isset( $field['type'] ) || 'typography' !== $field['type'] ) {
				continue;
			}

			$field_value = nebotheme_get_tm( $field['settings'], $field['default'] );

			if ( ! isset( $field_value['font-family'] ) || ! $field_value['font-family'] ) {
				continue;
			}

			$font_family = $field_value['font-family'];
			$font_weight = ( isset( $field_value['font-weight'] ) && $field_value['font-weight'] ) ? $field_value['font-weight'] : ( ( isset( $field_value['variant'] ) && $field_value['variant'] ) ? $field_value['variant'] : 400 );
			$font_weight = ( 'regular' === $font_weight ) ? 400 : absint( $font_weight );
			$font_style  = ( isset( $field_value['font-style'] ) && $field_value['font-style'] ) ? $field_value['font-style'] : 'normal';

			array_push( $extra_variants, $font_family . $font_weight . $font_style );

			if ( ! isset( $field['choices']['variant'] ) || ! $field['choices']['variant'] ) {
				continue;
			}

			foreach ( $field['choices']['variant'] as $variant ) {
				$font_weight = self::get_font_weight( $variant );
				$font_style  = self::get_font_style( $variant );

				array_push( $extra_variants, $font_family . $font_weight . $font_style );
			}
		}

		self::$theme_variants = $extra_variants;
	}

	/**
	 * Gets all our styles and returns them as a string.
	 */
	public function get_styles() {
		$this->init_theme_variants();

		ob_start();

		foreach ( self::$theme_fonts as $font_family => $font ) {
			foreach ( $font['variants'] as $variant ) {
				$font_weight = self::get_font_weight( $variant );
				$font_style  = self::get_font_style( $variant );
				$font_src    = YAFFO_URI_ASSETS . "/fonts/{$font['slug']}/{$font['slug']}-{$variant}";

				$hash = $font_family . $font_weight . $font_style;

				if ( ! in_array( $hash, self::$theme_variants, true ) && ! is_customize_preview() ) {
					continue;
				}
				?>
				@font-face {
					font-family: '<?php echo esc_html( $font_family ); ?>';
					src: url('<?php echo esc_html( $font_src ); ?>.woff2') format('woff2'),
						url('<?php echo esc_html( $font_src ); ?>.woff') format('woff');
					font-weight: <?php echo esc_html( $font_weight ); ?>;
					font-display: swap;
					font-style: <?php echo esc_html( $font_style ); ?>;
				}
				<?php
			}
		}

		$styles = ob_get_clean();

		$styles = str_replace( [ '\t', '\r', '\n' ], '', $styles );

		return $styles;
	}

	/**
	 * Enqueue scripts and styles.
	 */
	public function wp_enqueue_scripts() {
		wp_register_style( 'yaffo-theme-fonts', false, [], wp_get_theme()->get( 'Version' ) );
		wp_enqueue_style( 'yaffo-theme-fonts' );

		if ( is_customize_preview() ) {
			wp_add_inline_style( 'yaffo-theme-fonts', $this->get_styles() );
		} else {
			wp_add_inline_style( 'yaffo-theme-fonts', $this->get_styles() );
		}
	}

	/**
	 * Admin Enqueue scripts and styles.
	 */
	public function admin_enqueue_scripts() {
		if ( ! is_customize_preview() ) {
			wp_register_style( 'yaffo-theme-fonts', false, [], wp_get_theme()->get( 'Version' ) );
			wp_enqueue_style( 'yaffo-theme-fonts' );

			wp_add_inline_style( 'yaffo-theme-fonts', $this->get_styles() );
		}
	}
}
