<?php
/**
 * General Function.
 *
 * @package Yaffo
 */

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Blog_Main;
use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Customize\Customize;
use NeboTheme\Yaffo\Homepage;
use NeboTheme\Yaffo\Menu\Menu_Manager;
use NeboTheme\Yaffo\Sidebars\Sidebar_Manager;
use NeboTheme\Yaffo\ThemeData\Default_Manager;
use NeboTheme\Yaffo\Utils;
use NeboTheme\YaffoCore\Lazy_Load;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Check core plugin.
 */
function nebotheme_is_core_active() {
	return class_exists( '\Yaffo_Core' );
}

/**
 * Get default data.
 */
function nebotheme_get_default() {
	return Default_Manager::get_data();
}

/**
 * Retrieve theme default data by theme-mod name
 *
 * @param string $tm_name
 * @param bool $type
 */
function nebotheme_get_tm_default( $tm_name, $type = false ) {
	$default = Utils::get_if_isset( nebotheme_get_default()['theme_mods'], $tm_name, null );

	if ( $type && isset( Default_Manager::CONTROL_DEFAULT[ $type ] ) ) {
		$default_data = Default_Manager::CONTROL_DEFAULT[ $type ];

		if ( is_array( $default ) && is_array( $default_data ) ) {
			$default = Utils::array_merge_safe( $default, $default_data );
		} elseif ( empty( $default ) ) {
			$default = $default_data;
		}
	}

	return $default;
}

/**
 * Get theme-mod
 *
 * Retrieve theme-mod with default theme.
 */
function nebotheme_get_tm_fallback( $theme_mod_name ) {
	$theme_mod = nebotheme_get_tm( $theme_mod_name );

	if ( ! $theme_mod ) {
		$field = Customize_Manager::get_field( $theme_mod_name );

		if ( $field ) {
			if ( 'text' === $field['type'] && isset( $field['input_attrs']['placeholder'] ) ) {
				return $field['input_attrs']['placeholder'];
			}
		}
	}

	return $theme_mod;
}

/**
 * Get theme-mod
 *
 * Retrieve theme-mod with default theme.
 */
function nebotheme_get_tm( $theme_mod_name, $default = null ) {
	return nebotheme_get_transformed_value(
		Customize_Manager::get_option( Customize::KIRKI_CONFIG_ID, $theme_mod_name, $default )
	);
}

/**
 * Echoes html attr from array.
 */
function nebotheme_render_attrs( array $array ) {
	echo nebotheme_get_render_attrs( $array );
}

/**
 * Get html attr from array.
 */
function nebotheme_get_render_attrs( array $attrs ) {
	$to_skip = [ null, [] ];

	return implode(
		' ',
		array_map(
			function( $key, $attr_value ) use ( $to_skip ) {
				$to_esc = true;

				if ( in_array( $attr_value, $to_skip, true ) ) {
					return '';
				}

				if ( is_array( $attr_value ) ) {
					$attr_value = implode(
						' ',
						array_filter(
							$attr_value,
							function( $attr_value_item ) use ( $to_skip ) {
								return ! in_array( $attr_value_item, $to_skip, true );
							}
						)
					);
				}

				if (
					$to_esc &&
					(
						(
							'href' === $key &&
							preg_match( '/^javascript:/', $attr_value )
						) ||
						(
							'src' === $key &&
							preg_match( '/^data:/', $attr_value )
						)
					)
				) {
					$to_esc = false;
				}

				if ( $to_esc ) {
					switch ( $key ) {
						case 'action':
						case 'background':
						case 'cite':
						case 'classid':
						case 'codebase':
						case 'formaction':
						case 'href':
						case 'longdesc':
						case 'manifest':
						case 'profile':
						case 'src':
							$attr_value = esc_url( $attr_value );
							break;

						default:
							$attr_value = esc_attr( $attr_value );
							break;
					}
				}

				return esc_attr( $key ) . '="' . $attr_value . '"';
			},
			array_keys( $attrs ),
			$attrs
		)
	);
}

/**
 * Get current palette from theme-mod.
 */
function nebotheme_get_color_palette() {
	return array_values( nebotheme_get_color_general() );
}

/**
 * Get General Color from theme-mod.
 */
function nebotheme_get_color_general( $color_name = '' ) {
	$color_general = nebotheme_get_tm( 'color_general', [] );

	if ( ! is_array( $color_general ) ) {
		$color_general = [];
	}

	if ( $color_name ) {
		return Utils::get_if_isset( $color_general, $color_name );
	}

	return $color_general;
}

/**
 * Get General Color from default.
 */
function nebotheme_get_color_general_default_palette() {
	return nebotheme_get_tm_default( 'color_general' );
}

/**
 * Get H1 only once.
 */
function nebotheme_get_tag_h1( $tag = 'h2' ) {
	static $h1_allow = true;

	if ( is_singular() && preg_match( '/<\s*h1[^>]*>/', get_the_content() ) ) {
		$h1_allow = false;
	}

	if ( ! $h1_allow ) {
		return $tag;
	}

	$h1_allow = false;

	return 'h1';
}

/**
 * Displays the classes for the post container element.
 */
function nebotheme_post_class( $css_classes ) {
	post_class( "nebotheme-post $css_classes" );
}

/**
 * Ger social list from wp users.
 */
function nebotheme_get_user_socials() {
	$social_types = nebotheme_get_social_links();

	foreach ( $social_types as $type => $social ) {
		if ( ! empty( $social['user_skip'] ) ) {
			unset( $social_types[ $type ] );
		}
	}

	return $social_types;
}

/**
 * Retrieve theme-vars data.
 *
 * @param string $theme_vars_name
 * @param mixed $fail
 */
function nebotheme_get_theme_vars( $theme_vars_name = '', $fail = false ) {
	$theme_vars = nebotheme_get_default()['theme_vars'];

	if ( $theme_vars_name ) {
		return Utils::get_if_isset( $theme_vars, $theme_vars_name, $fail );
	}

	return $theme_vars;
}

/**
 * Ger social list options.
 */
function nebotheme_social_options() {
	return nebotheme_wp_list_pluck( nebotheme_get_social_links(), 'label' );
}

/**
 * Converts a value (e.g. no' or 'false') to a bool.
 */
function nebotheme_get_transformed_value( $value, $invert = false ) {
	$values = [
		[
			true,
			false,
			false,
		],
		[
			YAFFO_VALUE_POSITIVE,
			YAFFO_VALUE_NEGATIVE,
			'false',
		],
	];
	$values_from = $invert ? $values[0] : $values[1];
	$values_to = $invert ? $values[1] : $values[0];
	$found_index = array_search( $value, $values_from, true );

	if ( false !== $found_index && isset( $values_to[ $found_index ] ) ) {
		return $values_to[ $found_index ];
	}

	return $value;
}

/**
 * Get branded image.
 */
function nebotheme_get_placeholder_image_url( $args = [] ) {
	static $args_default = [
		'size' => 'yaffo-grid',
		'color_name_bg' => 'border',
		'color_name_text' => 'color_general',
		'text' => '+',
	];
	$args = array_merge( $args_default, $args );

	if ( 'full' === $args['size'] ) {
		$args['size'] = 'thumbnail';
	}

	$color_general = nebotheme_get_tm( 'color_general' );

	$size_data = Utils::get_image_size( $args['size'] );
	$bg_color = Utils::get_if_isset( $color_general, $args['color_name_bg'] );
	$text_color = Utils::get_if_isset( $color_general, $args['color_name_text'] );

	$text_color = str_replace( '#', '', $text_color );
	$bg_color = str_replace( '#', '', $bg_color );

	return sprintf(
		'https://via.placeholder.com/%sx%s.jpg/%s/%s?text=%s',
		$size_data['width'],
		$size_data['height'],
		$bg_color,
		$text_color,
		$args['text']
	);
}

/**
 * Pluck a certain field out of each object in a list.
 *
 * Safer for negative values.
 */
function nebotheme_wp_list_pluck( $list, $field, $index_key = null ) {
	if ( ! $list || ! is_array( $list ) ) {
		$list = [];
	}

	return wp_list_pluck( $list, $field, $index_key );
}

/**
 * Get social url from theme-mod.
 */
function nebotheme_get_social_url( $type ) {
	return nebotheme_get_tm( "{$type}_url" );
}

/**
 * Get social icon from theme-vars.
 */
function nebotheme_get_social_data( $type, $key = '' ) {
	$socials_date = nebotheme_get_theme_vars( 'socials_data' );
	$social_date = Utils::get_if_isset( $socials_date, $type );

	if ( ! $social_date ) {
		return;
	}

	return Utils::get_if_isset( $social_date, $key, $social_date );
}

/**
 * Get sortable for social icons.
 */
function nebotheme_get_social_icons_sortable() {
	return (array) nebotheme_get_tm( 'social_icons_sortable' );
}

/**
 * Get Fields for Meta Boxes
 */
function nebotheme_get_social_links() {
	$social_links_values = nebotheme_get_theme_vars( 'social_links' );
	$social_links = [];

	foreach ( $social_links_values as $social_name ) {
		$social_data = nebotheme_get_social_data( $social_name );

		if ( ! $social_data ) {
			continue;
		}

		$social_links[ $social_name ] = $social_data;
	}

	return $social_links;
}

/**
 * Get url for documentation theme.
 */
function nebotheme_get_url_docs_theme() {
	$url_docs = nebotheme_get_url_docs();
	$theme = wp_get_theme();

	return "{$url_docs}/themes/{$theme->template}";
}

/**
 * Get url for documentation.
 */
function nebotheme_get_url_docs() {
	return 'https://docs.nebotheme.com';
}

/**
 * Get url for support.
 */
function nebotheme_get_url_support() {
	return 'https://support.nebotheme.com/customer/tickets/create';
}

/**
 * Return content
 *
 * @param mixed $content
 */
function nebotheme_unsanitize( $content ) {
	return $content;
}

/**
 * Return content.
 */
function nebotheme_get_render_selected( $selected, $current = true, $echo = true ) {
	if ( is_array( $selected ) ) {
		$found = in_array( $current, $selected ); // phpcs:ignore WordPress.PHP.StrictInArray.MissingTrueStrict

		return selected( true, $found, $echo );
	} else {
		return selected( $selected, $current, $echo );
	}
}

/**
 * Check lazyload.
 */
function nebotheme_is_custom_lazyload() {
	return class_exists( Lazy_Load::class ) && Lazy_Load::is_enabled();
}

/**
 * Check off-canvas.
 */
function nebotheme_is_render_off_canvas() {
	return Menu_Manager::instance()->has_nav_menu( Menu_Manager::MENU_ID_OFF_CANVAS ) || Sidebar_Manager::is_active( Sidebar_Manager::SIDEBAR_OFF_CANVAS );
}

/**
 * Check if masonry.
 */
function nebotheme_is_masonry() {
	static $is_masonry = null;

	if ( is_null( $is_masonry ) ) {
		if ( is_archive() ) {
			$blog_layout = explode( '/', nebotheme_get_tm( 'archive_blog_layout' ) );
			$is_masonry = in_array( 'grid', $blog_layout, true ) && nebotheme_get_tm( 'archive_blog_masonry' );
		} elseif ( is_search() ) {
			$blog_layout = explode( '/', nebotheme_get_tm( 'search_blog_layout' ) );
			$is_masonry = in_array( 'grid', $blog_layout, true ) && nebotheme_get_tm( 'search_blog_masonry' );
		} else {
			$blog_layout = explode( '/', Homepage::get_settings( 'homepage_blog_layout' ) );
			$is_masonry = in_array( 'grid', $blog_layout, true ) && Homepage::get_settings( 'homepage_blog_masonry' );
		}
	}

	return $is_masonry;
}

/**
 * Get Theme Image Sizes
 */
function nebotheme_get_theme_image_sizes() {
	return nebotheme_get_theme_vars( 'image_sizes', [] );
}

/**
 * Check if post has video bg.
 *
 * @return bool
 */
function nebotheme_is_post_video_bg() {
	return (
		nebotheme_get_location_video_bg() &&
		Meta_Boxes::get_pm( 'video_bg' ) &&
		nebotheme_get_youtube_video_id() &&
		nebotheme_is_location_video_bg()
	);
}

/**
 * Get Youtube video ID from YouTube URL.
 *
 * @return string
 */
function nebotheme_get_youtube_video_id() {
	$video_bg_youtube_url = Meta_Boxes::get_pm( 'video_bg_youtube_url' );

	if ( ! $video_bg_youtube_url ) {
		return '';
	}

	preg_match( '/(http(s|):|)\/\/(www\.|)yout(.*?)\/(embed\/|watch.*?v=|)([a-z_A-Z0-9\-]{11})/i', $video_bg_youtube_url, $results );
	$id = '';

	if ( ! empty( $results[6] ) ) {
		$id = $results[6];
	}

	return $id;
}

/**
 * Get location for video background.
 *
 * @return string
 */
function nebotheme_get_location_video_bg() {
	if ( Sidebar_Manager::is_rendering_sidebar() ) {
		return;
	}

	if ( Blog_Main::is_active() ) {
		if ( is_search() || is_archive() ) {
			return 'archive/search';
		} else {
			return 'page/home';
		}
	} elseif ( Homepage::is_slider_rendering() ) {
		return 'page/home:slider';
	} elseif ( is_single() ) {
		return 'single';
	}
}

/**
 * Check location for video background.
 *
 * @return bool
 */
function nebotheme_is_location_video_bg() {
	$location = nebotheme_get_location_video_bg();

	if ( ! $location ) {
		return false;
	}

	$video_bg_location = Meta_Boxes::get_pm( 'video_bg_location' );

	if ( ! $video_bg_location ) {
		return;
	}

	return in_array( $location, $video_bg_location, true );
}

/**
 * Check author box
 *
 * @return bool
 */
function nebotheme_is_render_author_box() {
	if ( ! is_author() && ! is_single() ) {
		return;
	}

	$id = get_the_author_meta( 'ID' );

	return (bool) get_the_author_meta( 'description', $id );
}

/**
 * Get social author
 *
 * @return array
 */
function nebotheme_get_social_author() {
	$id = get_the_author_meta( 'ID' );
	$website_url = get_the_author_meta( 'url', $id );
	$socials = [];

	if ( $website_url ) {
		$socials[] = [
			'icon' => 'nebotheme-icon-globe',
			'url' => $website_url,
			'title' => esc_html__( 'Website', 'yaffo' ),
		];
	}

	foreach ( nebotheme_get_user_socials() as $type => $social ) {
		$url_profile = get_the_author_meta( $type, $id );

		if ( ! $url_profile ) {
			continue;
		}

		$socials[] = [
			'url' => $url_profile,
			'title' => nebotheme_get_social_data( $type, 'label' ),
			'icon' => nebotheme_get_social_data( $type, 'icon' ),
		];
	}

	if ( ! get_the_author_meta( 'hide_email', $id ) ) {
		$email = get_the_author_meta( 'email', $id );

		if ( $email ) {
			$socials[] = [
				'icon' => 'nebotheme-icon-mail-alt',
				'url' => $email,
				'title' => esc_html__( 'Email', 'yaffo' ),
			];
		}
	}

	return $socials;
}
