<?php
/**
 * Base class menu.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Menu;

use NeboTheme\Yaffo\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Base class menu.
 */
abstract class Base {

	/**
	 * ID menu.
	 *
	 * @var string
	 */
	private $id;

	/**
	 * Args menu.
	 *
	 * @var array
	 */
	private $args = [];

	/**
	 * Array of nav menu arguments.
	 *
	 * @var array
	 */
	private $wp_nav_menu_args = [];

	/**
	 * Get default arguments for menu.
	 */
	public static function get_wp_nav_menu_args_default() {
		return [
			'container_class' => 'navigation-menu-container',
			'container' => 'ul',
			'fallback_cb' => 'wp_page_menu',
			'link_after' => '</span>',
			'link_before' => '<span class="nav-item-text">',
			'menu_class' => 'navigation-menu',
		];
	}

	/**
	 * Get class name with namespace.
	 */
	public static function get_class_full_name() {
		return get_called_class();
	}

	/**
	 * The class constructor
	 */
	public function __construct( $id, array $args ) {
		$this->id = $id;
		$this->args = $args;

		$this->ensure_wp_nav_menu_args();
	}


	/**
	 * Ensure arguments for menu.
	 */
	protected function ensure_wp_nav_menu_args() {
		if ( ! empty( $this->wp_nav_menu_args ) ) {
			return;
		}

		$wp_nav_menu_args = array_merge( static::get_wp_nav_menu_args_default(), $this->get_add_wp_nav_menu_args() );

		$wp_nav_menu_args['theme_location'] = $this->get_id();

		$this->wp_nav_menu_args = $wp_nav_menu_args;
	}

	/**
	 * Get id menu.
	 */
	public function get_id() {
		return $this->id;
	}

	/**
	 * Get args menu.
	 */
	public function get_args( $key = null ) {
		if ( $key ) {
			return Utils::get_if_isset( $this->args, $key );
		}

		return $this->args;
	}

	/**
	 * Get label menu.
	 */
	public function get_label() {
		return $this->get_args( 'label' );
	}

	/**
	 * Check is the menu active.
	 */
	public function has_nav_menu() {
		return has_nav_menu( $this->get_id() );
	}

	/**
	 * Check is the menu render.
	 */
	public function is_fallback_render() {
		$has_nav_menu = $this->has_nav_menu();

		return ! $has_nav_menu && 'wp_page_menu' === $this->get_wp_nav_menu_args( 'fallback_cb' );
	}

	/**
	 * Check is the menu render.
	 */
	public function is_render() {
		$has_nav_menu = $this->has_nav_menu();

		if ( ! $has_nav_menu ) {
			$fallback_cb = $this->get_wp_nav_menu_args( 'fallback_cb' );

			if ( 'wp_page_menu' === $fallback_cb ) {
				$has_nav_menu = true;
			}
		}

		return $has_nav_menu;
	}

	/**
	 * Get additional arguments for menu.
	 */
	protected function get_add_wp_nav_menu_args() {
		return Utils::get_if_isset( $this->get_args(), 'wp_nav_menu_args', [] );
	}

	/**
	 * Get arguments for menu.
	 */
	protected function get_wp_nav_menu_args( $key = null ) {
		if ( $key ) {
			return Utils::get_if_isset( $this->wp_nav_menu_args, $key );
		}

		return $this->wp_nav_menu_args;
	}

	/**
	 * Echoes menu.
	 */
	public function render() {
		$is_render = $this->is_render();

		if ( ! $is_render ) {
			return;
		}

		$attrs = [
			'class' => [
				'nebotheme-navigation',
				"nebotheme-navigation-{$this->get_id()}",
			],
		];

		if ( $this->is_fallback_render() ) {
			$attrs['class'][] = 'nebotheme-navigation--is-fallback';
		}

		echo '<div ' . nebotheme_get_render_attrs( $attrs ) . '>';

		echo '<nav class="navigation">';

		wp_nav_menu( $this->get_wp_nav_menu_args() );

		echo '</nav>';

		echo '</div>';
	}

	/**
	 * Get name.
	 */
	abstract public static function get_name();
}
