<?php
/**
 * Manager menus.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Menu;

use NeboTheme\Yaffo\Menu\Base;
use NeboTheme\Yaffo\Menu\Menu_Horizontal_Smart;
use NeboTheme\Yaffo\Menu\Menu_Vertical;
use NeboTheme\Yaffo\Menu\Menu_Footer;
use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Class manager menus.
 */
class Menu_Manager {
	const MENU_ID_PRIMARY = 'primary';
	const MENU_ID_STICKY = 'sticky';
	const MENU_ID_OFF_CANVAS = 'off_canvas';
	const MENU_ID_FOOTER = 'footer';

	use Singleton;

	/**
	 * Menus objects
	 *
	 * @var Base[]
	 */
	protected $menus = [];

	/**
	 * The class constructor
	 */
	private function __construct() {
		$this->add_menus();
		$this->add_hooks();
	}

	/**
	 * Register new hooks.
	 */
	private function add_hooks() {
		add_action( 'after_setup_theme', [ $this, 'register_nav_menus' ] );
	}

	/**
	 * Register support for various WordPress features.
	 */
	public function register_nav_menus() {
		$locations = [];

		foreach ( $this->get_menus() as  $menu_obj ) {
			$locations[ $menu_obj->get_id() ] = $menu_obj->get_label();
		}

		register_nav_menus( $locations );
	}

	/**
	 * Initialize and add menus.
	 */
	private function add_menus() {
		foreach ( $this->get_default_menu() as $id => $args ) {
			$this->add_menu( $id, $args );
		}
	}

	/**
	 * Get default data menus.
	 */
	private function get_default_menu() {
		return [
			self::MENU_ID_PRIMARY => [
				'label' => esc_html__( 'Primary Navigation', 'yaffo' ),
				'type' => Menu_Horizontal_Smart::get_name(),
			],
			self::MENU_ID_STICKY => [
				'label' => esc_html__( 'Sticky Navigation', 'yaffo' ),
				'type' => Menu_Horizontal_Smart::get_name(),
				'wp_nav_menu_args' => [
					'fallback_cb' => '__return_empty_string',
				],
			],
			self::MENU_ID_OFF_CANVAS => [
				'label' => esc_html__( 'Off-Canvas Navigation', 'yaffo' ),
				'type' => Menu_Vertical::get_name(),
				'wp_nav_menu_args' => [
					'fallback_cb' => '__return_empty_string',
				],
			],
			self::MENU_ID_FOOTER => [
				'label' => esc_html__( 'Footer Navigation', 'yaffo' ),
				'type' => Menu_Footer::get_name(),
				'wp_nav_menu_args' => [
					'fallback_cb' => '__return_empty_string',
				],
			],
		];
	}

	/**
	 * Initialize and add menu.
	 */
	public function add_menu( $id, $args ) {
		$class = static::get_name_class( $args['type'] );

		if ( ! $class && ! isset( $this->menus[ $id ] ) ) {
			return false;
		}

		$this->menus[ $id ] = new $class( $id, $args );

		return $this->menus[ $id ];
	}

	/**
	 * Get list menu classes
	 */
	protected static function get_name_classes() {
		return [
			Menu_Footer::get_name() => Menu_Footer::get_class_full_name(),
			Menu_Horizontal_Smart::get_name() => Menu_Horizontal_Smart::get_class_full_name(),
			Menu_Vertical::get_name() => Menu_Vertical::get_class_full_name(),
		];
	}

	/**
	 * Get class menu by name.
	 */
	protected static function get_name_class( $type ) {
		return Utils::get_if_isset( static::get_name_classes(), $type );
	}

	/**
	 * Get menu list.
	 */
	public function get_menus() {
		return $this->menus;
	}

	/**
	 * Get menu by id.
	 *
	 * @return Base
	 */
	public function get_menu( $id ) {
		return Utils::get_if_isset( $this->get_menus(), $id );
	}

	/**
	 * Echoes menu by id.
	 */
	public function render_menu( $id ) {
		$menu = $this->get_menu( $id );

		if ( ! $menu ) {
			return;
		}

		$menu->render();
	}

	/**
	 * Check is the menu active.
	 */
	public function is_render( $id ) {
		$menu = $this->get_menu( $id );

		if ( ! $menu ) {
			return;
		}

		return $menu->is_render();
	}

	/**
	 * Check is the menu active.
	 */
	public function has_nav_menu( $id ) {
		$menu = $this->get_menu( $id );

		if ( ! $menu ) {
			return;
		}

		return $menu->has_nav_menu();
	}
}
