<?php
/**
 * Class sidebars organizer
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Sidebars;

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Traits\Singleton;
use NeboTheme\Yaffo\Utils;
use NeboTheme\Yaffo\Customize\Customize_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Sidebar Manager Class
 */
final class Sidebar_Manager {
	use Singleton;

	const SIDEBAR_HOME = 'sidebar_default';
	const SIDEBAR_OFF_CANVAS = 'sidebar_off_canvas';
	const SIDEBAR_POST = 'sidebar_post';
	const SIDEBAR_PAGE = 'sidebar_page';
	const SIDEBAR_ARCHIVE = 'sidebar_archive';
	const SIDEBAR_SEARCH = 'sidebar_search';

	/**
	 * State render sidebar.
	 *
	 * @var bool
	 */
	protected static $rendering_sidebar = false;

	/**
	 * Check state render sidebar.
	 *
	 * @return bool
	 */
	public static function is_rendering_sidebar() {
		return static::$rendering_sidebar;
	}

	/**
	 * The class constructor
	 *
	 * @return void
	 */
	protected function __construct() {
		add_action( 'widgets_init', [ __CLASS__, 'register_sidebars' ] );
		add_filter( 'nebotheme_var_block_class', [ __CLASS__, 'var_block_class' ] );
		add_filter( 'nebotheme_meta_boxes_fields', [ $this, 'meta_boxes_fields' ] );

		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
	}

	/**
	 * Add Fields for Meta Boxes
	 */
	public function meta_boxes_fields( array $fields ) {
		global $content_width;

		$theme_mod_name = false;
		$sidebar_id = self::get_sidebar_id_by_condition();
		$layout_name = self::get_layout_name();

		if ( $sidebar_id ) {
			$theme_mod_name = self::get_layout_name( $sidebar_id );
		}

		$fields[ $layout_name ] = [
			'label' => esc_html__( 'Page Layout', 'yaffo' ),
			'description' => esc_html__( 'In Block Editor, wide width and full width choices function only for posts without sidebars.', 'yaffo' ),
			'type' => Meta_Boxes::CONTROL_SELECT,
			'tab' => 'layout',
			'theme_mod_name' => $theme_mod_name,
			'options' => [
				YAFFO_VALUE_NEGATIVE => esc_html__( 'No Sidebar', 'yaffo' ),
				'left' => esc_html__( 'Left Sidebar', 'yaffo' ),
				'right' => esc_html__( 'Right Sidebar', 'yaffo' ),
			],
		];

		if ( 'page' === get_post_type() ) {
			$fields['content_width'] = [
				'label' => esc_html__( 'Content Width', 'yaffo' ),
				'type' => Meta_Boxes::CONTROL_SLIDER,
				'tab' => 'layout',
				'min' => 700,
				'max' => $content_width,
				'placeholder' => $content_width,
				'selectors' => [
					'.entry-content' => '--entry-content-width: {{VALUE}}px',
				],
				'condition' => [
					$layout_name => YAFFO_VALUE_NEGATIVE,
				],
			];
		}

		return $fields;
	}

	/**
	 * Adds a customize fields with kirki.
	 *
	 * @return void
	 */
	public function customize_add_fields() {
		$sidebar_choices = [
			YAFFO_VALUE_NEGATIVE => esc_html__( 'No Sidebar', 'yaffo' ),
			'left' => esc_html__( 'Left Sidebar', 'yaffo' ),
			'right' => esc_html__( 'Right Sidebar', 'yaffo' ),
		];

		$single_post_meta_data_choices = apply_filters( 'nebotheme_single_post_meta_data_choices', [
			'author' => esc_html__( 'Author', 'yaffo' ),
			'comments' => esc_html__( 'Comments', 'yaffo' ),
			'date' => esc_html__( 'Date', 'yaffo' ),
			'update_date' => esc_html__( 'Update Date', 'yaffo' ),
			'read_time' => esc_html__( 'Read Time', 'yaffo' ),
		] );

		Customize_Manager::add_panel(
			'layout_sidebar',
			[
				'title' => esc_html__( 'Layouts', 'yaffo' ),
				'priority' => 6,
				'icon' => 'dashicons-layout',
			]
		);

		foreach ( self::get_sidebars_list_default() as $sidebar_id => $sidebar_args ) {
			if ( ! self::is_side_panel( $sidebar_id ) ) {
				continue;
			}

			$setting_name = self::get_layout_name( $sidebar_id );

			$section_id = "layout_sidebar_{$sidebar_id}";

			Customize_Manager::add_section(
				$section_id,
				[
					'title' => $sidebar_args['name'],
					'panel' => 'layout_sidebar',
				]
			);

			Customize_Manager::add_field(
				[
					'type' => 'radio',
					'settings' => $setting_name,
					'label' => esc_html__( 'Sidebar Layout', 'yaffo' ),
					'default' => 'right',
					'choices' => $sidebar_choices,
					'theme_export' => true,
					'section' => $section_id,
				]
			);

			if ( self::SIDEBAR_ARCHIVE === $sidebar_id ) {
				$partial_refresh_blog = [
					'container_inclusive' => true,
					'selector' => 'body.archive .nebotheme-blog',
					'render_callback' => 'nebotheme_render_main_blog',
				];

				Customize_Manager::add_field(
					[
						'type' => 'radio',
						'settings' => 'archive_blog_layout',
						'label' => esc_html__( 'Blog Layout', 'yaffo' ),
						'section' => $section_id,
						'default' => 'list',
						'choices' => [
							'list' => esc_html__( 'List', 'yaffo' ),
							'grid' => esc_html__( 'Grid', 'yaffo' ),
						],
						'partial_refresh' => [
							'archive_blog_layout' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'archive_blog_date',
						'label' => esc_html__( 'Display Post Date', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
						'theme_export' => true,
						'partial_refresh' => [
							'archive_blog_date' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'archive_blog_read_time',
						'label' => esc_html__( 'Display Post Read Time', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
						'theme_export' => true,
						'partial_refresh' => [
							'archive_blog_read_time' => $partial_refresh_blog,
						],
						'active_callback' => [
							[
								'setting' => 'search_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'archive_blog_author',
						'label' => esc_html__( 'Display Post Author', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
						'theme_export' => true,
						'partial_refresh' => [
							'archive_blog_author' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'archive_blog_excerpt',
						'label' => esc_html__( 'Display Post Excerpt', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => false,
						'theme_export' => true,
						'partial_refresh' => [
							'archive_blog_excerpt' => $partial_refresh_blog,
						],
						'active_callback' => [
							[
								'setting' => 'archive_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'label' => esc_html__( 'Masonry', 'yaffo' ),
						'settings' => 'archive_blog_masonry',
						'type' => 'toggle',
						'section' => $section_id,
						'default' => false,
						'active_callback' => [
							[
								'setting' => 'archive_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
						],
						'partial_refresh' => [
							'archive_blog_masonry' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'label' => esc_html__( 'Grid Columns', 'yaffo' ),
						'settings' => 'archive_blog_grid_columns',
						'section' => $section_id,
						'type' => 'radio',
						'default' => '2',
						'choices' => [
							'2' => esc_html__( '2 Columns', 'yaffo' ),
							'3' => esc_html__( '3 Columns', 'yaffo' ),
						],
						'active_callback' => [
							[
								'setting' => 'archive_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
							[
								'setting' => 'archive_blog_masonry',
								'operator' => '==',
								'value' => '',
							],
						],
						'output' => [
							[
								'element' => 'body.archive .nebotheme-var-block:not(.sidebar-primary--exists) .nebotheme-blog__posts[data-layout="grid"]',
								'property' => '--nebotheme-blog-columns',
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'label' => esc_html__( 'Post Image', 'yaffo' ),
						'settings' => 'archive_image_cropped',
						'type' => 'radio',
						'section' => $section_id,
						'default' => 'cropped',
						'choices'  => [
							'cropped' => esc_attr__( 'Display Cropped Image', 'yaffo' ),
							'uncropped' => esc_attr__( 'Display in Original Ratio', 'yaffo' ),
						],
						'active_callback' => [
							[
								'setting' => 'archive_blog_layout',
								'operator' => '==',
								'value' => 'grid',
							],
							[
								'setting' => 'archive_blog_masonry',
								'operator' => '==',
								'value' => '',
							],
						],
						'partial_refresh' => [
							'archive_image_cropped' => $partial_refresh_blog,
						],
					]
				);
			} elseif ( self::SIDEBAR_SEARCH === $sidebar_id ) {
				$partial_refresh_blog = [
					'container_inclusive' => true,
					'selector' => 'body.search .nebotheme-blog',
					'render_callback' => 'nebotheme_render_main_blog',
				];

				Customize_Manager::add_field(
					[
						'type' => 'radio',
						'settings' => 'search_blog_layout',
						'label' => esc_html__( 'Blog Layout', 'yaffo' ),
						'section' => $section_id,
						'default' => 'list',
						'choices' => [
							'list' => esc_html__( 'List', 'yaffo' ),
							'grid' => esc_html__( 'Grid', 'yaffo' ),
						],
						'partial_refresh' => [
							'search_blog_layout' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'search_blog_date',
						'label' => esc_html__( 'Display Post Date', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
						'theme_export' => true,
						'partial_refresh' => [
							'search_blog_date' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'search_blog_read_time',
						'label' => esc_html__( 'Display Post Read Time', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
						'theme_export' => true,
						'partial_refresh' => [
							'search_blog_read_time' => $partial_refresh_blog,
						],
						'active_callback' => [
							[
								'setting' => 'search_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'search_blog_author',
						'label' => esc_html__( 'Display Post Author', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
						'theme_export' => true,
						'partial_refresh' => [
							'search_blog_author' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'search_blog_excerpt',
						'label' => esc_html__( 'Display Post Excerpt', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => false,
						'theme_export' => true,
						'partial_refresh' => [
							'search_blog_excerpt' => $partial_refresh_blog,
						],
						'active_callback' => [
							[
								'setting' => 'search_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'label' => esc_html__( 'Masonry', 'yaffo' ),
						'settings' => 'search_blog_masonry',
						'type' => 'toggle',
						'section' => $section_id,
						'default' => false,
						'active_callback' => [
							[
								'setting' => 'search_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
						],
						'partial_refresh' => [
							'search_blog_masonry' => $partial_refresh_blog,
						],
					]
				);

				Customize_Manager::add_field(
					[
						'label' => esc_html__( 'Grid Columns', 'yaffo' ),
						'settings' => 'search_blog_grid_columns',
						'section' => $section_id,
						'type' => 'radio',
						'default' => '2',
						'choices' => [
							'2' => esc_html__( '2 Columns', 'yaffo' ),
							'3' => esc_html__( '3 Columns', 'yaffo' ),
						],
						'active_callback' => [
							[
								'setting' => 'search_blog_layout',
								'operator' => 'in',
								'value' => [ 'grid' ],
							],
							[
								'setting' => 'search_blog_masonry',
								'operator' => '==',
								'value' => '',
							],
						],
						'output' => [
							[
								'element' => 'body.search .nebotheme-var-block:not(.sidebar-primary--exists) .nebotheme-blog__posts[data-layout="grid"]',
								'property' => '--nebotheme-blog-columns',
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'label' => esc_html__( 'Post Image', 'yaffo' ),
						'settings' => 'search_image_cropped',
						'type' => 'radio',
						'section' => $section_id,
						'default' => 'cropped',
						'choices'  => [
							'cropped' => esc_attr__( 'Display Cropped Image', 'yaffo' ),
							'uncropped' => esc_attr__( 'Display in Original Ratio', 'yaffo' ),
						],
						'active_callback' => [
							[
								'setting' => 'search_blog_layout',
								'operator' => '==',
								'value' => 'grid',
							],
							[
								'setting' => 'search_blog_masonry',
								'operator' => '==',
								'value' => '',
							],
						],
						'partial_refresh' => [
							'search_image_cropped' => $partial_refresh_blog,
						],
					]
				);
			}

			if ( self::get_layout_name( self::SIDEBAR_POST ) === $setting_name ) {
				Customize_Manager::add_field(
					[
						'settings' => 'homepage_meta_data_single',
						'label' => esc_html__( 'Display Meta Data', 'yaffo' ),
						'type' => 'sortable',
						'section' => $section_id,
						'choices' => $single_post_meta_data_choices,
						'default' => apply_filters( 'nebotheme_default_single_post_meta_data', [
							'author',
							'comments',
							'update_date',
							'date',
							'read_time',
						] ),
						'partial_refresh' => [
							'homepage_meta_data_single' => [
								'container_inclusive' => true,
								'selector' => '.nebotheme-post-single__header',
								'render_callback' => 'nebotheme_render_post_single_header',
							],
						],
					]
				);

				Customize_Manager::add_field(
					[
						'settings' => 'homepage_related_posts_single',
						'label' => esc_html__( 'Show Related Posts?', 'yaffo' ),
						'type' => 'toggle',
						'section' => $section_id,
						'default' => true,
					]
				);

				if ( nebotheme_is_core_active() ) {
					$social_types = [
						'facebook' => esc_html__( 'Facebook', 'yaffo' ),
						'twitter' => esc_html__( 'Twitter', 'yaffo' ),
						'pinterest' => esc_html__( 'Pinterest', 'yaffo' ),
						'email' => esc_html__( 'Email', 'yaffo' ),
						'telegram' => esc_html__( 'Telegram', 'yaffo' ),
						'linkedin' => esc_html__( 'Linkedin', 'yaffo' ),
						'reddit' => esc_html__( 'Reddit', 'yaffo' ),
						'stumbleupon' => esc_html__( 'Stumbleupon', 'yaffo' ),
						'whatsapp' => esc_html__( 'Whatsapp', 'yaffo' ),
					];

					Customize_Manager::add_field(
						[
							'settings' => 'homepage_post_shares_text',
							'label' => esc_html__( 'Shares Text', 'yaffo' ),
							'type' => 'text',
							'section' => $section_id,
							'input_attrs' => [
								'placeholder' => esc_html__( 'Shares', 'yaffo' ),
							],
							'partial_refresh' => [
								'homepage_post_shares_text' => [
									'container_inclusive' => true,
									'selector' => '.nebotheme-share-buttons',
									'render_callback' => 'nebotheme_render_share_buttons_single',
								],
							],
						]
					);

					Customize_Manager::add_field(
						[
							'settings' => 'homepage_post_shares',
							'label' => esc_html__( 'Shares Social', 'yaffo' ),
							'type' => 'sortable',
							'section' => $section_id,
							'choices' => $social_types,
							'default' => array_keys( $social_types ),
							'partial_refresh' => [
								'homepage_post_shares' => [
									'container_inclusive' => true,
									'selector' => '.nebotheme-share-buttons',
									'render_callback' => 'nebotheme_render_share_buttons_single',
								],
							],
						]
					);

					Customize_Manager::add_field(
						[
							'type' => 'toggle',
							'settings' => 'prev_next_pagination',
							'label' => esc_html__( 'Prev/Next Pagination', 'yaffo' ),
							'section' => $section_id,
							'default' => true,
						]
					);
				}
			}
		}
	}

	/**
	 * Adds css class to var-block
	 *
	 * @param array $classes
	 *
	 * @return array
	 */
	public static function var_block_class( array $classes ) {
		// Primary Sidebar.
		if ( self::is_active() ) {
			$primary_layout = self::get_side_layout();

			if ( $primary_layout ) {
				$classes[] = "sidebar-primary--{$primary_layout}";
			}

			$classes[] = 'sidebar-primary--exists';
		}

		return $classes;
	}

	/**
	 * Generate Condition By Post Types.
	 *
	 * @return callable
	 */
	public static function generate_condition_by_post_types( $post_types ) {
		if ( ! is_array( $post_types ) ) {
			$post_types = [ $post_types ];
		}

		if ( Utils::is_edit() ) {
			return function() use ( $post_types ) {
				return in_array( get_post_type(), $post_types, true );
			};
		}

		return function() use ( $post_types ) {
			return is_singular( $post_types );
		};
	}

	/**
	 * Sidebar list for default
	 *
	 * Sidebar Details
	 *
	 * @return array
	 */
	public static function get_sidebars_list_default() {
		static $sidebar_list = null;

		if ( empty( $sidebar_list ) ) {
			$sidebar_list = [
				self::SIDEBAR_HOME => [
					'name' => esc_html__( 'Home', 'yaffo' ),
					'description' => esc_html__( 'Widgets in this area will be shown in left or right sidebar on home.', 'yaffo' ),
					'condition_callable' => 'is_home',
					'side_panel' => true,
				],
				self::SIDEBAR_OFF_CANVAS => [
					'name' => esc_html__( 'Off-Canvas', 'yaffo' ),
					'description' => esc_html__( 'Widgets in this area will be shown in off-canvas.', 'yaffo' ),
					'side_panel' => false,
				],
				self::SIDEBAR_ARCHIVE => [
					'name' => esc_html__( 'Archive', 'yaffo' ),
					'description' => esc_html__( 'Widgets in this area will be shown in left or right sidebar on archive page.', 'yaffo' ),
					'condition_callable' => 'is_archive',
					'side_panel' => true,
				],
				self::SIDEBAR_SEARCH => [
					'name' => esc_html__( 'Search', 'yaffo' ),
					'description' => esc_html__( 'Widgets in this area will be shown in left or right sidebar on search page.', 'yaffo' ),
					'condition_callable' => 'is_search',
					'side_panel' => true,
				],
				self::SIDEBAR_PAGE => [
					'name' => esc_html__( 'Page', 'yaffo' ),
					'description' => esc_html__( 'Widgets in this area will be shown in left or right sidebar on page.', 'yaffo' ),
					'condition_callable' => self::generate_condition_by_post_types( 'page' ),
					'side_panel' => true,
				],
				self::SIDEBAR_POST => [
					'name' => esc_html__( 'Single Post', 'yaffo' ),
					'description' => esc_html__( 'Widgets in this area will be shown in left or right sidebar on single post & attachment.', 'yaffo' ),
					'condition_callable' => self::generate_condition_by_post_types( [ 'post', 'attachment' ] ),
					'side_panel' => true,
				],
			];

			$sidebar_list = apply_filters( 'nebotheme_sidebars_list_default', $sidebar_list );
		}

		return $sidebar_list;
	}

	/**
	 * Sidebar list
	 *
	 * Sidebar Details
	 *
	 * @return array
	 */
	public static function get_sidebars_list() {
		return apply_filters( 'nebotheme_sidebars_list', self::get_sidebars_list_default() );
	}

	/**
	 * Register Sidebars
	 *
	 * @return void
	 */
	public static function register_sidebars() {
		foreach ( self::get_sidebars_list() as $id => $args ) {
			self::register_sidebar(
				$id,
				$args['name'],
				Utils::get_if_isset( $args, 'description', '' )
			);
		}
	}

	/**
	 * Register Sidebar
	 *
	 * @param string $id
	 * @param string $name
	 * @param string $description
	 */
	public static function register_sidebar( $id, $name, $description = '' ) {
		register_sidebar(
			array_merge(
				self::get_default_sidebar_args(),
				[
					'id' => $id,
					'name' => $name,
					'description' => $description,
				]
			)
		);
	}

	/**
	 * Get default sidebar args.
	 */
	public static function get_default_sidebar_args() {
		return [
			'before_widget' => '<div id="%1$s" class="widget %2$s">',
			'after_widget' => '</div>',
			'before_title' => '<h5 class="widget-title"><span>',
			'after_title' => '</span></h5>',
		];
	}

	/**
	 * Get current sidebar id
	 *
	 * @param string $id
	 *
	 * @return string
	 */
	public static function get_id( $id = null ) {
		if ( $id ) {
			return $id;
		}

		return self::get_current_sidebar();
	}

	/**
	 * Retrieve args sidebar
	 *
	 * @param string $id
	 * @param string $arg_name
	 *
	 * @return array
	 */
	public static function get_args( $id = null, $arg_name = '' ) {
		$id = self::get_id( $id );
		$sidebars_list = self::get_sidebars_list();
		$args = Utils::get_if_isset( $sidebars_list, $id );

		if ( $arg_name ) {
			return Utils::get_if_isset( $args, $arg_name );
		}

		return $args;
	}

	/**
	 * Get current theme-mod by name with prefix.
	 */
	public static function get_theme_mod( $id = null ) {
		$id = self::get_id( $id );
		$theme_mod_name = self::get_layout_name( $id );

		if ( ! $theme_mod_name ) {
			return;
		}

		return nebotheme_get_tm( $theme_mod_name );
	}

	/**
	 * Retrieve sidebar layout name.
	 *
	 * @param string $id
	 */
	public static function get_layout_name( $id = null ) {
		return join( '_', array_filter( [ 'sidebar_layout', $id ] ) );
	}

	/**
	 * Check has layout.
	 *
	 * @param string $id
	 */
	public static function is_side_panel( $id = null ) {
		return static::get_args( self::get_id( $id ), 'side_panel' );
	}

	/**
	 * Retrieve current sidebar layout.
	 *
	 * @param string $id
	 */
	public static function get_side_layout( $id = null ) {
		$id = self::get_id( $id );

		if ( ! self::is_side_panel( $id ) ) {
			return false;
		}

		if ( is_singular() ) {
			return Meta_Boxes::get_pm_safe( self::get_layout_name() );
		}

		return self::get_theme_mod( $id );
	}

	/**
	 * Determines whether a sidebar is in use.
	 */
	public static function is_active( $id = null ) {
		$id = self::get_id( $id );

		if ( self::is_side_panel( $id ) ) {
			$is_correct_layout = self::get_side_layout( $id );
		} else {
			$is_correct_layout = true;
		}

		return $is_correct_layout && is_dynamic_sidebar( $id ) && is_active_sidebar( $id );
	}

	/**
	 * Get sidebar by condition.
	 */
	public static function get_sidebar_id_by_condition() {
		foreach ( self::get_sidebars_list() as $sidebar_id => $sidebar_args ) {
			if (
				empty( $sidebar_args['condition_callable'] ) ||
				! is_callable( $sidebar_args['condition_callable'] ) ||
				! call_user_func( $sidebar_args['condition_callable'] )
			) {
				continue;
			}

			return $sidebar_id;
		}
	}

	/**
	 * Retrieve current sidebar args.
	 */
	public static function get_current_sidebar_args() {
		$current_sidebar = self::get_current_sidebar();

		if ( ! $current_sidebar ) {
			return false;
		}

		return Utils::get_if_isset( self::get_sidebars_list(), $current_sidebar );
	}

	/**
	 * Retrieve current sidebar.
	 */
	public static function get_current_sidebar() {
		static $current_sidebar = null;

		if ( null === $current_sidebar ) {
			$current_sidebar = apply_filters( 'nebotheme_current_sidebar', $current_sidebar );

			if ( ! $current_sidebar ) {
				$current_sidebar = self::get_sidebar_id_by_condition();
			}
		}

		return $current_sidebar;
	}

	/**
	 * Echoes sidebar.
	 */
	public static function render( $id = null ) {
		$id = self::get_id( $id );

		if ( ! self::is_active( $id ) ) {
			return;
		}

		static::$rendering_sidebar = true;

		?>
		<aside class="sidebar">
			<div class="sidebar-inner">
				<?php dynamic_sidebar( $id ); ?>
			</div>
		</aside><!-- .sidebar -->
		<?php

		static::$rendering_sidebar = false;
	}
}
