<?php
/**
 * Class widget
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Sidebars\Widgets;

use NeboTheme\YaffoCore\Widget_Base;
use NeboTheme\YaffoCore\Ajax_Select;
use NeboTheme\Yaffo\Classes\WP_Query_Agent;
use NeboTheme\Yaffo\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Widget class Featured_Posts.
 */
class Featured_Posts extends Widget_Base {

	/**
	 * {@inheritDoc}
	 */
	public static function get_name() {
		return 'featured_post';
	}

	/**
	 * {@inheritDoc}
	 */
	public function get_label() {
		return '⛅ ' . esc_html__( 'NeboTheme - Featured Posts', 'yaffo' );
	}

	/**
	 * {@inheritDoc}
	 */
	public function update( $new_instance, $old_instance ) {
		$new_instance['category_show'] = Utils::get_if_isset( $new_instance, 'category_show', '' );
		$new_instance['date_show'] = Utils::get_if_isset( $new_instance, 'date_show', '' );

		$instance = parent::update( $new_instance, $old_instance );

		return $instance;
	}

	/**
	 * {@inheritDoc}
	 */
	public function form( $instance ) {
		$defaults = [
			'title' => esc_html__( 'Featured Posts', 'yaffo' ),
			'showposts' => 3,
			'categories' => [],
			'post_ids' => [],
			'category_show' => 'on',
			'date_show' => 'on',
		];
		$instance = wp_parse_args( (array) $instance, $defaults );
		?>

		<!-- Title -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title', 'yaffo' ); ?>:</label>
			<input  type="text" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" value="<?php echo esc_attr( $instance['title'] ); ?>">
		</p>
		<!-- Category -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'categories' ) ); ?>"><?php esc_html_e( 'Filter by Category', 'yaffo' ); ?>:</label>
			<?php
			Ajax_Select::render_select(
				[
					'type' => 'taxonomy',
					'sub_type' => 'category',
					'value' => $instance['categories'],
				],
				[
					'class' => 'widefat',
					'id' => $this->get_field_id( 'categories' ),
					'name' => $this->get_field_name( 'categories[]' ),
					'multiple' => 'multiple',
				]
			);
			?>
		</p>
		<!-- Post IDs -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'post_ids' ) ); ?>"><?php esc_html_e( 'Find Posts', 'yaffo' ); ?>:</label>
			<?php
			Ajax_Select::render_select(
				[
					'type' => 'post',
					'sub_type' => 'post',
					'value' => $instance['post_ids'],
				],
				[
					'class' => 'widefat',
					'id' => $this->get_field_id( 'post_ids' ),
					'name' => $this->get_field_name( 'post_ids[]' ),
					'multiple' => 'multiple',
				]
			);
			?>
			<small>
				<?php esc_html_e( 'This option overrides the option above in the category.', 'yaffo' ); ?>
			</small>
		</p>
		<!-- Number of posts -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'showposts' ) ); ?>">
				<?php esc_html_e( 'Number of posts to show', 'yaffo' ); ?>:
			</label>
			<input type="number" id="<?php echo esc_attr( $this->get_field_id( 'showposts' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'showposts' ) ); ?>" value="<?php echo esc_attr( $instance['showposts'] ); ?>" class="widefat" min="1" size="3">
		</p>
		<!-- Show Category -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'category_show' ) ); ?>"><?php esc_html_e( 'Show Category', 'yaffo' ); ?>:</label>
			<input type="checkbox" id="<?php echo esc_attr( $this->get_field_id( 'category_show' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'category_show' ) ); ?>" <?php checked( (bool) $instance['category_show'], true ); ?> />
		</p>
		<!-- Show Date -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'date_show' ) ); ?>"><?php esc_html_e( 'Show Date', 'yaffo' ); ?>:</label>
			<input type="checkbox" id="<?php echo esc_attr( $this->get_field_id( 'date_show' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'date_show' ) ); ?>" <?php checked( (bool) $instance['date_show'], true ); ?> />
		</p>
		<?php
	}

	/**
	 * {@inheritDoc}
	 */
	public function render( $args, $instance ) {
		$defaults = [
			'showposts' => 3,
			'categories' => [],
			'category_show' => '',
			'date_show' => '',
			'post_ids' => [],
		];
		$instance = wp_parse_args( (array) $instance, $defaults );
		$post_ids = $instance['post_ids'];
		$query_args = [
			'ignore_sticky_posts' => true,
			'post_status' => 'publish',
			'showposts' => $instance['showposts'],
			'nopaging' => false,
		];

		if ( $post_ids ) {
			$query_args = array_merge(
				$query_args,
				[
					'post_type' => [ 'post' ],
					'post__in' => $post_ids,
					'orderby' => 'post__in',
				]
			);
		} elseif ( $instance['categories'] ) {
			$query_args['cat'] = $instance['categories'];
		}

		if ( is_singular() ) {
			$query_args['post__not_in'] = [ get_the_ID() ];
		}

		$featured_query = new \WP_Query( $query_args );

		if ( ! $featured_query->have_posts() ) {
			nebotheme_render_notification(
				[
					'type' => 'warning',
					'body' => esc_html__( 'Could not find posts.', 'yaffo' ),
				]
			);

			return;
		}

		$slider_option = [];

		if ( $featured_query->found_posts > 1 ) {
			$slider_option['autoplaySpeed'] = 5000;
			$slider_option['autoplay'] = true;
		}

		nebotheme_render_post_slider(
			$featured_query,
			function() use ( $instance ) {
				nebotheme_render_post_open( 'nebotheme-wp-widget-featured-post__item', false );

				/* Content */
				echo '<div class="content-wrap">' .
				'<div class="content-wrap-inner nebotheme-padding-children">';

				if ( $instance['category_show'] ) {
					nebotheme_render_post_meta(
						[
							'category',
						]
					);
				}

				nebotheme_render_post_title(
					[
						'tag' => 'h3',
						'title_attr' => true,
					]
				);

				if ( $instance['date_show'] ) {
					nebotheme_render_post_meta(
						[
							'date',
						]
					);
				}

				echo '</div>' .
				'</div>';

				/* Thumbnail */
				if ( has_post_thumbnail() ) {
					nebotheme_render_thumbnail(
						[
							'permalink' => false,
							'size' => 'yaffo-square',
							'bg_image' => true,
						]
					);
				}

				nebotheme_render_post_close();
			},
			$slider_option
		);
	}
}
