<?php
/**
 * Class widget
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Sidebars\Widgets;

use NeboTheme\YaffoCore\Widget_Base;
use NeboTheme\YaffoCore\Ajax_Select;
use NeboTheme\Yaffo\Classes\WP_Query_Agent;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Widget class Post_List.
 */
class Post_List extends Widget_Base {

	/**
	 * {@inheritDoc}
	 */
	public static function get_name() {
		return 'post_list';
	}

	/**
	 * {@inheritDoc}
	 */
	public function get_label() {
		return '⛅ ' . esc_html__( 'NeboTheme - Latest Posts', 'yaffo' );
	}

	/**
	 * {@inheritDoc}
	 */
	public function update( $new_instance, $old_instance ) {
		if ( ! isset( $new_instance['show_date'] ) ) {
			$new_instance['show_date'] = '';
		}

		$instance = parent::update( $new_instance, $old_instance );

		return $instance;
	}

	/**
	 * {@inheritDoc}
	 */
	public function form( $instance ) {
		$defaults = [
			'title' => esc_html__( 'Latest Posts', 'yaffo' ),
			'showposts' => 3,
			'categories' => [],
			'show_date' => 'on',
		];
		$instance = wp_parse_args( (array) $instance, $defaults );
		?>

		<!-- Title -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title', 'yaffo' ); ?>:</label>
			<input  type="text" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" value="<?php echo esc_attr( $instance['title'] ); ?>">
		</p>
		<!-- Category -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'categories' ) ); ?>"><?php esc_html_e( 'Filter by Category', 'yaffo' ); ?>:</label>
			<?php
			Ajax_Select::render_select(
				[
					'type' => 'taxonomy',
					'sub_type' => 'category',
					'value' => $instance['categories'],
				],
				[
					'class' => 'widefat',
					'id' => $this->get_field_id( 'categories' ),
					'name' => $this->get_field_name( 'categories[]' ),
					'multiple' => 'multiple',
				]
			);
			?>
		</p>
		<!-- Number of posts -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'showposts' ) ); ?>">
				<?php esc_html_e( 'Number of posts to show', 'yaffo' ); ?>:
			</label>
			<input type="number" id="<?php echo esc_attr( $this->get_field_id( 'showposts' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'showposts' ) ); ?>" value="<?php echo esc_attr( $instance['showposts'] ); ?>" class="widefat" min="1" size="3">
		</p>
		<!-- Show Date -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_date' ) ); ?>">
				<?php esc_html_e( 'Display post date?', 'yaffo' ); ?>
			</label>
			<input class="checkbox" type="checkbox"<?php checked( (bool) $instance['show_date'] ); ?> id="<?php echo esc_attr( $this->get_field_id( 'show_date' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'show_date' ) ); ?>" />
		</p>
		<?php
	}

	/**
	 * {@inheritDoc}
	 */
	public function render( $args, $instance ) {
		$defaults = [
			'showposts' => 3,
			'categories' => '',
			'show_date' => 'on',
		];
		$instance = array_replace_recursive( $defaults, (array) $instance );
		$args_posts = [
			'ignore_sticky_posts' => true,
			'nopaging' => false,
			'post_status' => 'publish',
			'showposts' => $instance['showposts'],
		];

		if ( $instance['categories'] && 'all' !== $instance['categories'] ) {
			$args_posts['cat'] = $instance['categories'];
		}

		$loop = new \WP_Query( $args_posts );

		if ( ! $loop->have_posts() ) {
			nebotheme_render_notification(
				[
					'type' => 'warning',
					'body' => esc_html__( 'Could not find posts.', 'yaffo' ),
				]
			);

			return;
		}

		WP_Query_Agent::loop_start();

		echo '<div class="nebotheme-widget-post-list">';

		while ( $loop->have_posts() ) {
			$loop->the_post();

			nebotheme_render_post_open( 'nebotheme-widget-post-list__item', false );

			/* Content */
			echo '<div class="nebotheme-widget-post-list__content">';

			if ( $instance['show_date'] ) {
				nebotheme_render_post_meta(
					[
						'date',
					]
				);
			}

			nebotheme_render_post_title(
				[
					'tag' => 'h4',
				]
			);

			echo '</div>';

			echo '<div class="nebotheme-widget-post-list__image-wrap">';

			nebotheme_render_thumbnail(
				[
					'size' => 'yaffo-small-square',
					'placeholder' => true,
				]
			);

			echo '</div>';

			nebotheme_render_post_close();
		}

		echo '</div>';

		WP_Query_Agent::loop_end();
	}
}
