<?php

/**
 * Template Functions for posts.
 *
 * Functions components.
 *
 * @package Yaffo
 */

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Classes\WP_Query_Agent;
use NeboTheme\Yaffo\Blog_Main;
use NeboTheme\Yaffo\Post\Post;
use NeboTheme\Yaffo\Utils;
use NeboTheme\YaffoCore\Lazy_Load;
use NeboTheme\Yaffo\Homepage;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Echoes blog
 */
function nebotheme_render_main_blog() {
	Blog_Main::instance()->render();
}

/**
 * Echoes category for blog
 */
function nebotheme_render_post_meta_category() {
	nebotheme_render_taxonomy(
		[
			'taxonomy' => 'category',
		]
	);
}

/**
 * Echoes taxonomy for blog
 *
 * @param array $args
 */
function nebotheme_render_taxonomy( array $args ) {
	$args_default = [
		'separator' => '',
		'taxonomy' => 'category',
	];

	$args = array_merge( $args_default, $args );
	$post = get_post();
	$post_id = $post->ID;

	if ( 'category' === $args['taxonomy'] && 'product' === $post->post_type ) {
		$args['taxonomy'] = 'product_cat';
	}

	switch ( $args['taxonomy'] ) {
		case 'category':
			$terms = get_the_category( $post_id );
			break;

		case 'post_tag':
			$terms = get_the_tags( $post_id );
			break;

		default:
			$terms = get_the_terms( $post_id, $args['taxonomy'] );
			break;
	}

	if ( is_wp_error( $terms ) || empty( $terms ) ) {
		return;
	}

	$taxonomy = get_taxonomy( $args['taxonomy'] );
	$taxonomy_attr = $args['taxonomy'];

	if ( 'product_cat' === $taxonomy_attr ) {
		$taxonomy_attr = 'category';
	}

	?>
	<div class="nebotheme-meta nebotheme-meta-taxonomy" data-taxonomy="<?php echo esc_attr( $taxonomy_attr ); ?>" title="<?php echo esc_attr( $taxonomy->label ); ?>">
		<div class="nebotheme-meta-taxonomy__inner">
			<?php
			foreach ( $terms as $term ) {
				$link = get_term_link( $term, $args['taxonomy'] );

				if ( is_wp_error( $link ) ) {
					break;
				}
				?>
				<a href="<?php echo esc_url( $link ); ?>" class="term">
					<?php echo esc_html( $term->name ); ?>
				</a>
				<?php
			}
			?>
		</div>
	</div>
	<?php
}

/**
 * Retrieve the post title.
 */
function nebotheme_get_post_title() {
	$title = get_the_title();

	if ( ! $title ) {
		$title = '(' . esc_html__( 'no title', 'yaffo' ) . ')';
	}

	return $title;
}

/**
 * Echoes post title
 *
 * @param array $args
 */
function nebotheme_render_post_title( array $args = [] ) {
	$args_default = [
		'tag' => 'h2',
		'text' => false,
		'href' => null,
		'adaptive' => false,
		'title_attr' => false,
	];

	$args = array_merge( $args_default, $args );
	$heading_classes = 'entry-title';

	if ( $args['adaptive'] ) {
		$heading_classes .= ' nebotheme-font-adaptive';
	}

	if ( 'h1' === $args['tag'] ) {
		$args['tag'] = nebotheme_get_tag_h1( $args['tag'] );
	}

	if ( ! $args['text'] ) {
		$args['text'] = get_the_title();
	}

	if ( ! $args['text'] ) {
		$args['text'] = '(' . esc_html__( 'no title', 'yaffo' ) . ')';
	}

	if ( null === $args['href'] ) {
		$args['href'] = get_permalink();
	}

	echo '<div class="nebotheme-post__title">' .
		'<' . esc_attr( $args['tag'] ) . ' class="' . esc_attr( $heading_classes ) . '">';

	if ( $args['href'] ) {
		$link_attrs = [
			'href' => $args['href'],
		];

		if ( $args['title_attr'] ) {
			$link_attrs['title'] = nebotheme_get_post_title();
		}

		echo '<a ' . nebotheme_get_render_attrs( $link_attrs ) . '>';
	}

	echo '<span>' . nebotheme_get_post_title() . '</span>';

	if ( $args['href'] ) {
		echo '</a>';
	}

	echo '</' . esc_attr( $args['tag'] ) . '>' .
		'</div>';
}

/**
 * Echoes date for blog
 */
function nebotheme_render_post_meta_date() {
	$date = get_the_date();

	echo '<div class="nebotheme-meta nebotheme-meta-date">' .
		'<span>' .
			esc_html( $date ) .
		'</span>' .
	'</div>';
}

/**
 * Echoes update data for blog
 */
function nebotheme_render_post_meta_update_date() {
	$post = get_post();
	$modified_date = get_the_modified_date();

	if ( $post->post_date === $post->post_modified ) {
		nebotheme_render_post_meta_date();

		return;
	}

	echo '<div class="nebotheme-meta nebotheme-meta-date">' .
		'<span>' .
			esc_html__( 'Last updated on', 'yaffo' ) . ' ' . esc_html( $modified_date ) .
		'</span>' .
	'</div>';
}

/**
 * Echoes author for blog.
 */

if ( ! function_exists( 'nebotheme_render_post_meta_author' ) ) {
	function nebotheme_render_post_meta_author() {
		$user_id = nebotheme_get_author_id();

		if ( ! $user_id ) {
			return;
		}

		$display_name = get_the_author_meta( 'display_name', $user_id );
		$author_url = get_author_posts_url( $user_id );

		echo '<div class="nebotheme-meta nebotheme-meta-author">' .
			'<a href="' . esc_url( $author_url ) . '">' .
			esc_html( $display_name ) .
			'</a>' .
			'</div>';
	}
}

/**
 * Echoes read time for blog
 */
function nebotheme_render_post_meta_read_time() {
	$read_time = Post::get_read_time();

	echo '<div class="nebotheme-meta nebotheme-meta-read-time">' .
		esc_html(
			sprintf(
				/* translators: count minutes */
				_n(
					'%s Min read',
					'%s Mins read',
					$read_time,
					'yaffo'
				),
				$read_time
			)
		) .
		'</div>';
}

/**
 * Echoes comments for blog
 */
function nebotheme_render_post_meta_comments() {
	$comments_number = (int) get_comments_number();
	$is_link = ( $comments_number || comments_open() );
	$tag = $is_link ? 'a' : 'span';
	$attrs = [
		'class' => 'comments-count',
	];

	if ( $is_link ) {
		$attrs['href'] = get_comments_link();
	}

	echo '<div class="nebotheme-meta nebotheme-meta-comments">' .
		'<' . tag_escape( $tag ) . ' ' . nebotheme_get_render_attrs( $attrs ) . '>';

	if ( $comments_number ) {
		printf(
			esc_html(
				/* translators: number of comments */
				_nx(
					'%1$s Comment',
					'%1$s Comments',
					$comments_number,
					'comments',
					'yaffo'
				)
			),
			esc_html( number_format_i18n( $comments_number ) )
		);
	} else {
		esc_html_e( 'No Comments', 'yaffo' );
	}

	echo '</' . tag_escape( $tag ) . '>' .
		'</div>';
}

/**
 * Check before render post meta
 *
 * @param string $meta_item
 */
function nebotheme_check_post_meta( $meta_item ) {
	global $wp_taxonomies;

	$post = get_post();

	if ( 'category' === $meta_item && 'product' === $post->post_type ) {
		$meta_item = 'product_cat';
	}

	$taxonomies = array_keys( nebotheme_wp_list_pluck( $wp_taxonomies, 'name', 'name' ) );

	if ( in_array( $meta_item, $taxonomies, true ) ) {
		$terms = get_the_terms( get_the_ID(), $meta_item );

		if ( is_wp_error( $terms ) || empty( $terms ) ) {
			return false;
		}
	} elseif ( 'author' === $meta_item ) {
		return (bool) nebotheme_get_author_id();
	}

	return true;
}

/**
 * Echoes post meta
 *
 * @param array $meta_items
 */
function nebotheme_render_post_meta( $meta_items ) {
	if ( ! is_array( $meta_items ) ) {
		return;
	}

	if ( $meta_items && Blog_Main::is_active() ) {
		$meta_data_excludes = nebotheme_get_exclude_post_meta();

		if ( $meta_data_excludes ) {
			$meta_items = array_diff( $meta_items, $meta_data_excludes );
		}
	}

	if ( empty( $meta_items ) ) {
		return;
	}

	static $meta_callbacks = null;

	if ( null === $meta_callbacks ) {
		$meta_callbacks = array_merge( apply_filters( 'nebotheme_metadata_types_callbacks', [] ), [
			'author' => 'nebotheme_render_post_meta_author',
			'category' => 'nebotheme_render_post_meta_category',
			'comments' => 'nebotheme_render_post_meta_comments',
			'date' => 'nebotheme_render_post_meta_date',
			'update_date' => 'nebotheme_render_post_meta_update_date',
			'post_tag' => 'nebotheme_render_post_meta_tag',
			'post_type' => 'nebotheme_render_post_meta_post_type',
			'read_time' => 'nebotheme_render_post_meta_read_time',
		]);
	}

	$meta_items_callback = [];

	foreach ( $meta_items as $index => $value ) {
		if ( is_numeric( $index ) ) {
			$callback_parameters = [];
			$meta_item = $value;
		} else {
			$callback_parameters = [ $value ];
			$meta_item = $index;
		}

		$callback = Utils::get_if_isset( $meta_callbacks, $meta_item );

		if ( ! $callback || ! is_callable( $callback ) || ! nebotheme_check_post_meta( $meta_item ) ) {
			continue;
		}

		$meta_items_callback[ $meta_item ] = [
			$callback,
			$callback_parameters,
		];
	}

	if ( ! $meta_items_callback ) {
		return;
	}

	echo '<div class="nebotheme-post__meta">' .
		'<div class="nebotheme-post__meta__inner">';

	foreach ( $meta_items_callback as $meta_item => $callback_data ) {
		list($callback, $callback_parameters) = $callback_data;

		echo '<div class="nebotheme-post__meta__item" data-type="' . esc_attr( $meta_item ) . '">';

		call_user_func_array( $callback, $callback_parameters );

		echo '</div>';
	}

	echo '</div>' .
		'</div>';
}

/**
 * Echoes post thumbnail
 *
 * @param array $args
 */
function nebotheme_render_thumbnail( array $args = [] ) {
	static $args_default = [
		'size' => 'full',
		'permalink' => true,
		'zoomable' => false,
		'lazyload' => true,
		'bg_image' => false,
		'placeholder' => false,
	];

	$args = apply_filters( 'nebotheme_render_thumbnail_args', array_merge( $args_default, $args ) );
	$thumbnail_id = get_post_thumbnail_id();
	$has_post_thumbnail = has_post_thumbnail() && wp_get_attachment_image_src( $thumbnail_id );
	$is_rendering = $has_post_thumbnail || $args['placeholder'];

	if ( $args['placeholder'] ) {
		$placeholder_url = nebotheme_get_placeholder_image_url([
			'color_name_bg' => 'alt',
			'size' => $args['size'],
		]);
	}

	if ( ! $is_rendering ) {
		return;
	}

	$post = get_post();

	if ( $args['lazyload'] && ! nebotheme_is_custom_lazyload() ) {
		if ( $args_default['lazyload'] ) {
			$args_default['lazyload'] = false;
		}

		$args['lazyload'] = false;
	}

	$attrs = [
		'class' => [ 'nebotheme-post__image' ],
	];

	$img_css_class = 'nebotheme-post__image__item';

	if ( $args['permalink'] ) {
		$permalink = get_permalink();
	} else {
		$permalink = false;
	}

	if ( $args['bg_image'] ) {
		$attrs['class'][] = 'nebotheme-post__image--bg-image';

		if ( $has_post_thumbnail ) {
			$bg_src = get_the_post_thumbnail_url( $post, $args['size'] );
			$attachment_metadata = get_post_meta( $thumbnail_id, '_wp_attachment_metadata', true );
		} elseif ( $args['placeholder'] && $placeholder_url ) {
			$bg_src = $placeholder_url;
		}

		if ( isset( $attachment_metadata ) && ! empty( $attachment_metadata ) && is_array( $attachment_metadata ) ) {
			if ( 'full' === $args['size'] ) {
				$image_size = [
					'width' => $attachment_metadata['width'],
					'height' => $attachment_metadata['height'],
				];
			} else {
				$image_size = Utils::get_if_isset( $attachment_metadata['sizes'], $args['size'] );
			}
		}

		if ( empty( $image_size ) ) {
			$image_size_current = Utils::get_image_size( $args['size'] );

			if ( $image_size_current && isset( $image_size_current['height'] ) && 9999 !== $image_size_current['height'] && 0 !== $image_size_current ) {
				$image_size = $image_size_current;
			}
		}

		if ( isset( $image_size ) && isset( $image_size['width'] ) && isset( $image_size['height'] ) ) {
			$ratio = floor( 100 * ( $image_size['height'] / $image_size['width'] ) );

			$attrs['style'] = '--nebotheme-ratio: ' . esc_attr( $ratio ) . '%;';
		}
	}

	echo '<figure ' . nebotheme_get_render_attrs( $attrs ) . '>';

	echo '<div class="nebotheme-post__image__inner">';

	if ( $permalink ) {
		echo '<a href="' . esc_url( $permalink ) . '" class="nebotheme-post__image__link"></a>';
	}

	if ( $args['bg_image'] && isset( $bg_src ) ) {
		$bg_attrs = [
			'class' => [
				'nebotheme-post__image__bg',
				$img_css_class,
			],
		];

		if ( $args['lazyload'] ) {
			$bg_attrs['data-bg'] = esc_url( $bg_src );
			$bg_attrs['class'][] = Lazy_Load::CSS_CLASSES;
		} else {
			$bg_attrs['style'] = 'background-image: url("' . esc_url( $bg_src ) . '")';
		}

		echo '<div ' . nebotheme_get_render_attrs( $bg_attrs ) . '>' .
			'<div class="nebotheme-post__image__bg__inner"></div>' .
			'</div>';
	} else {
		$thumbnail_attrs = [
			'class' => $img_css_class,
		];

		if ( $args['zoomable'] && ! $args['placeholder'] && nebotheme_get_tm( 'zoomable' ) ) {
			$thumbnail_attrs['data-zoomable'] = 'true';

			if ( ! empty( $args['size'] ) && 'full' !== $args['size'] ) {
				$thumbnail_attrs['data-zoom-src'] = esc_url( get_the_post_thumbnail_url( $post, 'full' ) );
			}
		}

		if ( ! $args['lazyload'] ) {
			if ( nebotheme_is_custom_lazyload() ) {
				$thumbnail_attrs['class'] .= ' nebotheme-lazyload-disabled';
			}

			$thumbnail_attrs['loading'] = 'eager';
		}

		if ( get_the_post_thumbnail( $post, $args['size'], $thumbnail_attrs ) ) {
			echo get_the_post_thumbnail( $post, $args['size'], $thumbnail_attrs );
		} elseif ( $args['placeholder'] ) {
			$placeholder_attrs = [
				'src' => $placeholder_url,
				'class' => [
					$img_css_class,
				],
			];

			if ( $args['lazyload'] ) {
				$placeholder_attrs['class'][] = Lazy_Load::CSS_CLASSES;
			}

			echo '<img ' . nebotheme_get_render_attrs( $placeholder_attrs ) . '>';
		}
	}

	if ( $args['lazyload'] ) {
		Lazy_Load::render_animation();
	}

	if ( nebotheme_is_post_video_bg() ) {
		$video_start = Meta_Boxes::get_pm( 'video_bg_start' );
		$video_end = Meta_Boxes::get_pm( 'video_bg_end' );

		$bg_video_attrs = [
			'class' => 'nebotheme-post__video-bg',
			'data-video-id' => nebotheme_get_youtube_video_id(),
		];

		if ( $video_start ) {
			$bg_video_attrs['data-video-start'] = $video_start;
		}

		if ( $video_end ) {
			$bg_video_attrs['data-video-end'] = $video_end;
		}

		echo '<div ' . nebotheme_get_render_attrs( $bg_video_attrs ) . '>' .
			'<div class="nebotheme-post__video-bg__inner"></div>' .
			'<div class="nebotheme-post__video-bg__interface">' .
			'<a href="#" class="nebotheme-post__video-bg__control nebotheme-post__video-bg__control_volume">' .
			'<i class="nebotheme-post__video-bg__control_state__mute nebotheme-icon-mute nebotheme-icon-margin"></i>' .
			'<i class="nebotheme-post__video-bg__control_state__unmute nebotheme-icon-unmute nebotheme-icon-margin"></i>' .
			'</a>' .
			'<a href="#" class="nebotheme-post__video-bg__control nebotheme-post__video-bg__control_state">' .
			'<i class="nebotheme-post__video-bg__control_state__play nebotheme-icon-play nebotheme-icon-margin"></i>' .
			'<i class="nebotheme-post__video-bg__control_state__pause nebotheme-icon-pause nebotheme-icon-margin"></i>' .
			'</a>' .
			'</div>' .
			'</div>';
	}

	if ( is_search() && Blog_Main::is_active() ) {
		nebotheme_render_post_meta_post_type();
	}

	echo '</div>' .
	'</figure>';
}

/**
 * Echoes post (etc. thumbnail, gallery, video)
 *
 * @param array $args
 */
function nebotheme_render_post_single_view( array $args = [] ) {
	global $content_width;

	$args_default = [
		'thumbnail' => [
			'size' => 'full',
			'permalink' => false,
			'zoomable' => true,
		],
		'video' => [
			'width' => $content_width,
		],
		'gallery' => [
			'size' => 'full',
		],
	];
	$args = array_replace_recursive( $args_default, $args );
	$type = '';

	if ( nebotheme_render_post_is_gallery() ) {
		$type = 'gallery';
	} elseif ( nebotheme_render_post_is_video() ) {
		$type = 'video';
	} elseif ( has_post_thumbnail() ) {
		$type = 'thumbnail';

		if ( ! Meta_Boxes::get_pm_safe( 'featured_image_show' ) ) {
			return;
		}
	}

	if ( ! $type ) {
		return;
	}

	?>
	<div class="nebotheme-post-single-view">
		<div class="nebotheme-post-single-view__inner">
			<?php
			switch ( $type ) {
				case 'gallery':
					nebotheme_render_post_gallery( $args['gallery'] );
					break;
				case 'video':
					nebotheme_render_post_video( $args['video'] );
					break;
				case 'thumbnail':
				default:
					nebotheme_render_thumbnail( $args['thumbnail'] );
					break;
			}
			?>
		</div>
	</div>
	<?php
}

/**
 * Check before render gallery
 */
function nebotheme_render_post_is_gallery() {
	return ( 'gallery' === get_post_format() &&
		! empty( Meta_Boxes::get_pm_for_frontend( 'gallery_images' ) )
	);
}

/**
 * Check before render video
 */
function nebotheme_render_post_is_video() {
	 return ( 'video' === get_post_format() &&
		! empty( Meta_Boxes::get_pm_safe( 'video_url' ) )
	);
}

/**
 * Echoes post video
 *
 * @param array $attrs
 */
function nebotheme_render_post_video( array $attrs = [] ) {
	if ( ! nebotheme_render_post_is_video() ) {
		return;
	}

	echo '<div class="nebotheme-post-video">';

	nebotheme_render_embed(
		Meta_Boxes::get_pm_safe( 'video_url' ),
		$attrs
	);

	echo '</div>';
}

/**
 * Echoes post gallery
 *
 * @param array $args
 */
function nebotheme_render_post_gallery( array $args = [] ) {
	if ( ! nebotheme_render_post_is_gallery() ) {
		return;
	}

	$image_ids = Meta_Boxes::get_pm_for_frontend( 'gallery_images' );

	$args_default = [
		'size' => 'full',
	];
	$args = array_merge( $args_default, $args );

	echo '<div class="nebotheme-post-gallery">';

	nebotheme_render_slider(
		$image_ids,
		function ( $image_id ) use ( $args ) {
			echo '<div class="nebotheme-post-gallery__item">';

			nebotheme_render_attachment( [
				'id' => $image_id,
				'size' => $args['size'],
			] );

			echo '</div>';
		},
		[
			'adaptiveHeight' => true,
			'arrows' => true,
			'autoplay' => true,
			'autoplaySpeed' => 5000,
			'fade' => true,
		]
	);

	echo '</div>';
}

/**
 * Echoes tag for blog
 */
function nebotheme_render_post_meta_tag() {
	nebotheme_render_taxonomy(
		[
			'taxonomy' => 'post_tag',
		]
	);
}

/**
 * Echoes post type for blog
 */
function nebotheme_render_post_meta_post_type() {
	$post_type = get_post_type();
	$post_type_obj = get_post_type_object( $post_type );
	$post_type_name = isset( $post_type_obj->label ) ? $post_type_obj->label : ucwords( preg_replace( '/-|_/', ' ', $post_type ) );

	$url = get_post_type_archive_link( $post_type );

	echo '<span class="nebotheme-post__post-type">';

	if ( $url ) {
		echo '<a href="' . esc_url( $url ) . '">';
	}

	echo esc_html( $post_type_name );

	if ( $url ) {
		echo '</a>';
	}

	echo '</span>';
}

/**
 * Echoes share buttons
 *
 * @param array $args
 */
function nebotheme_render_share_buttons( array $args = [] ) {
	if ( ! nebotheme_is_core_active() ) {
		return;
	}

	yaffo_core_render_share_buttons( $args );
}

/**
 * Echoes share buttons.
 */
function nebotheme_render_share_buttons_single() {
	nebotheme_render_share_buttons();
}

/**
 * Echo render post excerpt
 */
function nebotheme_render_excerpt() {
	if ( ! get_the_excerpt() ) {
		return;
	}

	if (
		(
			Homepage::is_blog_enable() &&
			in_array( Blog_Main::get_current_layout(), [ 'list', 'grid' ], true ) &&
			! Homepage::get_settings( 'homepage_blog_excerpt' )
		) ||
		(
			is_search() &&
			in_array( Blog_Main::get_current_layout(), [ 'grid' ], true ) &&
			! nebotheme_get_tm( 'search_blog_excerpt' )
		) ||
		(
			is_archive() &&
			in_array( Blog_Main::get_current_layout(), [ 'grid' ], true ) &&
			! nebotheme_get_tm( 'archive_blog_excerpt' )
		)
	) {
		return;
	}

	?>
	<div class="nebotheme-post__excerpt">
		<?php the_excerpt(); ?>
	</div>
	<?php
}

/**
 * Echoes render author box post
 */
if ( ! function_exists( 'nebotheme_render_post_author_box' ) ) {
	function nebotheme_render_post_author_box() {
		if ( ! nebotheme_is_render_author_box() ) {
			return;
		}

		$id = get_the_author_meta( 'ID' );
		$socials = nebotheme_get_social_author();

		$heading_tag = 'h3';

		if ( is_author() ) {
			$heading_tag = nebotheme_get_tag_h1( $heading_tag );
		}

		?>
		<div class="nebotheme-author-box">
			<div class="author-content-left">
				<div class="author-avatar">
					<a href="<?php echo esc_url( get_author_posts_url( $id ) ); ?>" rel="author">
						<?php echo get_avatar( $id, '80' ); ?>
					</a>
				</div>
				<div class="author-name">
					<div class="author-name__before"><?php esc_html_e( 'Written by', 'yaffo' ); ?></div>
					<<?php echo esc_attr( $heading_tag ); ?> class="author-name__title">
						<a href="<?php echo esc_url( get_author_posts_url( $id ) ); ?>" rel="author">
							<?php the_author_meta( 'display_name', $id ); ?>
						</a>
					</<?php echo esc_attr( $heading_tag ); ?>>
				</div>
			</div>
			<div class="author-content-right">
				<div class="author-bio"><?php the_author_meta( 'description', $id ); ?></div>
				<div class="author-links">
					<?php nebotheme_render_social_icons( $socials ); ?>
				</div>
			</div>
		</div>
		<?php
	}
}

/**
 * Echoes read more button
 */
function nebotheme_render_post_read_more() {
	echo '<div class="nebotheme-post-read-more">';

	nebotheme_render_button(
		[
			'after' => '<i class="nebotheme-icon-right-open-1 nebotheme-icon-margin"></i>',
			'href' => get_permalink(),
			'text' => esc_html__( 'Continue Reading', 'yaffo' ),
			'title' => the_title_attribute( [ 'echo' => false ] ),
		]
	);

	echo '</div>';
}

/**
 * Echoes post pagination
 *
 * @param array $args
 */
function nebotheme_render_pagination_single( array $args = [] ) {
	if ( ! nebotheme_get_tm( 'prev_next_pagination' ) ) {
		return;
	}

	$args_default = [
		'excluded_terms' => '',
		'in_same_term' => false,
		'taxonomy' => 'category',
	];
	$args = array_merge( $args_default, $args );
	$post_type = get_post_type();
	$published_posts = wp_count_posts( $post_type )->publish;

	if ( $published_posts < 1 ) {
		return;
	}

	$link_args = [
		'%link',
		'%title',
		$args['in_same_term'],
		$args['excluded_terms'],
		$args['taxonomy'],
	];

	$link_previous = get_previous_post_link( ...$link_args );
	$link_next = get_next_post_link( ...$link_args );

	$items = [
		[
			'name' => 'previous',
			'label' => esc_html__( 'Previous Post', 'yaffo' ),
			'link' => $link_previous,
		],
		[
			'name' => 'next',
			'label' => esc_html__( 'Next Post', 'yaffo' ),
			'link' => $link_next,
		],
	];

	?>
	<aside class="nebotheme-post-single-pagination">
		<?php
		foreach ( $items as $item ) {
			if ( ! $item['link'] ) {
				continue;
			}

			?>
			<div class="nebotheme-post-single-pagination__item <?php echo esc_attr( $item['name'] ); ?>">
				<div class="description">
					<span><?php echo esc_html( $item['label'] ); ?></span>
				</div>
				<h3 class="link-wrap"><?php echo nebotheme_get_kses_content( $item['link'] ); ?></h3>
			</div>
			<?php
		}
		?>
	</aside>
	<?php
}

/**
 * Echoes related posts
 */
function nebotheme_render_related() {
	if ( ! is_singular( 'post' ) || ! nebotheme_get_tm( 'homepage_related_posts_single' ) ) {
		return;
	}

	$post = get_post();
	$tags_ids = nebotheme_wp_list_pluck( get_the_tags(), 'term_id' );
	$category_ids = nebotheme_wp_list_pluck( get_the_category(), 'term_id' );
	$query_args = [
		'ignore_sticky_posts' => true,
		'order' => 'DESC',
		'post_status' => 'publish',
		'orderby' => 'comment_count',
		'no_found_rows' => true,
		'post__not_in' => [ $post->ID ],
		'posts_per_page' => 3,
		'meta_query' => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
			[
				'relation' => 'OR',
				[
					'key' => '_thumbnail_id',
					'compare' => 'NOT EXISTS',
				],
				[
					'key' => '_thumbnail_id',
					'compare' => '!NOT EXISTS',
				],
			],
		],
	];

	if (
		$category_ids &&
		count( $category_ids ) === 1 &&
		Utils::get_default_category() === $category_ids[0]
	) {
		unset( $category_ids[0] );
	}

	if ( $category_ids && $tags_ids ) {
		$query_args['tax_query'] = [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_tax_query
			'relation' => 'OR',
		];

		if ( $category_ids ) {
			$query_args['tax_query'][] = [
				'field' => 'term_id',
				'include_children' => true,
				'operator' => 'IN',
				'taxonomy' => 'category',
				'terms' => $category_ids,
			];
		}

		if ( $tags_ids ) {
			$query_args['tax_query'][] = [
				'field' => 'term_id',
				'operator' => 'IN',
				'taxonomy' => 'post_tag',
				'terms' => $tags_ids,
			];
		}
	} elseif ( $tags_ids ) {
		$query_args['tag__in'] = $tags_ids;
	} elseif ( $category_ids ) {
		$query_args['category__in'] = $category_ids;
	}

	$similar_query = new \WP_Query( $query_args );

	if ( $similar_query->query['posts_per_page'] > $similar_query->post_count ) {
		$author_query = new \WP_Query( [
			'author' => $post->post_author,
			'post__not_in' => [ $post->ID ],
			'posts_per_page' => $similar_query->query['posts_per_page'] - $similar_query->post_count,
			'ignore_sticky_posts' => $similar_query->query['ignore_sticky_posts'],
			'order' => $similar_query->query['order'],
			'post_status' => $similar_query->query['post_status'],
			'orderby' => $similar_query->query['orderby'],
			'meta_query' => $similar_query->query['meta_query'],
			'no_found_rows' => $similar_query->query['no_found_rows'],
			'post__not_in' => $similar_query->query['post__not_in'],
		] );

		$related_query = new \WP_Query( [
			'post__in' => wp_list_pluck( array_merge( $similar_query->posts, $author_query->posts ), 'ID' ),
			'orderby' => 'post__in',
		] );
	} else {
		$related_query = $similar_query;
	}

	if ( ! $related_query->have_posts() ) {
		return;
	}

	?>
	<div class="nebotheme-related ta-c">
		<div class="grid-container">
			<h6 class="nebotheme-related__title">
				<?php esc_html_e( 'You might also like', 'yaffo' ); ?>
			</h6>
			<?php
			nebotheme_render_post_slider(
				$related_query,
				function () {
					nebotheme_render_post_open( 'nebotheme-post--blog nebotheme-post--grid' );

					nebotheme_render_thumbnail(
						[
							'size' => 'yaffo-grid',
						]
					);

					nebotheme_render_post_meta( [ 'category' ] );

					nebotheme_render_post_title(
						[
							'tag' => 'h3',
						]
					);

					nebotheme_render_post_meta(
						[
							'read_time',
							'date',
						]
					);

					nebotheme_render_excerpt();

					nebotheme_render_post_close();
				},
				[
					'adaptiveHeight' => true,
					'slidesToScroll' => 3,
					'slidesToShow' => 3,
					'arrows' => true,
				]
			);
			?>
		</div>
	</div>
	<?php
}

add_action( 'nebotheme_render_var_block_end', 'nebotheme_render_related' );

/**
 * Echoes opening post
 *
 * @param string $css_class
 * @param bool $render_id
 */
function nebotheme_render_post_open( $css_class = '', $render_id = true ) {
	$css_class = 'nebotheme-post' . ( $css_class ? " {$css_class}" : '' );
	$attrs = [
		'class' => get_post_class( $css_class ),
	];

	if ( $render_id ) {
		$attrs['id'] = 'post-' . get_the_ID();
	}

	echo '<article ' . nebotheme_get_render_attrs( $attrs ) . '>' .
		'<div class="nebotheme-post__inner">';
}

/**
 * Echoes closing post
 */
function nebotheme_render_post_close() {
	echo '</div>' .
		'</article>';
}

/**
 * Echoes single post/page
 */
function nebotheme_render_single() {
	WP_Query_Agent::loop_start();

	while ( have_posts() ) {
		the_post();

		get_template_part( 'template-parts/blog/post-single' );
	}

	WP_Query_Agent::loop_end();
}
