<?php
/**
 * Module Additional Content.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Additional_Content;

use NeboTheme\Yaffo\Modules\Base_Module;
use NeboTheme\Yaffo\Customize\Customize_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Module Additional Content class.
 */
final class Module extends Base_Module {
	const PREFIX_SETTING = 'add_content_field_';

	/**
	 * Get url to documentation.
	 *
	 * @return string
	 */
	public static function get_url_docs() {
		return nebotheme_get_url_docs_theme() . '/additional-content-ads';
	}

	/**
	 * Array with positions data.
	 *
	 * @var array
	 */
	public static $positions = [];

	/**
	 * {@inheritDoc}
	 */
	protected function __construct() {
		self::$positions = [
			'before' => esc_html__( 'Before', 'yaffo' ),
			'after' => esc_html__( 'After', 'yaffo' ),
		];

		parent::__construct();
	}

	/**
	 * {@inheritDoc}
	 */
	protected function add_hooks() {
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );
		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
		add_action( 'wp_enqueue_scripts', [ $this, 'add_script' ] );
		add_action( 'init', [ $this, 'add_filters_for_render' ] );
	}

	/**
	 * Add filters for render.
	 */
	public function add_filters_for_render() {
		foreach ( $this->get_locations() as $location_slug => $location ) {
			foreach ( self::$positions as $position => $position_label ) {
				$action_tag = "nebotheme_render_{$location_slug}_{$position}";

				add_action(
					$action_tag,
					function() use ( $location, $location_slug, $position ) {
						$this->render( $location, $location_slug, $position );
					}
				);
			}
		}
	}

	/**
	 * Handles JavaScript and CSS detection.
	 */
	public function enqueue_scripts() {
		if ( nebotheme_get_tm( 'enable_adsense' ) ) {
			wp_register_script(
				'nebotheme-adsense',
				'//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js',
				[],
				wp_get_theme( 'yaffo' )->version,
				true
			);

			wp_enqueue_script( 'nebotheme-adsense' );
		}
	}

	/**
	 * Handles JavaScript and CSS detection.
	 */
	public function add_script() {
		if ( nebotheme_get_tm( 'enable_adsense' ) ) {
			wp_register_script( 'yaffo-adsense', false, [], wp_get_theme()->get( 'Version' ), true );
			wp_enqueue_script( 'yaffo-adsense' );
			wp_add_inline_script( 'yaffo-adsense', '( adsbygoogle = window.adsbygoogle || [] ).push( {} );' );
		}
	}

	/**
	 * Add fields to customize.
	 */
	public function customize_add_fields() {
		$templates = $this->get_templates();
		$locations = $this->get_locations();

		Customize_Manager::add_panel(
			'additional_content',
			[
				'title' => esc_html__( 'Additional Content', 'yaffo' ) . ' (' . esc_html__( 'Google AdSense', 'yaffo' ) . ')',
				'priority' => 10,
				'icon' => 'dashicons-money-alt',
			]
		);

		Customize_Manager::add_section(
			'add_content_adsense',
			[
				'title' => esc_html__( 'Google AdSense', 'yaffo' ),
				'panel' => 'additional_content',
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'custom',
				'settings' => 'additional_content_notice',
				'section' => 'add_content_adsense',
				'default' => sprintf(
					/* translators: link to mailchimp documentation */
					esc_html__( 'Check out the %s for configuration instructions.', 'yaffo' ),
					'<a target="_blank" href="' . esc_url( self::get_url_docs() ) . '">' . esc_html__( 'Documentation', 'yaffo' ) . '</a>'
				),
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'toggle',
				'settings' => 'enable_adsense',
				'label' => esc_html__( 'Enable Script for Google AdSense?', 'yaffo' ),
				'description' => esc_html__( 'If you are using Google AdSense, allow this.', 'yaffo' ),
				'section' => 'add_content_adsense',
				'default' => false,
			]
		);

		foreach ( $templates as $template_slug => $template ) {
			Customize_Manager::add_section(
				"add_content_{$template_slug}",
				[
					'title' => $template['label'],
					'panel' => 'additional_content',
				]
			);

			foreach ( $locations as $location_slug => $location ) {
				if ( ! in_array( $template_slug, $location['template'], true ) ) {
					continue;
				}

				foreach ( self::$positions as $position => $position_label ) {
					Customize_Manager::add_field(
						[
							'type' => 'code',
							'settings' => self::PREFIX_SETTING . "{$template_slug}_{$location_slug}_{$position}",
							'label' => "{$position_label} {$location['name']}",
							'section' => "add_content_{$template_slug}",
							'sanitize_callback' => 'nebotheme_unsanitize',
							'choices' => [
								'language' => 'html',
							],
						]
					);
				}
			}
		}
	}

	/**
	 * Define array of Additional Content Locations
	 */
	public function get_locations() {
		$templates = array_keys( $this->get_templates() );
		$locations = [
			'header' => [
				'name' => esc_html__( 'Header', 'yaffo' ),
				'template' => $templates,
			],
			'content' => [
				'name' => esc_html__( 'Content', 'yaffo' ),
				'template' => $templates,
			],
			'footer' => [
				'name' => esc_html__( 'Footer', 'yaffo' ),
				'template' => $templates,
			],
		];

		return apply_filters( 'nebotheme_add_content_locations', $locations );
	}

	/**
	 * Define array of Additional Content Templates
	 */
	public function get_templates() {
		$templates = [
			'home' => [
				'label' => esc_html__( 'Homepage', 'yaffo' ),
				'condition' => 'is_home',
			],
			'single' => [
				'label' => esc_html__( 'Open Post', 'yaffo' ),
				'condition' => 'is_single',
			],
			'page' => [
				'label' => esc_html__( 'Page', 'yaffo' ),
				'condition' => 'is_page',
			],
			'archive' => [
				'label' => esc_html__( 'Archive', 'yaffo' ),
				'condition' => 'is_archive',
			],
		];

		return apply_filters( 'nebotheme_additional_content_templates', $templates );
	}

	/**
	 * Get Current slug template.
	 */
	public function get_template_slug() {
		foreach ( $this->get_templates() as $template_slug => $template ) {
			if (
				$template &&
				isset( $template['condition'] ) &&
				is_callable( $template['condition'] ) &&
				call_user_func( $template['condition'] )
			) {
				return $template_slug;
			}
		}

		return false;
	}

	/**
	 * Echoes content.
	 */
	public function render( array $location, $location_slug, $position ) {
		$template_slug = $this->get_template_slug();

		if ( ! $template_slug || ! in_array( $template_slug, $location['template'], true ) ) {
			return;
		}

		$field_id = self::PREFIX_SETTING . "{$template_slug}_{$location_slug}_{$position}";
		$additional_content = nebotheme_get_tm( $field_id );

		if ( ! $additional_content ) {
			return;
		}

		echo '<section class="nebotheme-additional-content">' .
			do_shortcode( $additional_content ) .
		'</section><!-- .nebotheme-additional-content -->';
	}
}
