<?php
/**
 * Base class for module.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules;

use NeboTheme\Yaffo\Utils;
use NeboTheme\Yaffo\Traits\Singleton;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Module base class.
 *
 * This abstract class must be extended in order to register new modules.
 */
abstract class Base_Module {
	use Singleton;

	/**
	 * Pre instance.
	 *
	 * Run before class call.
	 *
	 * @return void
	 */
	public static function pre_instance() {}

	/**
	 * Is class available for instance.
	 *
	 * @return bool
	 */
	public static function is_active() {
		return true;
	}

	/**
	 * Retrieve module Name.
	 *
	 * @return string
	 */
	final public static function get_name() {
		$called_class = get_called_class();
		$class_parse = explode( '\\', $called_class );

		$module_name = $class_parse[ count( $class_parse ) - 2 ];
		$module_name = strtolower( $module_name );
		$module_name = str_replace( '-', '_', $module_name );

		return $module_name;
	}

	/**
	 * Retrieve folder name.
	 *
	 * @return string
	 */
	final public static function get_basename() {
		$basename = str_replace( '_', '-', static::get_name() );

		return $basename;
	}

	/**
	 * Retrieve url from module folder.
	 *
	 * @return string
	 */
	public static function get_module_url() {
		return YAFFO_URI_ROOT . '/modules/' . static::get_basename();
	}

	/**
	 * Retrieve url from module assets
	 * folder.
	 *
	 * @return string
	 */
	public static function get_assets_uri() {
		return static::get_module_url() . '/assets';
	}

	/**
	 * Retrieve url from module assets/css folder.
	 *
	 * @return string
	 */
	public static function get_css_uri() {
		return static::get_assets_uri() . '/css';
	}

	/**
	 * Retrieve path from module folder.
	 *
	 * @return string
	 */
	public static function get_module_dir() {
		return YAFFO_DIR_MODULES . '/' . static::get_basename();
	}

	/**
	 * Retrieve relative path from module folder.
	 *
	 * @return string
	 */
	public static function get_module_dir_rel() {
		return trailingslashit( YAFFO_DIR_MODULES_REL ) . static::get_basename();
	}

	/**
	 * Constructor method for the module.
	 *
	 * @return void
	 */
	protected function __construct() {
		$this->add_hooks();
		$this->add_hooks_base();
	}

	/**
	 * Add Hooks.
	 *
	 * Initialize module hooks.
	 *
	 * @return void.
	 */
	protected function add_hooks() {}

	/**
	 * Add Hooks for base classes.
	 *
	 * Initialize module hooks.
	 *
	 * @return void.
	 */
	protected function add_hooks_base() {
		if ( is_admin() ) {
			add_filter( 'nebotheme_config_modules_admin', [ $this, 'config_admin' ] );
		} else {
			add_filter( 'nebotheme_config_modules_frontend', [ $this, 'config_frontend' ] );
		}
	}

	/**
	 * Get data to frontend.
	 *
	 * @param array $config
	 *
	 * @return array
	 */
	public function config_frontend( array $config ) {
		$frontend_config = $this->get_config_frontend();

		if ( $frontend_config ) {
			$config[ $this->get_name() ] = $frontend_config;
		}

		return $config;
	}

	/**
	 * Get data to admin.
	 *
	 * @param array $config
	 *
	 * @return array
	 */
	public function config_admin( array $config ) {
		$frontend_config = $this->get_config_admin();

		if ( $frontend_config ) {
			$config[ $this->get_name() ] = $frontend_config;
		}

		return $config;
	}

	/**
	 * Get data to frontend.
	 *
	 * @return array
	 */
	public function get_config_frontend() {
		return [];
	}

	/**
	 * Get data to admin.
	 *
	 * @return array
	 */
	public function get_config_admin() {
		return [];
	}

	/**
	 * Get environment variable
	 */
	public static function get_environment() {
		$environments_modules = yaffo_get_environment()['modules'];
		$environments_module = $environments_modules[ static::get_name() ];

		return $environments_module;
	}
}
