<?php
/**
 * Module Cookie Bar.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Cookie_Bar;

use NeboTheme\Yaffo\Modules\Base_Module;
use NeboTheme\Yaffo\Customize\Customize_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Module Cookie Bar class.
 */
final class Module extends Base_Module {
	/**
	 * Get default description text for cookie bar.
	 */
	public static function get_notice_default() {
		return sprintf(
			/* translators: link about GDPR */
			esc_html__( 'Our site uses cookies. Learn more about our use of cookies: %s', 'yaffo' ),
			'<a target="_blank" href="#">' . esc_html__( 'cookie policy', 'yaffo' ) . '</a>'
		);
	}

	/**
	 * Get description text for cookie bar.
	 */
	public static function get_notice() {
		return nebotheme_get_tm( 'cookie_bar_notice', self::get_notice_default() );
	}

	/**
	 * {@inheritDoc}
	 */
	public static function is_active() {
		return nebotheme_is_core_active();
	}

	/**
	 * {@inheritDoc}
	 */
	protected function add_hooks() {
		add_action( 'nebotheme_render_body_end', [ $this, 'render' ] );
		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
	}


	/**
	 * {@inheritDoc}
	 */
	public function get_config_frontend() {
		$frontend_config = parent::get_config_frontend();

		if ( $this->is_render() ) {
			$frontend_config = [
				'cookie_name' => static::get_environment()['cookie_name_accept'],
			];
		}

		return $frontend_config;
	}

	/**
	 * Add fields to customize.
	 */
	public function customize_add_fields() {
		$active_callback = [
			[
				'setting' => 'cookie_bar_enable',
				'operator' => '!=',
				'value' => '',
			],
		];

		Customize_Manager::add_section(
			'cookie_bar',
			[
				'title' => esc_html__( 'Cookie Bar', 'yaffo' ) . ' (GDPR)',
				'priority' => 11,
				'icon' => 'dashicons-privacy',
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'toggle',
				'settings' => 'cookie_bar_enable',
				'label' => esc_html__( 'Enable Cookie Bar?', 'yaffo' ),
				'description' => esc_html__( 'Would you like to show the cookie bar?', 'yaffo' ),
				'section' => 'cookie_bar',
				'default' => false,
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'code',
				'settings' => 'cookie_bar_notice',
				'label' => esc_html__( 'Cookie Bar Content', 'yaffo' ),
				'description' => esc_html__( 'This content appears inside the cookie bar.', 'yaffo' ),
				'section' => 'cookie_bar',
				'default' => self::get_notice_default(),
				'sanitize_callback' => 'nebotheme_get_kses_content',
				'choices' => [
					'language' => 'html',
				],
				'partial_refresh' => [
					'cookie_bar_notice' => [
						'selector' => '#nebothemeCookieBar .nebotheme-cookie-bar__notice',
						'render_callback' => [ $this, 'get_notice' ],
					],
				],
				'active_callback' => $active_callback,
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'nebotheme-button',
				'label' => esc_html__( 'Preview Cookie Bar', 'yaffo' ),
				'section' => 'cookie_bar',
				'event' => 'cookie_bar_preview',
				'active_callback' => $active_callback,
			]
		);
	}

	/**
	 * Retrieve whether the cookie-bar is available for render.
	 *
	 * @return bool
	 */
	public function is_render() {
		return (
			nebotheme_get_tm( 'cookie_bar_enable' ) &&
			(
				is_customize_preview() ||
				! isset( $_COOKIE[ static::get_environment()['cookie_name_accept'] ] )
			)
		);
	}

	/**
	 * Echoes cookie bar.
	 */
	public function render() {
		if ( ! $this->is_render() ) {
			return;
		}

		$notice = nebotheme_get_tm( 'cookie_bar_notice', self::get_notice_default() );
		?>
		<aside id="nebothemeCookieBar" class="nebotheme-cookie-bar">
			<div class="nebotheme-cookie-bar__notice">
				<?php echo do_shortcode( $notice ); ?>
			</div>
			<?php
			nebotheme_render_button(
				[
					'id' => 'nebothemeCookieBarAccept',
					'text' => esc_html__( 'I accept use of cookies', 'yaffo' ),
				]
			);
			nebotheme_render_trigger_close( 'cookie-bar' );
			?>
		</aside>
		<?php
	}
}
