<?php
/**
 * Module Gutenberg.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Gutenberg;

use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Customize\Font_Printer;
use NeboTheme\Yaffo\Modules\Base_Module;
use NeboTheme\Yaffo\Utils;
use NeboTheme\Yaffo\Theme;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Module Gutenberg class.
 */
final class Module extends Base_Module {
	/**
	 * Color palette.
	 *
	 * @var array
	 */
	protected $color_palette = [];

	/**
	 * Color palette styles.
	 *
	 * @var string CSS
	 */
	protected $gutenberg_color_css = '';

	/**
	 * Constructor method for the module.
	 *
	 * @return void
	 */
	public function __construct() {
		$this->set_color_palette();

		parent::__construct();
	}

	/**
	 * Set color palette.
	 */
	public function set_color_palette() {
		$theme_vars = nebotheme_get_theme_vars();
		$color_general = nebotheme_get_color_general_default_palette();

		if ( ! is_array( $theme_vars['color']['primary'] ) ) {
			return;
		}

		foreach ( $theme_vars['color']['primary'] as $color_name => $color_label ) {
			$color = Utils::get_if_isset( $color_general, $color_name );

			if ( ! $color ) {
				continue;
			}

			$color_slug = "nebotheme-{$color_name}";

			$this->color_palette[] = [
				'name' => $color_label,
				'slug' => $color_slug,
				'color' => $color,
			];

			$this->gutenberg_color_css .= ".has-{$color_slug}-color {\n" .
				"\tcolor: {$color} !important;\n" .
			"}\n\n" .
			".has-{$color_slug}-background-color {\n" .
				"\tbackground-color: {$color} !important;\n" .
			"}\n\n";
		}
	}

	/**
	 * Add Hooks.
	 *
	 * Initialize module hooks.
	 *
	 * @return void.
	 */
	protected function add_hooks() {
		add_action( 'enqueue_block_editor_assets', [ $this, 'enqueue_editor_assets' ] );
		add_action( 'after_setup_theme', [ $this, 'after_setup_theme' ] );

		if ( ! is_admin() ) {
			add_action( 'enqueue_block_assets', [ $this, 'enqueue_frontend_assets' ] );
		}
	}

	/**
	 * Add & Remove style on editor.
	 */
	public function enqueue_editor_assets() {
		Customize_Manager::instance()->enqueue_styles();
		Font_Printer::instance()->print_css();
		$this->enqueue_common_style();

		wp_enqueue_style(
			'nebotheme-gutenberg-editor',
			Theme::get_style_url(
				'editor',
				static::get_css_uri()
			),
			[],
			wp_get_theme( 'yaffo' )->version,
			'screen'
		);
	}

	/**
	 * Add & Remove style on frontend.
	 */
	public function enqueue_frontend_assets() {
		$this->enqueue_common_style();

		wp_enqueue_style( 'yaffo-gutenberg', Theme::get_style_url( 'frontend', static::get_css_uri() ), [], wp_get_theme( 'yaffo' )->version, 'screen' );
	}

	/**
	 * Remove gutenberg theme style.
	 */
	protected function enqueue_common_style() {
		wp_deregister_style( 'wp-block-library-theme' );
		wp_register_style( 'wp-block-library-theme', '', [], '1.0.0' );
		wp_deregister_style( 'wp-block-library' );
		wp_register_style( 'wp-block-library', '', [], '1.0.0' );

		wp_register_style( 'yaffo-gutenberg-color', false, [], wp_get_theme()->get( 'Version' ) );
		wp_enqueue_style( 'yaffo-gutenberg-color' );
		wp_add_inline_style( 'yaffo-gutenberg-color', $this->gutenberg_color_css );
	}

	/**
	 * Register support for various WordPress features.
	 */
	public function after_setup_theme() {
		// Gutenberg support for full-width/wide alignment of supported blocks.
		add_theme_support( 'align-wide' );

		// Gutenberg editor styles support.
		add_theme_support( 'editor-styles' );
		add_theme_support( 'editor-color-palette', $this->color_palette );
	}
}
