<?php
/**
 * Module Import Manager.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Importer;

use NeboTheme\Yaffo\Admin\Admin_Menu;
use NeboTheme\Yaffo\Menu\Menu_Manager;
use NeboTheme\Yaffo\Modules\Base_Module;
use NeboTheme\Yaffo\Modules\Importer\Custom_Merlin;
use NeboTheme\Yaffo\Sidebars\Sidebar_Manager;
use NeboTheme\Yaffo\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Module Import Manager class.
 */
final class Module extends Base_Module {
	const PAGE_TITLE_HOME = 'Home';
	const PAGE_TITLE_BLOG = 'Blog';
	const MERLIN_PAGE = 'merlin';
	const PINGBACK_FLAG_OPTION_NAME = 'default_pingback_flag';
	const PINGBACK_FLAG_TM_NAME_OLD = self::PINGBACK_FLAG_OPTION_NAME . '_old';

	/**
	 * Custom class MerlinWP.
	 *
	 * @var Custom_Merlin
	 */
	public $custom_merlin;

	/**
	 * {@inheritDoc}
	 */
	public static function is_active() {
		return class_exists( '\Merlin' ) && is_admin();
	}

	/**
	 * {@inheritDoc}
	 */
	public static function pre_instance() {
		if ( ! is_admin() ) {
			return;
		}

		$merlinwp_dir = static::get_merlinwp_dir();

		require_once "{$merlinwp_dir}/packages/autoload.php";
		require_once "{$merlinwp_dir}/class-merlin.php";
	}

	/**
	 * Add Hooks.
	 *
	 * Initialize module hooks.
	 *
	 * @return void.
	 */
	protected function add_hooks() {
		add_filter( 'merlin_content_home_page_title', [ $this, 'content_home_page_title' ] );
		add_filter( 'merlin_content_blog_page_title', [ $this, 'content_blog_page_title' ] );
		add_filter( 'merlin_generate_child_functions_php', [ $this, 'generate_child_functions_php' ] );
		add_filter( 'merlin_generate_child_style_css', [ $this, 'generate_child_style_css' ] );
		add_filter( 'merlin_import_files', function() {
			return array_values( self::get_demos() );
		} );
		add_filter( 'merlin_unset_default_widgets_args', [ $this, 'unset_default_widgets_args' ] );
		add_filter( 'cei_export_option_keys', [ $this, 'export_option_keys' ] );

		add_action( 'import_start', [ $this, 'import_start' ] );
		add_action( 'merlin_after_all_import', [ $this, 'after_import_setup' ] );
	}

	/**
	 * The class constructor.
	 */
	public function __construct() {
		parent::__construct();

		$this->init_merlin();
	}

	/**
	 * Init merlinWP.
	 */
	public function init_merlin() {
		$this->get_merlin();
	}

	/**
	 * Get path to merlinwp dir.
	 */
	protected static function get_merlinwp_dir() {
		return static::get_module_dir() . static::get_merlinwp_rel();
	}

	/**
	 * Get relative path to merlinwp dir.
	 */
	protected static function get_merlinwp_rel() {
		return '/includes/merlin';
	}

	/**
	 * Get merlinwp object.
	 *
	 * @return Custom_Merlin
	 */
	public function get_merlin() {
		if ( ! $this->custom_merlin ) {
			$config = [
				'child_action_btn_url' => 'https://codex.wordpress.org/child_themes', // URL for the 'child-action-link'.
				'dev_mode' => ( defined( 'WP_DEBUG' ) && WP_DEBUG ), // Enable development mode for testing.
				'directory' => static::get_module_dir_rel() . static::get_merlinwp_rel(), // Location / directory where Merlin WP is placed in your theme.
				'edd_item_name' => '', // EDD_Theme_Updater_Admin item_name.
				'edd_remote_api_url' => '', // EDD_Theme_Updater_Admin remote_api_url.
				'edd_theme_slug' => '', // EDD_Theme_Updater_Admin item_slug.
				'license_help_url' => nebotheme_get_url_docs_theme(), // URL for the 'license-tooltip'.
				'license_required' => false, // Require the license activation step.
				'license_step' => false, // EDD license activation step.
				'parent_slug' => Admin_Menu::MENU_SLUG, // The wp-admin parent page slug for the admin menu item.
				'merlin_url' => static::MERLIN_PAGE,
			];

			$strings = [
				'admin-menu' => esc_html__( 'Demo Importer', 'yaffo' ),
				/* translators: 1: Title Tag 2: Theme Name 3: Closing Title Tag */
				'title%s%s%s%s' => esc_html__( '%1$s%2$s Themes &lsaquo; Demo Importer: %3$s%4$s', 'yaffo' ),
				'return-to-dashboard' => esc_html__( 'Return to the dashboard', 'yaffo' ),
				'ignore' => esc_html__( 'Disable this wizard', 'yaffo' ),

				'btn-skip' => esc_html__( 'Skip', 'yaffo' ),
				'btn-next' => esc_html__( 'Next', 'yaffo' ),
				'btn-start' => esc_html__( 'Start', 'yaffo' ),
				'btn-no' => esc_html__( 'Cancel', 'yaffo' ),
				'btn-plugins-install' => esc_html__( 'Install', 'yaffo' ),
				'btn-child-install' => esc_html__( 'Install', 'yaffo' ),
				'btn-content-install' => esc_html__( 'Install', 'yaffo' ),
				'btn-import' => esc_html__( 'Import', 'yaffo' ),
				'btn-license-activate' => esc_html__( 'Activate', 'yaffo' ),
				'btn-license-skip' => esc_html__( 'Later', 'yaffo' ),

				/* translators: Theme Name */
				'welcome-header%s' => esc_html__( 'Welcome to %s', 'yaffo' ),
				'welcome-header-success%s' => esc_html__( 'Hi. Welcome back', 'yaffo' ),
				'welcome%s' => esc_html__( 'This wizard will set up your theme, install plugins, and import content. It is optional & should take only a few minutes.', 'yaffo' ),
				'welcome-success%s' => esc_html__( 'You may have already run this theme setup wizard. If you would like to proceed anyway, click on the "Start" button below.', 'yaffo' ),

				'child-header' => esc_html__( 'Install Child Theme', 'yaffo' ),
				'child-header-success' => esc_html__( 'You\'re good to go!', 'yaffo' ),
				'child' => esc_html__( 'Let\'s build & activate a child theme so you may easily make theme changes.', 'yaffo' ),
				'child-success%s' => esc_html__( 'Your child theme has already been installed and is now activated, if it wasn\'t already.', 'yaffo' ),
				'child-action-link' => esc_html__( 'Learn about child themes', 'yaffo' ),
				'child-json-success%s' => esc_html__( 'Awesome. Your child theme has already been installed and is now activated.', 'yaffo' ),
				'child-json-already%s' => esc_html__( 'Awesome. Your child theme has been created and is now activated.', 'yaffo' ),

				'plugins-header' => esc_html__( 'Install Plugins', 'yaffo' ),
				'plugins-header-success' => esc_html__( 'You\'re up to speed!', 'yaffo' ),
				'plugins' => esc_html__( 'Let\'s install some essential WordPress plugins to get your site up to speed.', 'yaffo' ),
				'plugins-success%s' => esc_html__( 'The required WordPress plugins are all installed and up to date. Press "Next" to continue the setup wizard.', 'yaffo' ),
				'plugins-action-link' => esc_html__( 'Advanced', 'yaffo' ),

				'import-header' => esc_html__( 'Import Content', 'yaffo' ),
				'import' => esc_html__( 'Let\'s import content to your website, to help you get familiar with the theme.', 'yaffo' ),
				'import-action-link' => esc_html__( 'Advanced', 'yaffo' ),

				'ready-header' => esc_html__( 'All done. Have fun!', 'yaffo' ),

				/* translators: Theme Author */
				'ready%s' => esc_html__( 'Your theme has been all set up. Enjoy your new theme by %s.', 'yaffo' ),
				'ready-action-link' => esc_html__( 'Extras', 'yaffo' ),
				'ready-big-button' => esc_html__( 'View your website', 'yaffo' ),
				'ready-link-1' => sprintf( '<a href="%1$s">%2$s</a>', admin_url( 'customize.php' ), esc_html__( 'Start Customizing', 'yaffo' ) ),
				'ready-link-2' => sprintf( '<a href="%1$s" target="_blank">%2$s</a>', esc_url( nebotheme_get_url_docs_theme() ), esc_html__( 'Get Theme Documentation', 'yaffo' ) ),
				'ready-link-3' => sprintf( '<a href="%1$s" target="_blank">%2$s</a>', esc_url( nebotheme_get_url_support() ), esc_html__( 'Get Theme Support', 'yaffo' ) ),
			];

			$this->custom_merlin = new Custom_Merlin( $config, $strings );
		}

		return $this->custom_merlin;
	}

	/**
	 * Filter the home page title from your demo content.
	 * If your demo's home page title is "Home", you don't need this.
	 */
	public function content_home_page_title() {
		return self::PAGE_TITLE_HOME;
	}

	/**
	 * Filter the blog page title from your demo content.
	 * If your demo's blog page title is "Blog", you don't need this.
	 */
	public function content_blog_page_title() {
		return self::PAGE_TITLE_BLOG;
	}

	/**
	 * Custom template for the child theme style.css file.
	 */
	public function generate_child_style_css() {
		$wp_theme = wp_get_theme();

		$output = "
/*
Theme Name: {$wp_theme->get( 'Name' )} Child
Theme URI: {$wp_theme->get( 'ThemeURI' )}
Author: {$wp_theme->get( 'Author' )}
Author URI: {$wp_theme->get( 'AuthorURI' )}
Description: A child theme of {$wp_theme->get( 'Name' )} Theme
Template: {$wp_theme->template}
Version: 1.0.0
License:
License URI:
Text Domain: {$wp_theme->template}-child
*/\n";

		return $output;
	}

	/**
	 * Custom content for the generated child theme's functions.php file.
	 */
	public function generate_child_functions_php() {
		$output = "<?php
/**
 * Theme functions and definitions.
 * This child theme was generated by Merlin WP.
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * If your child theme has more than one .css file (eg. ie.css, style.css, main.css) then
 * you will have to make sure to maintain all of the parent theme dependencies.
 *
 * Make sure you're using the correct handle for loading the parent theme's styles.
 * Failure to use the proper tag will result in a CSS file needlessly being loaded twice.
 * This will usually not affect the site appearance, but it's inefficient and extends your page's loading time.
 *
 * @link https://codex.wordpress.org/Child_Themes
 */

function yaffo_child_enqueue_styles() {
	wp_enqueue_style( 'yaffo-child', get_stylesheet_directory_uri() . '/style.css', [ 'yaffo' ], wp_get_theme()->get( 'Version' ) );
}

add_action( 'wp_enqueue_scripts', 'yaffo_child_enqueue_styles' );\n";

		return $output;
	}



	/**
	 * Define the demo import files (local files).
	 *
	 * You have to use the same filter as in above example,
	 * but with a slightly different array keys: local_*.
	 * The values have to be absolute paths (not URLs) to your import files.
	 * To use local import files, that reside in your theme folder,
	 * please use the below code.
	 * Note: make sure your import files are readable!
	 */
	public static function get_demos() {
		static $demos = [];

		if ( ! empty( $demos ) ) {
			return $demos;
		}

		$wp_filesystem = Utils::get_wp_filesystem();
		$module_url    = static::get_module_url();
		$module_dir    = static::get_module_dir();
		$demos_url     = "{$module_url}/demos";
		$demos_dir     = "{$module_dir}/demos";
		$dirlist       = $wp_filesystem->dirlist( $demos_dir );

		if ( ! $dirlist ) {
			return $demos;
		}

		$dirlist = array_keys( $dirlist );

		uasort( $dirlist, function( $demo_slug ) {
			if ( 'primary' === $demo_slug ) {
				return -1;
			}

			return 1;
		} );

		if ( $dirlist && is_array( $dirlist ) ) {
			$filename_preview  = static::get_environment()['filename_preview'];
			$filename_content  = static::get_environment()['filename_content'];
			$filename_widgets  = static::get_environment()['filename_widgets'];
			$filename_settings = static::get_environment()['filename_settings'];

			foreach ( $dirlist as $demo_slug ) {
				$demo_url   = "{$demos_url}/{$demo_slug}";
				$demo_dir   = "{$demos_dir}/{$demo_slug}";
				$files_name = array_keys( $wp_filesystem->dirlist( $demo_dir ) );
				$demo       = [];

				// sets file path.
				foreach ( $files_name as $file_name ) {
					$property_name = false;

					switch ( $file_name ) {
						case $filename_content:
							$property_name = 'local_import_file';

							break;

						case $filename_widgets:
							$property_name = 'local_import_widget_file';

							break;

						case $filename_settings:
							$property_name = 'local_import_customizer_file';

							break;

						case $filename_preview:
							$property_name = 'import_preview_image_url';

							break;
					}

					if ( $filename_preview === $file_name ) {
						$file_path = "{$demo_url}/{$file_name}";
					} else {
						$file_path = "{$demo_dir}/{$file_name}";
					}

					if ( $property_name ) {
						$demo[ $property_name ] = $file_path;
					}
				}

				if ( empty( $demo['import_preview_image_url'] ) ) {
					$demo['import_preview_image_url'] = wp_get_theme()->get_screenshot();
				}

				if (
					empty( $demo['local_import_file'] ) ||
					empty( $demo['local_import_widget_file'] ) ||
					empty( $demo['local_import_customizer_file'] ) ||
					empty( $demo['import_preview_image_url'] )
				) {
					continue;
				}

				// Demo info by xml file.
				$content_xml = new \SimpleXMLElement( $wp_filesystem->get_contents( $demo['local_import_file'] ) );

				$demo['import_notice']    = $content_xml->channel->description->__toString();
				$demo['import_file_name'] = $content_xml->channel->title->__toString();
				$demo['preview_url']      = $content_xml->channel->link->__toString();

				$demo['dir'] = $demo_dir;

				$demos[ $demo_slug ] = $demo;
			}
		}

		return $demos;
	}

	/**
	 * Remove widgets from sidebars.
	 * By default none are removed, but with the filter you can remove them.
	 */
	public function unset_default_widgets_args( $widget_areas ) {
		$sidebars_list_default = Sidebar_Manager::get_sidebars_list_default();

		$widget_areas = array_fill_keys( array_keys( $sidebars_list_default ), [] );

		return $widget_areas;
	}

	/**
	 * Execute custom code after the whole import has started.
	 */
	public function import_start() {
		if ( ! self::is_merlin_page() ) {
			return;
		}

		$pingback_flag = get_option( self::PINGBACK_FLAG_OPTION_NAME );

		if ( ! $pingback_flag ) {
			return;
		}

		set_theme_mod( self::PINGBACK_FLAG_TM_NAME_OLD, $pingback_flag );
		update_option( self::PINGBACK_FLAG_OPTION_NAME, 0 );
	}

	/**
	 * Execute custom code after the whole import has finished.
	 */
	public function after_import_setup() {
		$pingback_flag_old = get_theme_mod( self::PINGBACK_FLAG_OPTION_NAME );
		$nav_menu_locations = nebotheme_get_tm( 'nav_menu_locations', [] );

		if ( ! is_array( $nav_menu_locations ) ) {
			$nav_menu_locations = [];
		}

		foreach ( Menu_Manager::instance()->get_menus() as  $menu_obj ) {
			// Assign menus to their locations.
			$main_menu = get_term_by( 'name', $menu_obj->get_label(), 'nav_menu' );

			if ( ! $main_menu || is_wp_error( $main_menu ) ) {
				continue;
			}

			$nav_menu_locations[ $menu_obj->get_id() ] = $main_menu->term_id;
		}

		if ( $nav_menu_locations ) {
			set_theme_mod( 'nav_menu_locations', $nav_menu_locations );
		}

		// Assign front page and posts page (blog page).
		$front_page = get_page_by_title( self::PAGE_TITLE_HOME );
		$blog_page = get_page_by_title( self::PAGE_TITLE_BLOG );

		if ( $front_page ) {
			update_option( 'show_on_front', 'page' );
			update_option( 'page_on_front', $front_page->ID );
		}

		if ( $blog_page ) {
			update_option( 'page_for_posts', $blog_page->ID );
		}

		if ( $pingback_flag_old ) {
			update_option( self::PINGBACK_FLAG_OPTION_NAME, $pingback_flag_old );
			remove_theme_mod( self::PINGBACK_FLAG_TM_NAME_OLD );
		}
	}

	/**
	 * Additional option to export.
	 */
	public function export_option_keys( array $option_keys ) {
		if ( 10 !== get_option( 'posts_per_page' ) ) {
			$option_keys[] = 'posts_per_page';
		}

		return $option_keys;
	}

	/**
	 * Check if current page is for merlinwp.
	 */
	public static function is_merlin_page() {
		if ( Utils::is_ajax() ) {
			$parsed = wp_parse_url( wp_get_referer(), PHP_URL_QUERY );

			if ( $parsed ) {
				parse_str( $parsed, $url_query_array );
			} else {
				$url_query_array = [];
			}

			if ( $url_query_array && isset( $url_query_array['page'] ) && static::MERLIN_PAGE === $url_query_array['page'] ) {
				return true;
			}
		}

		return ! empty( $_GET['page'] ) && static::MERLIN_PAGE === $_GET['page']; //phpcs:ignore WordPress.Security.NonceVerification.Recommended
	}

	/**
	 * Check complete status.
	 */
	public static function is_completed() {
		$theme = wp_get_theme();
		$slug  = strtolower( preg_replace( '#[^a-zA-Z]#', '', $theme->template ) );

		return get_option( "merlin_{$slug}_completed" );
	}
}
