<?php
/**
 * Module Mailchimp.
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Mailchimp;

use NeboTheme\Yaffo\Admin\Meta_Boxes;
use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\Yaffo\Homepage;
use NeboTheme\Yaffo\Sidebars\Sidebar_Manager;
use NeboTheme\Yaffo\Modules\Base_Module;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Module Mailchimp class.
 */
final class Module extends Base_Module {
	/**
	 * Get url to documentation.
	 *
	 * @return string
	 */
	public static function get_url_docs() {
		return nebotheme_get_url_docs_theme() . '/mailchimp-newsletter';
	}

	/**
	 * Is class available for instance.
	 *
	 * @return bool
	 */
	public static function is_active() {
		return function_exists( 'mc4wp' );
	}

	/**
	 * Add Hooks.
	 *
	 * Initialize module hooks.
	 *
	 * @return void
	 */
	protected function add_hooks() {
		add_action(
			'wp',
			function() {
				if ( is_singular( 'post' ) ) {
					if ( ! Meta_Boxes::get_pm_safe( 'newsletter_post' ) ) {
						return;
					}

					add_action(
						'nebotheme_render_content_after',
						function() {
							$this->render_container( 'nebotheme-mailchimp-form--submit-top nebotheme-mailchimp-form--single' );
						}
					);
				} elseif ( Homepage::is_first_page() ) {
					$position = Homepage::get_settings( 'homepage_newsletter_position' );

					if ( 'before' === $position ) {
						add_action( 'nebotheme_render_main_start', [ $this, 'render_large' ] );
					} elseif ( 'after' === $position ) {
						add_action( 'nebotheme_render_content_after', [ $this, 'render_large' ] );
					}
				}

				if ( $this->is_render_popup() ) {
					add_action( 'nebotheme_render_body_end', [ $this, 'render_popup' ] );
				}
			}
		);

		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
		add_action( 'merlin_after_all_import', [ $this, 'after_import_setup' ] );
		add_action( 'customize_register', [ $this, 'change_newsletter_post_control' ], 999 );

		add_filter( 'nebotheme_meta_boxes_fields', [ $this, 'get_meta_boxes_fields' ] );
		add_filter( 'nebotheme_meta_boxes_tabs', [ $this, 'get_meta_boxes_tabs' ] );

		// If on preview.
		if ( ! empty( $_GET['mc4wp_preview_form'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			add_filter( 'kirki_output_inline_styles', '__return_false' );
		}
	}

	/**
	 * Move newsletter_post field.
	 */
	public function change_newsletter_post_control( $wp_customize ) {
		$newsletter_post_control = $wp_customize->get_control( 'newsletter_post' );
		$sidebar_layout_control = $wp_customize->get_control( Sidebar_Manager::get_layout_name( Sidebar_Manager::SIDEBAR_POST ) );

		if ( ! $newsletter_post_control || ! $sidebar_layout_control ) {
			return;
		}

		$newsletter_post_control->priority = $sidebar_layout_control->priority;
		$newsletter_post_control->instance_number = $sidebar_layout_control->instance_number + 1;
	}

	/**
	 * Retrieve Mailchimp default form id
	 *
	 * @return int Default mc4 form ID
	 */
	protected function get_form_id() {
		return (int) get_option( 'mc4wp_default_form_id', 0 );
	}

	/**
	 * Returns whether the form exists.
	 *
	 * @return bool
	 */
	protected function is_has_form() {
		try {
			mc4wp_get_form( $this->get_form_id() );
		} catch ( \Exception $e ) {
			return false;
		}

		return true;
	}

	/**
	 * Get tabs for meta boxes.
	 *
	 * @return array
	 */
	public function get_meta_boxes_tabs( array $tabs ) {
		$tabs['newsletter'] = [
			'label' => esc_html__( 'Newsletter', 'yaffo' ),
			'post_type' => [ 'page', 'post' ],
		];

		return $tabs;
	}

	/**
	 * Get fields for meta boxes.
	 *
	 * @return array
	 */
	public function get_meta_boxes_fields( array $field ) {
		$post_type = get_post_type();

		switch ( $post_type ) {
			case 'post':
				$field['newsletter_post'] = [
					'label' => esc_html__( 'Display Post Newsletter', 'yaffo' ),
					'type' => Meta_Boxes::CONTROL_SWITCHER,
					'tab' => 'newsletter',
					'theme_mod_name' => 'newsletter_post',
					'label_enable' => esc_html__( 'Yes', 'yaffo' ),
					'label_disabled' => esc_html__( 'No', 'yaffo' ),
				];
				break;
			case 'page':
				$field['homepage_newsletter_position'] = [
					'label' => Customize_Manager::get_field( 'homepage_newsletter_position', 'label' ),
					'type' => Meta_Boxes::CONTROL_SELECT,
					'tab' => 'newsletter',
					'default' => Customize_Manager::get_field( 'homepage_newsletter_position', 'default' ),
					'options' => Customize_Manager::get_field( 'homepage_newsletter_position', 'choices', [] ),
				];
				break;
		}

		return $field;
	}

	/**
	 * {@inheritDoc}
	 */
	public function get_config_frontend() {
		$frontend_config = [];

		if ( $this->is_render_popup() ) {
			$frontend_config = [
				'expires' => nebotheme_get_tm( 'newsletter_popup_expires' ),
				'cookie_name' => static::get_environment()['cookie_name_popup'],
			];
		}

		return $frontend_config;
	}

	/**
	 * Retrieve whether the popup is available for render.
	 *
	 * @return bool
	 */
	protected function is_render_popup() {
		return (
			nebotheme_get_tm( 'newsletter_show' ) &&
			(
				is_customize_preview() ||
				! isset( $_COOKIE[ static::get_environment()['cookie_name_popup'] ] )
			)
		);
	}

	/**
	 * Echoes html form.
	 *
	 * @param string $class_extra
	 *
	 * @return void
	 */
	public function render( $class_extra = '' ) {
		if ( ! $this->is_has_form() ) {
			return;
		}

		if ( $class_extra ) {
			$class_extra = " {$class_extra}";
		}

		echo '<div class="nebotheme-mailchimp-form' . esc_attr( $class_extra ) . '">';

		mc4wp_show_form( $this->get_form_id() );

		echo '</div>';
	}

	/**
	 * Echoes html form.
	 *
	 * Form Large.
	 *
	 * @return void
	 */
	public function render_large() {
		$this->render_container( 'nebotheme-mailchimp-form--submit-top nebotheme-mailchimp-form--half' );
	}

	/**
	 * Echoes html form.
	 *
	 * Form container.
	 *
	 * @param string $classes
	 *
	 * @return void
	 */
	public function render_container( $classes = '' ) {
		if ( ! $this->is_has_form() ) {
			return;
		}

		echo '<div class="nebotheme-mailchimp-container">' .
		'<div class="grid-container">' .
		'<div class="nebotheme-mailchimp-container__inner">';

		$this->render( $classes );

		echo '</div>' .
		'</div>' .
		'</div><!-- .nebotheme-mailchimp-container -->';
	}

	/**
	 * Registration customizer fields.
	 *
	 * @return void
	 */
	public function customize_add_fields() {
		$active_callback = [
			[
				'setting' => 'newsletter_show',
				'operator' => '!=',
				'value' => '',
			],
		];

		Customize_Manager::add_section(
			'newsletter_popup',
			[
				'title' => esc_html__( 'Newsletter Pop-up', 'yaffo' ),
				'priority' => 8,
				'icon' => 'dashicons-email-alt2',
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'custom',
				'settings' => 'newsletter_notice',
				'section' => 'newsletter_popup',
				'default' => sprintf(
					/* translators: link to mailchimp documentation */
					esc_html__( 'Check out the %s for configuration instructions.', 'yaffo' ),
					'<a target="_blank" href="' . esc_url( self::get_url_docs() ) . '">' . esc_html__( 'Documentation', 'yaffo' ) . '</a>'
				),
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'toggle',
				'settings' => 'newsletter_show',
				'label' => esc_html__( 'Show Newsletter Pop-up', 'yaffo' ),
				'section' => 'newsletter_popup',
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'select',
				'settings' => 'newsletter_popup_expires',
				'label' => esc_html__( 'Newsletter refresh interval', 'yaffo' ),
				'description' => esc_html__( 'When the user closes the popup, the newsletter will not be visible on the next page. After the below period, its going to be visible again unless he closes it again', 'yaffo' ),
				'section' => 'newsletter_popup',
				'default' => '7',
				'choices' => [
					'0' => esc_html__( 'Never - the popup will be shown every page', 'yaffo' ),
					'1' => esc_html__( '1 Day', 'yaffo' ),
					'2' => esc_html__( '2 Days', 'yaffo' ),
					'3' => esc_html__( '3 Days', 'yaffo' ),
					'7' => esc_html__( '1 Week', 'yaffo' ),
					'14' => esc_html__( '2 Weeks', 'yaffo' ),
					'21' => esc_html__( '3 Weeks', 'yaffo' ),
					'30' => esc_html__( '1 Month', 'yaffo' ),
				],
				'active_callback' => $active_callback,
			]
		);

		Customize_Manager::add_field(
			[
				'settings' => 'newsletter_popup_image',
				'label' => esc_html__( 'Side Image', 'yaffo' ),
				'type' => 'cropped_image',
				'section' => 'newsletter_popup',
				'flex_width' => true,
				'flex_height' => false,
				'width' => 880,
				'height' => 880,
				'partial_refresh' => [
					'newsletter_popup_image' => [
						'selector' => '#nebotheme-popup-newsletter .nebotheme-popup__inner',
						'render_callback' => [ $this, 'render_popup_inner' ],
					],
				],
				'active_callback' => $active_callback,
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'nebotheme-button',
				'label' => esc_html__( 'Preview popup', 'yaffo' ),
				'section' => 'newsletter_popup',
				'event' => 'newsletter_preview',
				'active_callback' => $active_callback,
			]
		);

		/* Homepage section */
		Customize_Manager::add_field(
			[
				'type' => 'select',
				'settings' => 'homepage_newsletter_position',
				'label' => esc_html__( 'Display Newsletter', 'yaffo' ),
				'section' => 'homepage',
				'default' => YAFFO_VALUE_NEGATIVE,
				'priority' => 999,
				'choices' => [
					YAFFO_VALUE_NEGATIVE => esc_html__( 'None', 'yaffo' ),
					'before' => esc_html__( 'Before Content', 'yaffo' ),
					'after' => esc_html__( 'After Content', 'yaffo' ),
				],
			]
		);

		/* Layouts post */
		Customize_Manager::add_field(
			[
				'type' => 'toggle',
				'settings' => 'newsletter_post',
				'label' => esc_html__( 'Display Post Newsletter', 'yaffo' ),
				'section' => 'layout_sidebar_sidebar_post',
				'default' => true,
			]
		);
	}

	/**
	 * Execute custom code after the whole import has finished.
	 */
	public function after_import_setup() {
		foreach ( mc4wp_get_forms() as $form ) {
			if ( ! preg_match( '/nebotheme/i', $form->name ) ) {
				continue;
			}

			update_option( 'mc4wp_default_form_id', $form->ID );

			break;
		}
	}

	/**
	 * Echoes html popup.
	 *
	 * @return void
	 */
	public function render_popup() {
		if ( ! $this->is_has_form() && ! is_customize_preview() ) {

			return;
		}

		nebotheme_render_popup(
			'newsletter',
			[ $this, 'render_popup_inner' ],
			[
				'trigger' => true,
			]
		);
	}

	/**
	 * Echoes html popup inner.
	 *
	 * @return void
	 */
	public function render_popup_inner() {
		if ( ! $this->is_has_form() && is_customize_preview() ) {
			echo '<div class="p-5">';

			nebotheme_render_notification(
				[
					'type' => 'warning',
					'title' => 'admin',
					'body' => function() {
						esc_html_e( 'Mailchimp plugin not configured. Read documentation.', 'yaffo' );
					},
				]
			);

			echo '</div>';

			return;
		}

		$popup_image_id = nebotheme_get_tm( 'newsletter_popup_image' );
		$popup_image_size = 'full';

		if ( $popup_image_id && wp_get_attachment_image_src( $popup_image_id, $popup_image_size ) ) {
			echo '<div class="nebotheme-newsletter-popup-image">' .
				wp_get_attachment_image( $popup_image_id, $popup_image_size, false, [
					'data-zoomable' => 'false',
				] ) .
			'</div>';
		}

		$this->render();
	}
}
