<?php
/**
 * Class widget twitter
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Twitter\Widgets;

use NeboTheme\Yaffo\Modules\Twitter\Module as TwitterModule;
use NeboTheme\Yaffo\Utils;
use NeboTheme\YaffoCore\Lazy_Load;
use NeboTheme\YaffoCore\Widget_Base;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * {@inheritDoc}
 */
class Twitter_Old extends Widget_Base {

	/**
	 * {@inheritDoc}
	 */
	public static function get_name() {
		return 'twitter_timeline';
	}

	/**
	 * {@inheritDoc}
	 */
	public function get_label() {
		return esc_html__( 'NeboTheme - Twitter Timeline', 'yaffo' );
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		if ( ! isset( $new_instance['chrome'] ) ) {
			$new_instance['chrome'] = [];
		}

		$instance = parent::update( $new_instance, $old_instance );

		$tweet_limit = (int) $instance['tweet_limit'];

		if ( $tweet_limit ) {
			$instance['tweet_limit'] = min( max( $tweet_limit, 1 ), 20 );
		} else {
			$instance['tweet_limit'] = 3;
		}

		// If they entered something that might be a full URL, try to parse it out.
		if ( is_string( $instance['username'] ) ) {
			preg_match(
				'/(https)?:\/\/twitter\.com\/(.*)$/',
				$instance['username'],
				$matches
			);

			if ( isset( $matches[2] ) ) {
				$instance['username'] = $matches[2];
			}
		}

		// If they entered wrong HEX color.
		if ( isset( $instance['border_color'] ) && '#' !== substr( $instance['border_color'], 0, 1 ) ) {
			$instance['border_color'] = '';
		}

		return $instance;
	}

	/**
	 * Back end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		$instance = Utils::array_merge_safe( (array) $instance, [
			'title' => null,
			'username' => '',
			'border_color' => '',
			'theme' => 'light',
			'tweet_limit' => 3,
			'chrome' => [],
		] );

		if ( null === $instance['title'] ) {
			$instance['title'] = esc_html__( 'Follow me on Twitter', 'yaffo' );
		}

		?>
		<!-- Title -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>">
				<?php esc_html_e( 'Title', 'yaffo' ); ?>:
			</label>
			<input
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"
				type="text"
				value="<?php echo esc_attr( $instance['title'] ); ?>"
			/>
		</p>
		<!-- Username -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'username' ) ); ?>">
				@<?php esc_html_e( 'username', 'yaffo' ); ?>:
			</label>
			<input
				type="text"
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'username' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'username' ) ); ?>"
				placeholder="<?php echo esc_attr( TwitterModule::get_username_default() ); ?>"
				value="<?php echo esc_attr( $instance['username'] ); ?>"
			/>
		</p>
		<!-- Limit -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'tweet_limit' ) ); ?>">
				<?php esc_html_e( 'Tweets Shown (1 to 20)', 'yaffo' ); ?>:
			</label>
			<input
				type="number"
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'tweet_limit' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'tweet_limit' ) ); ?>"
				min="1"
				max="20"
				value="<?php echo esc_attr( $instance['tweet_limit'] ); ?>"
				placeholder="<?php echo esc_attr( $instance['tweet_limit'] ); ?>"
			/>
		</p>
		<!-- Layout -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'chrome-noheader' ) ); ?>">
				<?php esc_html_e( 'Layout Options', 'yaffo' ); ?>:
			</label>
			<?php
			$chrome = $instance['chrome'];

			if ( ! is_array( $chrome ) ) {
				$chrome = [];
			}

			$chrome_options = [
				'noheader' => esc_html__( 'No Header', 'yaffo' ),
				'nofooter' => esc_html__( 'No Footer', 'yaffo' ),
				'noscrollbar' => esc_html__( 'No Scrollbar', 'yaffo' ),
				'transparent' => esc_html__( 'Transparent Background', 'yaffo' ),
			];

			foreach ( $chrome_options as $chrome_option => $chrome_label ) {
				?>
				<br />
				<input
					type="checkbox"<?php checked( in_array( $chrome_option, $chrome, true ) ); ?>
					id="<?php echo esc_attr( $this->get_field_id( "chrome-{$chrome_option}" ) ); ?>"
					name="<?php echo esc_attr( $this->get_field_name( 'chrome' ) ); ?>[]"
					value="<?php echo esc_attr( $chrome_option ); ?>"
				/>
				<label for="<?php echo esc_attr( $this->get_field_id( "chrome-{$chrome_option}" ) ); ?>">
					<?php echo esc_html( $chrome_label ); ?>
				</label>
				<?php
			}
			?>
		</p>
		<!-- Border Color -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'border_color' ) ); ?>">
				<?php esc_html_e( 'Border Color (hex)', 'yaffo' ); ?>:
			</label>
			<input
				type="text"
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'border_color' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'border_color' ) ); ?>"
				placeholder="<?php echo esc_attr( nebotheme_get_color_general( 'border' ) ); ?>"
				value="<?php echo esc_attr( $instance['border_color'] ); ?>"
			/>
		</p>
		<!-- Theme -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'theme' ) ); ?>">
				<?php esc_html_e( 'Theme', 'yaffo' ); ?>:
			</label>
			<?php
			nebotheme_render_select(
				[
					'id' => $this->get_field_id( 'theme' ),
					'name' => $this->get_field_name( 'theme' ),
					'class' => 'widefat',
				],
				[
					'light' => esc_html__( 'Light', 'yaffo' ),
					'dark' => esc_html__( 'Dark', 'yaffo' ),
				],
				$instance['theme']
			);
			?>
		</p>
		<?php
	}

	/**
	 * Front-end display of widget.
	 *
	 * @param array $args Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function render( $args, $instance ) {
		$instance_default = [
			'border_color' => nebotheme_get_color_general( 'border' ),
			'chrome' => [],
			'theme' => 'light',
			'tweet_limit' => 3,
			'username' => TwitterModule::get_username_default(),
		];
		$instance = Utils::array_merge_safe( (array) $instance, $instance_default );
		$attrs = [
			'data-twitter' => 'true',
			'class' => 'twitter-timeline',
			'data-border-color' => $instance['border_color'],
			'data-theme' => $instance['theme'],
			'data-lang' => substr( strtoupper( get_locale() ), 0, 2 ),
			'data-tweet-limit' => $instance['tweet_limit'],
			'data-username' => $instance['username'],
			'href' => "https://twitter.com/{$instance['username']}",
		];

		if ( nebotheme_is_custom_lazyload() ) {
			$attrs['class'] .= ' ' . Lazy_Load::CSS_CLASSES;
		}

		if ( ! empty( $instance['chrome'] ) && is_array( $instance['chrome'] ) ) {
			$attrs['data-chrome'] = esc_attr( join( ' ', $instance['chrome'] ) );
		}

		echo '<a ' . nebotheme_get_render_attrs( $attrs ) . '>' .
			esc_html__( 'My Tweets', 'yaffo' ) .
		'</a>';
	}
}
