<?php
/**
 * Class widget twitter
 *
 * @package Yaffo
 */

namespace NeboTheme\Yaffo\Modules\Twitter\Widgets;

use NeboTheme\Yaffo\Modules\Twitter\Module as TwitterModule;
use NeboTheme\Yaffo\Utils;
use NeboTheme\YaffoCore\Widget_Base;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * {@inheritDoc}
 */
class Twitter extends Widget_Base {

	/**
	 * {@inheritDoc}
	 */
	public static function get_name() {
		return 'twitter_timeline';
	}

	/**
	 * {@inheritDoc}
	 */
	public function get_label() {
		return '⛅ ' . esc_html__( 'NeboTheme - Twitter Timeline', 'yaffo' );
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		if ( ! isset( $new_instance['show_follow_button'] ) ) {
			$new_instance['show_follow_button'] = '';
		}

		if ( ! isset( $new_instance['show_user_counters'] ) ) {
			$new_instance['show_user_counters'] = '';
		}

		if ( ! isset( $new_instance['show_tweet_counters'] ) ) {
			$new_instance['show_tweet_counters'] = '';
		}

		$instance = parent::update( $new_instance, $old_instance );

		$tweet_limit = (int) $instance['tweet_limit'];

		if ( $tweet_limit ) {
			$instance['tweet_limit'] = min( max( $tweet_limit, 1 ), 20 );
		} else {
			$instance['tweet_limit'] = 3;
		}

		if ( is_string( $instance['username'] ) ) {
			preg_match(
				'/(https)?:\/\/twitter\.com\/(.*)$/',
				$instance['username'],
				$matches
			);

			if ( isset( $matches[2] ) ) {
				$instance['username'] = $matches[2];
			}
		}

		return $instance;
	}

	/**
	 * Back end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		$instance = array_replace_recursive( [
			'title' => null,
			'username' => '',
			'tweet_limit' => 3,
			'show_user_counters' => 'on',
			'show_tweet_counters' => 'on',
			'show_follow_button' => 'on',
			'follow_button_text' => '',
		], $instance );

		if ( null === $instance['title'] ) {
			$instance['title'] = esc_html__( 'Twitter', 'yaffo' );
		}

		?>
		<!-- Title -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>">
				<?php esc_html_e( 'Title', 'yaffo' ); ?>:
			</label>
			<input
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>"
				type="text"
				value="<?php echo esc_attr( $instance['title'] ); ?>"
			/>
		</p>
		<!-- Username -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'username' ) ); ?>">
				@<?php esc_html_e( 'username', 'yaffo' ); ?>:
			</label>
			<input
				type="text"
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'username' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'username' ) ); ?>"
				placeholder="<?php echo esc_attr( TwitterModule::get_username_default() ); ?>"
				value="<?php echo esc_attr( $instance['username'] ); ?>"
			/>
		</p>
		<!-- Limit -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'tweet_limit' ) ); ?>">
				<?php esc_html_e( 'Tweets Shown (1 to 20)', 'yaffo' ); ?>:
			</label>
			<input
				type="number"
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'tweet_limit' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'tweet_limit' ) ); ?>"
				min="1"
				max="20"
				value="<?php echo esc_attr( $instance['tweet_limit'] ); ?>"
				placeholder="<?php echo esc_attr( $instance['tweet_limit'] ); ?>"
			/>
		</p>
		<!-- Show User Counters -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_user_counters' ) ); ?>">
				<?php esc_html_e( 'Display User Following/Followers?', 'yaffo' ); ?>
			</label>
			<input class="checkbox" type="checkbox"<?php checked( (bool) $instance['show_user_counters'] ); ?> id="<?php echo esc_attr( $this->get_field_id( 'show_user_counters' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'show_user_counters' ) ); ?>" />
		</p>
		<!-- Show Tweet Counters -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_tweet_counters' ) ); ?>">
				<?php esc_html_e( 'Display Tweet Likes/Retweets?', 'yaffo' ); ?>
			</label>
			<input class="checkbox" type="checkbox"<?php checked( (bool) $instance['show_tweet_counters'] ); ?> id="<?php echo esc_attr( $this->get_field_id( 'show_tweet_counters' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'show_tweet_counters' ) ); ?>" />
		</p>
		<!-- Show Follow Button -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'show_follow_button' ) ); ?>">
				<?php esc_html_e( 'Display Follow Button?', 'yaffo' ); ?>
			</label>
			<input class="checkbox" type="checkbox"<?php checked( (bool) $instance['show_follow_button'] ); ?> id="<?php echo esc_attr( $this->get_field_id( 'show_follow_button' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'show_follow_button' ) ); ?>" />
		</p>
		<!-- Text Follow Button -->
		<p>
			<label for="<?php echo esc_attr( $this->get_field_id( 'follow_button_text' ) ); ?>">
				<?php esc_html_e( 'Text of Follow button', 'yaffo' ) . ': '; ?>
			</label>
			<input
				type="text"
				class="widefat"
				id="<?php echo esc_attr( $this->get_field_id( 'follow_button_text' ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( 'follow_button_text' ) ); ?>"
				placeholder="<?php esc_html_e( 'Follow Twitter', 'yaffo' ); ?>"
				value="<?php echo esc_attr( $instance['follow_button_text'] ); ?>"
			/>
		</p>
		<?php
	}

	/**
	 * Front-end display of widget.
	 *
	 * @param array $args Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function render( $args, $instance ) {
		$instance_default = [
			'tweet_limit' => 3,
			'username' => TwitterModule::get_username_default(),
			'show_user_counters' => 'on',
			'show_follow_button' => 'on',
			'show_tweet_counters' => 'on',
			'follow_button_text' => '',
		];
		$instance = array_replace_recursive( $instance_default, $instance );

		if ( empty( $instance['follow_button_text'] ) ) {
			$instance['follow_button_text'] = esc_html( 'Follow Twitter', 'yaffo' );
		}

		$twitter = TwitterModule::get_tweets( $instance['tweet_limit'], $instance['username'] );

		if ( isset( $twitter['errors'][0]['message'] ) ) {
			echo esc_html( $twitter['errors'][0]['message'] );

			return;
		}

		if ( empty( $twitter['tweets'] ) ) {
			esc_html_e( 'No Tweets!', 'yaffo' );

			return;
		}

		$attrs_user_link = [
			'rel' => 'nofollow',
			'target' => '_blank',
			'href' => esc_url( $twitter['user']['url'] ),
		];

		?>
		<div class="nebotheme-tw-timeline">
			<div class="nebotheme-tw-timeline__header">
				<div class="nebotheme-tw-timeline__person">
					<?php if ( ! empty( $twitter['user']['avatar'] ) ) : ?>
						<a <?php nebotheme_render_attrs( $attrs_user_link ); ?>>
							<?php nebotheme_render_attachment( [ 'src' => $twitter['user']['avatar'] ] ); ?>
						</a>
					<?php endif; ?>
					<div class="nebotheme-tw-timeline__person-info">
						<div class="nebotheme-tw-timeline__person__name">
							<h4 class="nebotheme-tw-timeline__person__name__title">
								<a <?php nebotheme_render_attrs( $attrs_user_link ); ?>>
									<?php echo esc_html( $twitter['user']['name'] ); ?>
								</a>
							</h4>
							<?php if ( $twitter['user']['verified'] ) : ?>
								<svg class="nebotheme-tw-timeline__person__verified" viewBox="0 0 24 24" aria-hidden="true" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg">
									<g style="fill:#1d9bf0;fill-opacity:1">
										<path d="M22.25 12c0-1.43-.88-2.67-2.19-3.34.46-1.39.2-2.9-.81-3.91s-2.52-1.27-3.91-.81c-.66-1.31-1.91-2.19-3.34-2.19s-2.67.88-3.33 2.19c-1.4-.46-2.91-.2-3.92.81s-1.26 2.52-.8 3.91c-1.31.67-2.2 1.91-2.2 3.34s.89 2.67 2.2 3.34c-.46 1.39-.21 2.9.8 3.91s2.52 1.26 3.91.81c.67 1.31 1.91 2.19 3.34 2.19s2.68-.88 3.34-2.19c1.39.45 2.9.2 3.91-.81s1.27-2.52.81-3.91c1.31-.67 2.19-1.91 2.19-3.34zm-11.71 4.2L6.8 12.46l1.41-1.42 2.26 2.26 4.8-5.23 1.47 1.36-6.2 6.77z" style="fill:#1d9bf0;fill-opacity:1" />
									</g>
								</svg>
							<?php endif; ?>
						</div>
						<div class="nebotheme-tw-timeline__person__screen-name">
							<a <?php nebotheme_render_attrs( $attrs_user_link ); ?>>
								@<?php echo esc_html( $twitter['user']['screen_name'] ); ?>
							</a>
						</div>
					</div>
				</div>
				<?php if ( $instance['show_user_counters'] ) : ?>
					<div class="nebotheme-tw-timeline__counters">
						<div class="nebotheme-tw-timeline__counter">
							<a rel="nofollow" target="_blank" href="<?php echo esc_url( $twitter['user']['url'] ); ?>/following">
								<?php
									echo esc_html( Utils::get_number_shorten( $twitter['user']['following'] ) ) . ' ';
									esc_html_e( 'Following', 'yaffo' );
								?>
							</a>
						</div>
						<div class="nebotheme-tw-timeline__counter">
							<a rel="nofollow" target="_blank" href="<?php echo esc_url( $twitter['user']['url'] ); ?>/followers">
								<?php
									echo esc_html( Utils::get_number_shorten( $twitter['user']['followers'] ) ) . ' ';
									esc_html_e( 'Followers', 'yaffo' );
								?>
							</a>
						</div>
					</div>
				<?php endif; ?>
			</div>
			<div class="nebotheme-tw-timeline__tweets">
				<?php foreach ( $twitter['tweets'] as $tweet ) : ?>
					<div class="nebotheme-tw-timeline__tweet">
						<div class="nebotheme-tw-timeline__tweet__content">
							<?php echo nebotheme_unsanitize( $tweet['text'] ); ?>
						</div>
						<?php if ( $tweet['time'] || $instance['show_tweet_counters'] ) : ?>
							<div class="nebotheme-tw-timeline__tweet__footer">
								<?php if ( $tweet['time'] ) : ?>
									<a rel="nofollow" target="_blank" href="<?php echo esc_url( $tweet['url'] ); ?>" class="nebotheme-tw-timeline__tweet__time nebotheme-icon-twitter">
										<?php echo nebotheme_unsanitize( $tweet['time'] ); ?>
									</a>
								<?php endif; ?>
								<?php if ( $instance['show_tweet_counters'] ) : ?>
									<div class="nebotheme-tw-timeline__tweet__counters">
										<a rel="nofollow" target="_blank" href="<?php echo esc_url( isset( $tweet['retweeted_status']['url'] ) ? $tweet['retweeted_status']['url'] : $tweet['url'] ); ?>/likes" title="<?php echo esc_html( 'Link' ); ?>" class="nebotheme-icon-custom-heart nebotheme-tw-timeline__tweet__counter">
											<?php echo esc_html( Utils::get_number_shorten( $tweet['favorite_count'] ) ); ?>
										</a>
										<a rel="nofollow" target="_blank" href="<?php echo esc_url( isset( $tweet['retweeted_status']['url'] ) ? $tweet['retweeted_status']['url'] : $tweet['url'] ); ?>/retweets" title="<?php echo esc_html( 'Retweet' ); ?>" class="nebotheme-icon-custom-retweet nebotheme-tw-timeline__tweet__counter">
											<?php echo esc_html( Utils::get_number_shorten( $tweet['retweet_count'] ) ); ?>
										</a>
									</div>
								<?php endif; ?>
							</div>
						<?php endif; ?>
					</div>
				<?php endforeach; ?>
			</div>
			<?php if ( $instance['show_follow_button'] ) : ?>
				<div class="nebotheme-tw-timeline__footer">
					<a class="nebotheme-button" rel="nofollow" target="_blank" href="https://twitter.com/intent/follow?screen_name=<?php echo esc_html( $twitter['user']['screen_name'] ); ?>">
						<?php echo esc_html( $instance['follow_button_text'] ); ?>
					</a>
				</div>
			<?php endif; ?>
		</div>
		<?php
	}
}
