<?php
namespace NeboTheme\YaffoCore;

use NeboTheme\Yaffo\Utils;
use NeboTheme\YaffoCore\Traits\Singleton;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Admin class.
 */
class Ajax_Select {
	const AJAX_ACTION_OPTIONS = 'nebotheme_ajax_search_options';
	const AJAX_ACTION_TITLE = 'nebotheme_ajax_search_title';

	const NONCE_ACTION = 'nebotheme_ajax_select';

	const QUERY_TYPE_POST = 'post';
	const QUERY_TYPE_TAX = 'taxonomy';

	use Singleton;

	/**
	 * Echoes select for ajax.
	 */
	public static function render_select( array $settings, array $attrs = [] ) {
		$options = [];

		if ( isset( $attrs['class'] ) && ! is_array( $attrs['class'] ) ) {
			$attrs['class'] = [ $attrs['class'] ];
		}

		if ( empty( $attrs['id'] ) ) {
			$attrs['id'] = uniqid( 'nebothemeAjaxSelect' );
		}

		if ( empty( $attrs['placeholder'] ) ) {
			$attrs['placeholder'] = esc_attr__( 'Enter to search...', 'yaffo-core' );
		}

		$attrs['class'][] = 'nebotheme-ajax-select';

		if (
			! empty( $settings['value'] ) &&
			is_array( $settings['value'] ) &&
			Utils::is_array_sequential( $settings['value'] )
		) {
			$options = static::get_select_title( $settings );
		}

		nebotheme_render_select( $attrs, $options, $settings['value'] );

		if ( empty( $attrs['placeholder'] ) ) {
			$attrs['placeholder'] = esc_html__( 'Enter to search...', 'yaffo-core' );
		}

		?>
		<script data-id="<?php echo esc_attr( $attrs['id'] ); ?>">
			jQuery( () => {
				var $placeholder = jQuery( '<div id="<?php echo esc_attr( $attrs['id'] ); ?>"></div>' ); // __i__ to id
				var id = $placeholder.attr( 'id' );
				var $select = jQuery( `#${ id }` );

				if ( id.match( '__i__' ) ) {
					return;
				}

				$select.nebothemeAjaxSelect( <?php echo wp_json_encode( $settings ); ?> );
			} );
		</script>
		<?php
	}

	/**
	 * The class constructor
	 */
	public function __construct() {
		add_filter( 'nebotheme_config_admin', [ $this, 'config_admin' ] );

		add_action( 'wp_ajax_' . self::AJAX_ACTION_OPTIONS, [ $this, 'ajax_select_options' ] );
	}

	/**
	 * Set localize config for admin.
	 */
	public function config_admin( $config ) {
		$config['ajax_select'] = [
			'nonce' => wp_create_nonce( self::NONCE_ACTION ),
			'action_options' => self::AJAX_ACTION_OPTIONS,
			'action_title' => self::AJAX_ACTION_TITLE,
		];

		return $config;
	}

	/**
	 * Get options data.
	 */
	public function ajax_select_options() {
		if (
			! check_ajax_referer( self::NONCE_ACTION, false, false ) ||
			empty( $_POST['type'] ) ||
			empty( $_POST['sub_type'] ) ||
			empty( $_POST['q'] )
		) {
			wp_send_json_error( '', 400 );
		}

		$results = [];
		$query_search = sanitize_text_field( wp_unslash( $_POST['q'] ) );

		switch ( $_POST['type'] ) {
			case self::QUERY_TYPE_POST:
				$post_type = Utils::get_if_isset( $_POST, 'sub_type', 'post' );
				$query_params = [
					'post_status' => 'publish',
					'post_type' => $post_type,
					's' => $query_search,
					'posts_per_page' > -1,
				];

				if ( 'attachment' === $post_type ) {
					$query_params['post_status'] = 'inherit';
				}

				$query = new \WP_Query( $query_params );

				foreach ( $query->posts as $post ) {
					$text = $post->post_title;

					$results[] = [
						'id' => $post->ID,
						'text' => $text,
					];
				}

				break;

			case self::QUERY_TYPE_TAX:
				$taxonomy = Utils::get_if_isset( $_POST, 'sub_type', 'category' );
				$query_params = [
					'taxonomy' => $taxonomy,
					'search' => $query_search,
					'hide_empty' => false,
				];

				if ( class_exists( 'Polylang' ) ) {
					$query_params['lang'] = '';
				}

				$terms = get_terms( $query_params );

				if ( is_wp_error( $terms ) ) {
					break;
				}

				foreach ( $terms as $term ) {
					$results[] = [
						'id' => $term->term_id,
						'text' => $term->name,
					];
				}

				break;

			default:
				wp_send_json_error( 'Bad Request', 400 );
				break;
		}

		wp_send_json_success( $results );
	}

	/**
	 * Get title.
	 */
	public static function get_select_title( array $settings ) {
		$results = [];
		$ids = (array) $settings['value'];

		switch ( $settings['type'] ) {
			case self::QUERY_TYPE_POST:
				$post_type = Utils::get_if_isset( $settings, 'sub_type', 'post' );
				$query_params = [
					'order' => 'ASC',
					'post_status' => 'publish',
					'post_type' => $post_type,
					'posts_per_page' => -1,
					'orderby' => 'post__in',
					'post__in' => $ids,
				];
				$query = new \WP_Query( $query_params );

				foreach ( $query->posts as $post ) {
					$results[ $post->ID ] = $post->post_title;
				}

				break;

			case self::QUERY_TYPE_TAX:
				$taxonomy = Utils::get_if_isset( $settings, 'sub_type', 'category' );
				$terms_args = [
					'hide_empty' => false,
					'include' => $ids,
					'orderby' => 'include',
					'taxonomy' => [ $taxonomy ],
				];

				if ( class_exists( 'Polylang' ) ) {
					$terms_args['lang'] = '';
				}

				$terms = get_terms( $terms_args );

				if ( is_wp_error( $terms ) ) {
					break;
				}

				foreach ( $terms as $term ) {
					$results[ $term->term_id ] = $term->name;
				}

				if ( count( $results ) !== count( $ids ) ) {
					$taxonomy_features = get_taxonomy( $taxonomy );

					foreach ( $ids as $id ) {
						if ( isset( $results[ $id ] ) ) {
							continue;
						}

						$results[ $id ] = sprintf(
							/* translators: term id, taxonomy name */
							esc_html__( '#%1$s | Broken %2$s', 'yaffo-core' ),
							$id,
							$taxonomy_features->labels->singular_name
						);
					}
				}

				break;

			default:
				throw new \Exception( 'Not found type.' );
		}

		return $results;
	}
}
