<?php
/**
 * Custom Sidebar
 *
 * @package Yaffo
 */

namespace NeboTheme\YaffoCore;

use NeboTheme\YaffoCore\Traits\Singleton;
use NeboTheme\Yaffo\Admin\Meta_Boxes as MetaBoxesTheme;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Custom Sidebar class.
 */
final class Custom_Sidebars {
	use Singleton;

	const PM_SIDEBAR_ID = 'sidebar_id';
	const MENU_SLUG = 'nebotheme';
	const TAXONOMY_NAME = 'nebotheme_sidebar';

	/**
	 * Custom sidebars details.
	 *
	 * @var array
	 */
	private $sidebars = [];

	/**
	 * Get placeholder text for description field.
	 */
	public static function get_description_placeholder() {
		return esc_html__( 'This sidebar can be selected in single open post or page.', 'yaffo-core' );
	}

	/**
	 * The class constructor
	 */
	protected function __construct() {
		$this->add_hooks();
		$this->register_taxonomy();
	}

	/**
	 * Add hooks.
	 */
	private function add_hooks() {
		add_action( 'admin_menu', [ $this, 'register_menu' ] );
		add_action( 'admin_footer', [ $this, 'admin_footer' ] );
		add_action( 'admin_head', [ $this, 'admin_head' ] );
		add_action( 'admin_head', [ $this, 'submenu_highlight' ] );

		add_filter( 'manage_edit-' . self::TAXONOMY_NAME . '_columns', [ $this, 'manage_columns' ] );
		add_filter( 'manage_' . self::TAXONOMY_NAME . '_custom_column', [ $this, 'manage_custom_columns' ], 10, 3 );
		add_filter( 'manage_edit-' . self::TAXONOMY_NAME . '_sortable_columns', [ $this, 'sortable_columns' ] );
		add_filter( 'nebotheme_sidebars_list', [ $this, 'register_sidebars' ] );
		add_filter( 'nebotheme_meta_boxes_fields', [ $this, 'get_meta_boxes_fields' ], 9999 );
		add_filter( 'nebotheme_current_sidebar', [ $this, 'current_sidebar' ] );
	}

	/**
	 * Add menu to dashboard.
	 */
	public function register_menu() {
		add_submenu_page(
			self::MENU_SLUG,
			esc_html__( 'Custom Sidebars', 'yaffo-core' ),
			esc_html__( 'Custom Sidebars', 'yaffo-core' ),
			'manage_options',
			$this->get_slug()
		);
	}

	/**
	 * Get slug to page menu.
	 */
	public function get_slug() {
		return 'edit-tags.php?taxonomy=' . self::TAXONOMY_NAME;
	}

	/**
	 * Register for custom sidebar.
	 */
	private function register_taxonomy() {
		$args = [
			'capabilities' => [ 'manage_options' ],
			'hierarchical' => false,
			'labels' => [
				'add_new_item' => esc_html__( 'Add New Sidebar', 'yaffo-core' ),
				'all_items' => esc_html__( 'All Sidebars', 'yaffo-core' ),
				'edit_item' => esc_html__( 'Edit Sidebar', 'yaffo-core' ),
				'menu_name' => esc_html_x( 'Sidebars', 'Admin menu name', 'yaffo-core' ),
				'name' => esc_html__( 'Custom Sidebars', 'yaffo-core' ),
				'new_item_name' => esc_html__( 'New Sidebar Name', 'yaffo-core' ),
				'parent_item_colon' => esc_html__( 'Parent Sidebar', 'yaffo-core' ) . ': ',
				'parent_item' => esc_html__( 'Parent Sidebar', 'yaffo-core' ),
				'search_items' => esc_html__( 'Search Sidebars', 'yaffo-core' ),
				'singular_name' => esc_html__( 'Sidebar', 'yaffo-core' ),
				'update_item' => esc_html__( 'Update Sidebar', 'yaffo-core' ),
			],
			'public' => false,
			'query_var' => false,
			'rewrite' => false,
			'show_in_nav_menus' => false,
			'show_ui' => true,
		];

		register_taxonomy( self::TAXONOMY_NAME, [], $args );
	}

	/**
	 * Get sidebars details.
	 */
	public function get_sidebars() {
		if ( empty( $this->sidebars ) ) {
			$sidebar_terms = get_terms(
				self::TAXONOMY_NAME,
				[
					'hide_empty' => false,
				]
			);

			foreach ( $sidebar_terms as $sidebar_term ) {
				$sidebar_id = static::get_sidebar_id_by_name( $sidebar_term->name );

				if ( $sidebar_term->description ) {
					$description = $sidebar_term->description;
				} else {
					$description = self::get_description_placeholder();
				}

				$this->sidebars[ $sidebar_id ] = [
					'name' => $sidebar_term->name,
					'description' => $description,
				];
			}
		}

		return $this->sidebars;
	}

	/**
	 * Get sidebars list.
	 */
	public function get_sidebars_options() {
		$sidebars = $this->get_sidebars();

		return wp_list_pluck( $sidebars, 'name' );
	}

	/**
	 * Check has any sidebars.
	 */
	public function has_sidebars() {
		return ! empty( $this->get_sidebars() );
	}

	/**
	 * Register custom sidebars.
	 */
	public function register_sidebars( array $sidebars ) {
		foreach ( $this->get_sidebars() as $sidebar_id => $sidebar ) {
			$sidebar['name'] = $sidebar['name'] . ' (' . esc_html__( 'custom', 'yaffo-core' ) . ')';
			$sidebars[ $sidebar_id ] = [
				'name' => $sidebar['name'],
				'description' => $sidebar['description'],
			];
		}

		return $sidebars;
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function manage_columns( $columns ) {
		if ( ! empty( $columns ) ) {
			$columns = [
				'cb' => $columns['cb'],
				'name' => $columns['name'],
				'description' => $columns['description'],
				'ID' => esc_html__( 'ID', 'yaffo-core' ),
			];
		}

		return $columns;
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function sortable_columns( $sortable_columns ) {
		$sortable_columns['ID'] = 'ID';

		return $sortable_columns;
	}

	/**
	 * Get sidebar nama width prefix.
	 */
	public static function get_sidebar_id_by_name( $sidebar_name ) {
		$taxonomy_name = self::TAXONOMY_NAME;
		$sidebar_name = strtolower( sanitize_title( $sidebar_name ) );
		$id = "{$taxonomy_name}_custom_{$sidebar_name}";

		return str_replace( '-', '_', $id );
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function manage_custom_columns( $value, $column_name, $term_id ) {
		$term = get_term( $term_id, self::TAXONOMY_NAME );

		switch ( $column_name ) {
			case 'ID':
				$value = static::get_sidebar_id_by_name( $term->name );
				break;
		}

		return $value;

	}

	/**
	 * Get Current Sidebar.
	 */
	public function current_sidebar( $current_sidebar ) {
		if ( is_singular() ) {
			global $wp_registered_sidebars;

			$sidebar_id = MetaBoxesTheme::get_pm_for_frontend( self::PM_SIDEBAR_ID );

			if ( isset( $wp_registered_sidebars[ $sidebar_id ] ) ) {
				$current_sidebar = $sidebar_id;
			}
		}

		return $current_sidebar;
	}

	/**
	 * Get Fields for Meta Boxes
	 */
	public function get_meta_boxes_fields( array $fields ) {
		$custom_sidebars_options = $this->get_sidebars_options();

		if ( empty( $custom_sidebars_options ) ) {
			$sidebar_id_options = [];
		} else {
			$sidebar_id_options = array_merge(
				[
					YAFFO_VALUE_NEGATIVE => esc_html__( 'Default Sidebar', 'yaffo-core' ),
				],
				$custom_sidebars_options
			);
		}

		$fields[ self::PM_SIDEBAR_ID ] = [
			'label' => esc_html__( 'Choose Custom Sidebar', 'yaffo-core' ),
			'type' => empty( $sidebar_id_options ) ? MetaBoxesTheme::CONTROL_HIDDEN : MetaBoxesTheme::CONTROL_SELECT,
			'tab' => 'layout',
			'options' => $sidebar_id_options,
			'condition' => [
				'sidebar_layout!' => YAFFO_VALUE_NEGATIVE,
			],
		];

		return $fields;
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function submenu_highlight() {
		global $submenu_file, $parent_file;

		if ( $this->get_slug() === $submenu_file ) {
			$parent_file = self::MENU_SLUG; // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		}
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function is_current_screen() {
		$current_screen = get_current_screen();
		$screen_id = 'edit-' . self::TAXONOMY_NAME;

		return $current_screen && $screen_id === $current_screen->id;
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function admin_footer() {
		if ( ! $this->is_current_screen() ) {
			return;
		}

		?>
		<script type="text/javascript">
			var $wraps = jQuery( '#addtag, #edittag' );
			var $postsFilter = jQuery( '#posts-filter' );
			var $description = jQuery( '[name="description"]' );

			$wraps
				.find( 'tr.form-field.term-name-wrap p, div.form-field.term-name-wrap > p' )
				.text( '<?php esc_html_e( 'The name of the widgets area', 'yaffo-core' ); ?>' )
				.css( 'opacity', '1' );

			$wraps
				.find( 'tr.form-field.term-description-wrap p, div.form-field.term-description-wrap > p' )
				.text( '<?php esc_html_e( 'The description of the widgets area (opt.)', 'yaffo-core' ); ?>' )
				.css( 'opacity', '1' );

			$postsFilter
				.find( '.no-items .colspanchange' )
				.text( '<?php esc_html_e( 'No Custom Sidebars found.', 'yaffo-core' ); ?>' );

			$description.attr( 'placeholder', '<?php echo esc_attr( self::get_description_placeholder() ); ?>' );
		</script>
		<?php
	}

	/**
	 * Customization page for custom sidebar.
	 */
	public function admin_head() {
		if ( ! $this->is_current_screen() ) {
			return;
		}
		?>

		<style type="text/css">
			#addtag div.form-field.term-description-wrap > p,
			#addtag div.form-field.term-name-wrap > p,
			#edittag tr.form-field.term-description-wrap p,
			#edittag tr.form-field.term-name-wrap p {
				opacity: 0;
			}

			#addtag div.form-field.term-slug-wrap,
			#edittag tr.form-field.term-slug-wrap,
			#the-list tr.inline-editor .inline-edit-col label:last-child {
				display: none;
			}
		</style>
		<?php
	}
}
