<?php
namespace NeboTheme\YaffoCore;

use NeboTheme\Yaffo\Customize\Customize_Manager;
use NeboTheme\YaffoCore\Traits\Singleton;
use NeboTheme\Yaffo\Blog_Main;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

final class Lazy_Load {
	use Singleton;

	const CSS_CLASSES = 'nebotheme-lazyload lazyload';
	const CUSTOMIZE_FIELD_LAZYLOAD = 'lazyload_nebotheme';
	const PM_NAME_PLACEHOLDER = '_nebotheme_placeholder';

	/**
	 * Placeholder by default.
	 *
	 * @var string Placeholder by default.
	 */
	private static $placeholder;

	/**
	 * Check
	 *
	 * @return bool
	 */
	public static function is_enabled() {
		return nebotheme_get_tm( self::CUSTOMIZE_FIELD_LAZYLOAD );
	}

	/**
	 * Lazy_Load constructor.
	 *
	 * Run lazyload methods.
	 */
	public function __construct() {
		self::$placeholder = self::get_image_placeholder( 1, 1, true );

		add_action( 'nebotheme_customize_fields', [ $this, 'customize_add_fields' ] );
		add_filter( 'wp_update_attachment_metadata', [ $this, 'generate_attachment_placeholder' ] );
		add_filter( 'wp_generate_attachment_metadata', [ $this, 'generate_attachment_placeholder' ] );
		add_filter( 'wp_get_attachment_image_attributes', [ __CLASS__, 'get_with_placeholders' ], 10, 3 );

		if ( ! is_admin() ) {
			add_filter( 'the_content', [ __CLASS__, 'content_process_images' ], 200, 1 );
			add_filter( 'get_avatar', [ __CLASS__, 'content_process_images' ], 200, 1 );
			add_filter( 'nebotheme_lazyload_html', [ __CLASS__, 'content_process_images' ], 200, 1 );

			add_action( 'wp_enqueue_scripts', [ __CLASS__, 'add_lazysizes_script' ] );
		}
	}

	/**
	 * Add fields to customize.
	 */
	public function customize_add_fields() {
		Customize_Manager::add_section(
			'lazyload',
			[
				'title' => esc_html__( 'Lazy Load Images', 'yaffo-core' ),
				'icon' => 'dashicons-cover-image',
			]
		);

		Customize_Manager::add_field(
			[
				'type' => 'toggle',
				'label' => esc_html__( 'Enable Lazy Load?', 'yaffo-core' ),
				'settings' => self::CUSTOMIZE_FIELD_LAZYLOAD,
				'description' => esc_html__( 'You can toggle lazy-loading of images here.', 'yaffo-core' ),
				'default' => true,
				'section' => 'lazyload',
			]
		);
	}

	/**
	 * Enqueue scripts.
	 */
	public static function add_lazysizes_script() {
		if ( ! self::is_enabled() ) {
			return;
		}

		$suffix = defined( 'SCRIPT_DEBUG' ) && constant( 'SCRIPT_DEBUG' ) ? '' : '.min';

		// Enqueue script.
		wp_enqueue_script(
			'lazysizes',
			YAFFO_CORE_URL . "/assets/lib/lazysizes/lazysizes{$suffix}.js",
			[],
			'5.3.0',
			false
		);

		// Enqueue script.
		wp_enqueue_script(
			'lazysizes/unveilhooks',
			YAFFO_CORE_URL . "/assets/lib/lazysizes/plugins/unveilhooks/ls.unveilhooks{$suffix}.js",
			[],
			'5.3.0',
			false
		);
	}

	/**
	 * Get image placeholder by default.
	 *
	 * @param array $width Placeholder width.
	 * @param array $height Placeholder height.
	 *
	 * @return string Placeholder image src.
	 */
	protected static function get_image_placeholder( $width = 1, $height = 1 ) {
		$cache_name = sprintf( 'nebotheme_image_placeholder_%s_%s', $width, $height );
		$placeholder_image = get_transient( $cache_name );

		if ( ! $placeholder_image ) {
			$placeholder_code = ob_start();

			$image = imagecreate( $width, $height );
			$background = imagecolorallocatealpha( $image, 0, 0, 255, 127 );

			imagepng( $image, null, 9 );
			imagecolordeallocate( $image, $background );
			imagedestroy( $image );

			$placeholder_code = ob_get_clean();

			$placeholder_image = 'data:image/png;base64,' . base64_encode( $placeholder_code ); // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode

			set_transient( $cache_name, $placeholder_image );
		}

		return $placeholder_image;
	}

	/**
	 * Generate attachment placeholder.
	 *
	 * @param array $metadata Placeholder data.
	 *
	 * @return array Metadata.
	 */
	public function generate_attachment_placeholder( $metadata ) {
		if ( isset( $metadata['width'] ) && isset( $metadata['height'] ) ) {
			$metadata[ self::PM_NAME_PLACEHOLDER ] = self::get_image_placeholder( $metadata['width'], $metadata['height'] );
		}

		if ( isset( $metadata['sizes'] ) ) {
			foreach ( $metadata['sizes'] as &$size ) {
				if ( isset( $size['width'] ) && isset( $size['height'] ) ) {
					$size[ self::PM_NAME_PLACEHOLDER ] = self::get_image_placeholder( $size['width'], $size['height'] );
				}
			}
		}

		return $metadata;
	}

	/**
	 * Get image proportion from src.
	 *
	 * @param string $src Image src.
	 *
	 * @return array Image width and height.
	 */
	public static function get_proportion_from_src( $src ) {
		preg_match( '/-(\d*)x(\d*)\.(.*)$/i', $src, $matches );

		$width_index_group = 1;
		$height_index_group = 2;

		if (
			empty( $matches[ $width_index_group ] ) ||
			empty( $matches[ $height_index_group ] )
		) {
			return false;
		}

		return [
			'width' => (int) $matches[ $width_index_group ],
			'height' => (int) $matches[ $height_index_group ],
		];
	}

	/**
	 * Filter for wp_get_attachment_image_attributes, change image attributes.
	 *
	 * @param array $attr Attributes.
	 * @param mixed $attachment Current image (WP_Post).
	 * @param mixed $size Image size.
	 * @param bool $hw_attrs Height & width string.
	 *
	 * @return array Filtered image sizes.
	 */
	public static function get_with_placeholders( $attr, $attachment, $size, $hw_attrs = false ) {
		if ( ! self::is_enabled() || ! self::check_attr( $attr ) ) {
			return $attr;
		}

		// Init class of image.
		if ( ! isset( $attr['class'] ) ) {
			$attr['class'] = null;
		}

		// Init src of image.
		if ( ! isset( $attr['src'] ) ) {
			$attr['src'] = null;
		}

		$placeholder = self::$placeholder;

		if ( isset( $attachment->ID ) ) {
			$attachment_id = $attachment->ID;
		} elseif ( isset( $attachment['ID'] ) ) {
			$attachment_id = $attachment['ID'];
		} else {
			$attachment_id = null;
		}

		if ( $attachment_id && ! $size ) {
			$size = 'full';
		}

		// Is string.
		if ( is_string( $size ) ) {
			$metadata = get_post_meta( $attachment_id, '_wp_attachment_metadata', true );

			if ( '' !== $metadata ) {
				if ( 'full' === $size || ! isset( $metadata['sizes'][ $size ] ) ) {
					$metadata_size = $metadata;
				} else {
					$metadata_size = $metadata['sizes'][ $size ];
				}

				if ( isset( $metadata_size[ self::PM_NAME_PLACEHOLDER ] ) ) {
					$placeholder = $metadata_size[ self::PM_NAME_PLACEHOLDER ];
				} elseif ( isset( $metadata[ self::PM_NAME_PLACEHOLDER ] ) ) {
					$placeholder = $metadata[ self::PM_NAME_PLACEHOLDER ];
				}

				if ( $hw_attrs ) {
					if ( ! isset( $attr['width'] ) ) {
						$attr['width'] = $metadata_size['width'];
					}

					if ( ! isset( $attr['height'] ) ) {
						$attr['height'] = $metadata_size['height'];
					}
				}
			}
		} elseif ( is_array( $size ) && isset( $size[0] ) && isset( $size[1] ) ) {
			$placeholder = self::get_image_placeholder( $size[0], $size[1] );
		}

		// Lazy Sizes.
		$attr['class'] .= ' ' . self::CSS_CLASSES;

		// Set data-sizes.
		if ( ! isset( $attr['data-sizes'] ) ) {
			$attr['data-sizes'] = 'auto';
		}

		if ( isset( $attr['sizes'] ) ) {
			$attr['data-ls-sizes'] = $attr['sizes'];

			unset( $attr['sizes'] );
		}

		// Set data-src.
		if ( ! isset( $attr['data-src'] ) ) {
			$attr['data-src'] = $attr['src'];
		}

		if ( isset( $attr['srcset'] ) ) {
			$attr['data-srcset'] = $attr['srcset'];

			unset( $attr['srcset'] );
		}

		$attr['src'] = $placeholder;

		return $attr;
	}

	/**
	 * Check if is enabled lazyload.
	 *
	 * @param array $attr Attributes.
	 *
	 * @return bool Checked result.
	 */
	public static function check_attr( $attr = [] ) {

		if (
			( is_admin() && ! Blog_Main::is_ajax() ) ||
			is_feed() ||
			is_preview() ||
			get_query_var( 'print' ) ||
			get_query_var( 'printpage' ) ||
			(
				isset( $attr['class'] ) && preg_match( '/nebotheme-lazyload-disabled|custom-logo/', $attr['class'] )
			) ||
			(
				function_exists( 'is_amp_endpoint' ) && is_amp_endpoint()
			)
		) {
			return false;
		}

		return true;
	}

	/**
	 * Get attachment attributes to object.
	 *
	 * @param array $attr Attributes.
	 *
	 * @return array attachment attributes.
	 */
	public static function attachment_attr_to_object( $attr ) {
		if ( ! isset( $attr['src'] ) ) {
			return;
		}

		// Set ID by class.
		if ( isset( $attr['class'] ) && preg_match( '/wp-image-(\d*)/i', $attr['class'], $match ) ) {
			return [
				'ID' => $match[1],
			];
		}

		// Remove the thumbnail size.
		$src = preg_replace( '~-[0-9]+x[0-9]+(?=\..{2,6})~', '', $attr['src'] );

		// Set ID by src.
		return [
			'ID' => attachment_url_to_postid( $src ),
		];
	}

	/**
	 * Get attachment attributes to size.
	 *
	 * @param array $attr Attributes.
	 * @param string $attachment_id Attachment ID.
	 *
	 * @return array attachment attributes.
	 */
	public static function get_attachment_attr_to_size( $attr, $attachment_id = null ) {
		// Get size by class.
		if ( isset( $attr['class'] ) ) {
			if ( preg_match( '/size-(\S*)/i', $attr['class'], $match ) ) {
				return $match[1];
			}
		}

		if ( ! $attachment_id ) {
			return;
		}

		$metadata = get_post_meta( $attachment_id, '_wp_attachment_metadata', true );

		if ( ! is_array( $metadata ) ) {
			return 'full';
		}

		$proportion = self::get_proportion_from_src( $attr['src'] );
		$find_size_name = [];

		if ( isset( $proportion['width'] ) && isset( $proportion['height'] ) ) {
			$find_size_name[] = [
				$proportion['width'],
				$proportion['height'],
			];
		}

		if ( isset( $attr['width'] ) && isset( $attr['height'] ) ) {
			$find_size_name[] = [
				$attr['width'],
				$attr['height'],
			];
		}

		foreach ( $find_size_name as $find_sizes ) {
			list( $width, $height ) = $find_sizes;

			if ( $width === $metadata['width'] && $height === $metadata['height'] ) {
				return 'full';
			}

			$found_by_width = [];

			foreach ( $metadata['sizes'] as $size_name => $size_metadata ) {
				if ( $width === $size_metadata['width'] ) {
					$found_by_width[ $size_name ] = $size_metadata;
				}
			}

			if ( $found_by_width ) {
				if ( 1 === count( $found_by_width ) ) {
					return array_keys( $found_by_width )[0];
				}

				foreach ( $found_by_width as $size_name => $size_metadata ) {
					if ( $height === $size_metadata['height'] ) {
						return $size_name;
					}
				}
			}
		}
	}

	/**
	 * Get attributes from HTML.
	 *
	 * @param string $html HTML.
	 *
	 * @return array attributes.
	 */
	public static function get_array_attr_from_html( $html ) {
		preg_match_all( '/\s(.*?)="(.*?)"/', $html, $matches );

		$attr_data = array_shift( $matches );

		$attr = [];

		foreach ( $attr_data as $key => $full_data ) {
			$name = $matches[0][ $key ];
			$value = $matches[1][ $key ];

			$attr[ $name ] = $value;
		}

		return $attr;
	}

	/**
	 * Filter images in content.
	 *
	 * @param string $content Content.
	 *
	 * @return string Filtered content.
	 */
	public static function content_process_images( $content ) {
		if ( self::is_enabled() ) {
			// Get all images.
			preg_match_all( '/<img\s+.*?>/', $content, $matches );

			$images = array_shift( $matches );

			// Check exists images.
			if ( $images ) {
				foreach ( $images as $image ) {

					// Ignore init lazyload.
					if ( preg_match( '/nebotheme-lazyload/', $image ) ) {
						continue;
					}

					// Get Attributes for the image markup.
					$attr = self::get_array_attr_from_html( $image );

					if ( empty( $attr ) ) {
						continue;
					}

					/* Process image */
					$attachment = self::attachment_attr_to_object( $attr );

					if ( $attachment && isset( $attachment['ID'] ) && $attachment['ID'] ) {
						$size = self::get_attachment_attr_to_size( $attr, $attachment['ID'] );
					} else {
						$size = false;
						$attachment = false;
					}

					$attr = self::get_with_placeholders( $attr, $attachment, $size, true );

					// Variables for new image.
					$new_image = '<img [attr]>';

					// Create new image based on new attributes.
					$new_image = str_replace( '[attr]', nebotheme_get_render_attrs( $attr ), $new_image );

					// Update content.
					$content = str_replace( $image, $new_image, $content );
				}
			}
		}

		return $content;
	}


	/**
	 * Echoes animation.
	 */
	public static function render_animation() {
		echo '<span class="nebotheme-lazyload__animation"></span>';
	}
}
