<?php
/**
 * Class widget base
 *
 * @package Yaffo
 */

namespace NeboTheme\YaffoCore;

use NeboTheme\Yaffo\Sidebars\Sidebar_Manager;
use NeboTheme\Yaffo\Utils;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Class widget
 */
abstract class Widget_Base extends \WP_Widget {

	/**
	 * Undocumented function
	 *
	 * @param string $sidebar_id
	 *
	 * @return bool
	 */
	public static function is_ready( $sidebar_id = null ) {
		if ( ! $sidebar_id ) {
			$sidebar_id = Sidebar_Manager::get_current_sidebar();
		}

		if ( $sidebar_id ) {
			$sidebars_widgets = wp_get_sidebars_widgets();

			if ( $sidebars_widgets ) {
				$sidebar_widgets = array_map( '_get_widget_id_base', Utils::get_if_isset( $sidebars_widgets, $sidebar_id, [] ) );

				return in_array( static::get_id(), $sidebar_widgets, true );
			}
		}

		return false;
	}

	/**
	 * Check if active
	 *
	 * @return bool
	 */
	public static function is_active() {
		return true;
	}

	/**
	 * The class constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct( static::get_id(), $this->get_label(), $this->get_widget_options() );
	}

	/**
	 * Get id with prefix
	 */
	public static function get_id() {
		return 'nebotheme_' . static::get_name();
	}

	/**
	 * Updates a particular instance of a widget.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = $new_instance;

		foreach ( $new_instance as $name => $value ) {
			if ( ! $value && '' !== $value && ! is_array( $value ) ) {
				$value = '';
			}

			if ( 'title' === $name ) {
				$value = wp_strip_all_tags( $value );
			} elseif ( $value && ! is_numeric( $value ) && 'string' === gettype( $value ) ) {
				$value = sanitize_text_field( $value );
			}

			$instance[ $name ] = $value;
		}

		return $instance;
	}

	/**
	 * Echoes the widget content.
	 */
	public function widget( $args, $instance ) {
		echo wp_kses_post( $args['before_widget'] );

		$this->render_title( $args, $instance );

		echo '<div class="nebotheme-wp-widget nebotheme-wp-widget-' . esc_attr( str_replace( '_', '-', $this->get_name() ) ) . '">';

		$this->render( $args, $instance );

		echo '</div>';

		echo wp_kses_post( $args['after_widget'] );
	}

	/**
	 * Echoes image by settings
	 */
	public function render_image( $instance, $name = 'image' ) {
		$image_id = Utils::get_if_isset( $instance, $name, '' );
		$image_size = Utils::get_if_isset( $instance, "{$name}_size", 'full' );

		if ( ! $image_id ) {
			return;
		}

		nebotheme_render_attachment( [
			'id' => $image_id,
			'size' => $image_size,
		] );
	}

	/**
	 * Echoes image field by settings
	 */
	public function render_field_image( array $instance, $name = 'image', $label = '' ) {
		$name_size = "{$name}_size";
		$image_id = Utils::get_if_isset( $instance, $name, '' );
		$has_image = $image_id && wp_attachment_is_image( $image_id );
		$image_size = Utils::get_if_isset( $instance, $name_size, 'full' );
		$attrs = [
			'class' => [
				'button-upload-image',
			],
		];

		if ( $has_image ) {
			$attrs['class'][] = 'button-upload-image--exist';
		}
		?>
		<div <?php echo nebotheme_render_attrs( $attrs ); ?>>
			<div class="button-upload-image__holder">
				<div class="button-upload-image__holder__outer">
					<a href="#" class="button-upload-image__remove">
						<i class="dashicons dashicons-no-alt"></i>
					</a>
					<button type="button" class="button-upload-image__add" title="<?php echo esc_attr( $label ); ?>"></button>
					<div class="button-upload-image__holder__inner">
						<?php
						if ( $has_image ) {
							echo wp_get_attachment_image( $image_id, $image_size );
						}
						?>
					</div>
				</div>
			</div>
			<button type="button" class="button-upload-image__add">
				<?php
				esc_html_e( 'Add Image', 'yaffo-core' );

				if ( $label ) {
					echo ': ' . esc_html( $label );
				}
				?>
			</button>
			<input type="hidden" id="<?php echo esc_attr( $this->get_field_id( $name ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( $name ) ); ?>" value="<?php echo esc_attr( $image_id ); ?>" class="button-upload-image__id">
			<input type="hidden" id="<?php echo esc_attr( $this->get_field_id( $name_size ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( $name_size ) ); ?>" value="<?php echo esc_attr( $image_size ); ?>" class="button-upload-image__size">
		</div>
		<?php
	}

	/**
	 * Echoes title.
	 */
	protected function render_title( $args, $instance ) {
		if ( empty( $instance['title'] ) ) {
			return;
		}

		$title = apply_filters( 'widget_title', $instance['title'] );

		if ( $title ) {
			echo wp_kses_post( $args['before_title'] . esc_html( $title ) . $args['after_title'] );
		}
	}

	/**
	 * Widget control options.
	 */
	protected function get_widget_options() {
		return [
			'customize_selective_refresh' => true,
			'description' => ' ',
		];
	}

	/**
	 * Widget name.
	 */
	abstract public static function get_name();

	/**
	 * Widget label.
	 */
	abstract public function get_label();

	/**
	 * Echoes the widget content inner.
	 */
	abstract protected function render( $args, $instance );
}
